@@ -17,7 +17,7 @@
  * I just find Python to be very handy for unit testing, that's all.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -39,6 +39,33 @@
  ******************************************************************************
  *
  * $Log: mapcopy.c,v $
+ * Revision 1.40.2.2  2005/06/29 14:30:29  sean
+ * no longer copying layer and join items, as these will be initialized on opening of layer and joining of data (bug 1403)
+ *
+ * Revision 1.40.2.1  2005/06/28 17:01:51  sean
+ * Fixed copy of layer processing directives with use of msLayerGetProcessing and msLayerAddProcessing (bug 1399). Also caught error in the upper bounds of looping over layer joins.
+ *
+ * Revision 1.40  2005/06/14 16:03:33  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.39  2005/04/25 06:41:55  sdlime
+ * Applied Bill's newest gradient patch, more concise in the mapfile and potential to use via MapScript.
+ *
+ * Revision 1.38  2005/04/15 18:44:31  sdlime
+ * Updated msCopyStyle to account for the various item indexes.
+ *
+ * Revision 1.37  2005/04/15 17:50:20  sdlime
+ * Updated style copy function to handle the new gradient parameters (bug 1305).
+ *
+ * Revision 1.36  2005/02/18 03:06:45  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.35  2005/01/26 06:14:53  sdlime
+ * Added style maxwidth/minwidth read/write/copy.
+ *
+ * Revision 1.34  2005/01/26 05:21:19  sdlime
+ * Added support for reading/writing/copying a style width.
+ *
  * Revision 1.33  2004/11/15 19:16:53  sean
  * Do not copy layerinfo, wfslayerinfo, or ogrlayerinfo in msCopyLayer (bug 1055).
  *
@@ -51,7 +78,7 @@
 #include "map.h"
 #include "mapsymbol.h"
 
-MS_CVSID("$Id: mapcopy.c,v 1.33 2004/11/15 19:16:53 sean Exp $")
+MS_CVSID("$Id: mapcopy.c,v 1.40.2.2 2005/06/29 14:30:29 sean Exp $")
 
 #include "mapcopy.h"
 
@@ -173,8 +200,8 @@ int msCopyFontSet(fontSetObj *dst, fontSetObj *src, mapObj *map)
     MS_COPYSTRING(dst->filename, src->filename);
     MS_COPYSTELEM(numfonts);
     if (&(src->fonts)) {
-        //if (!dst->fonts)
-        //    dst->fonts = msCreateHashTable();
+        /* if (!dst->fonts) */
+        /* dst->fonts = msCreateHashTable(); */
         if (msCopyHashTable(&(dst->fonts), &(src->fonts)) != MS_SUCCESS)
             return MS_FAILURE;
     }
@@ -207,16 +234,10 @@ int msCopyExpression(expressionObj *dst, expressionObj *src)
 
 int msCopyJoin(joinObj *dst, joinObj *src)
 {
-    int i;
     MS_COPYSTRING(dst->name, src->name);
 
-    // not sure it makes sense to copy the values (or the items for that matter) since
-    // since they are runtime additions to the mapfile, probably should be NULL with numitems=0
-    MS_COPYSTELEM(numitems);
-    for (i = 0; i < dst->numitems; i++) {
-        MS_COPYSTRING(dst->items[i], src->items[i]);
-        MS_COPYSTRING(dst->values[i], src->values[i]);  
-    }
+    /* makes no sense to copy the values or items since
+       they are runtime additions to the mapfile */
 
     MS_COPYSTRING(dst->table, src->table);
     MS_COPYSTRING(dst->from, src->from);
@@ -233,7 +254,7 @@ int msCopyJoin(joinObj *dst, joinObj *src)
 
     MS_COPYSTELEM(connectiontype);
 
-    // TODO: need to handle joininfo (probably should be just set to NULL)
+    /* TODO: need to handle joininfo (probably should be just set to NULL) */
     dst->joininfo = NULL;
 
     return MS_SUCCESS;
@@ -334,7 +355,7 @@ int msCopyWeb(webObj *dst, webObj *src, mapObj *map)
     MS_COPYSTRING(dst->maxtemplate, src->maxtemplate);
 
     if (&(src->metadata)) {
-        //dst->metadata = msCreateHashTable();
+        /* dst->metadata = msCreateHashTable(); */
         if (msCopyHashTable(&(dst->metadata), &(src->metadata)) != MS_SUCCESS)
             return MS_FAILURE;
     }
@@ -355,24 +376,36 @@ int msCopyStyle(styleObj *dst, styleObj *src)
     MS_COPYCOLOR(&(dst->outlinecolor),&(src->outlinecolor));
     MS_COPYCOLOR(&(dst->backgroundcolor), &(src->backgroundcolor));
    
+    MS_COPYCOLOR(&(dst->mincolor), &(src->mincolor));
+    MS_COPYCOLOR(&(dst->maxcolor), &(src->maxcolor));
+
     /* The macro should protect us but it isn't in this case */
     if (src->symbolname) {
         MS_COPYSTRING(dst->symbolname, src->symbolname);
     }
 
-
     MS_COPYSTELEM(symbol);
     MS_COPYSTELEM(size);
     MS_COPYSTELEM(minsize);
     MS_COPYSTELEM(maxsize);
+    MS_COPYSTELEM(width);
+    MS_COPYSTELEM(minwidth);
+    MS_COPYSTELEM(maxwidth);
     MS_COPYSTELEM(offsetx);
     MS_COPYSTELEM(offsety);
     MS_COPYSTELEM(antialias);
     MS_COPYSTELEM(isachild);
     MS_COPYSTELEM(angle);
+    MS_COPYSTELEM(minvalue);
+    MS_COPYSTELEM(maxvalue);
 
     if (src->angleitem) MS_COPYSTRING(dst->angleitem, src->angleitem);
     if (src->sizeitem) MS_COPYSTRING(dst->sizeitem, src->sizeitem);
+    if (src->rangeitem) MS_COPYSTRING(dst->rangeitem, src->rangeitem);
+
+    MS_COPYSTELEM(angleitemindex);
+    MS_COPYSTELEM(sizeitemindex);
+    MS_COPYSTELEM(rangeitemindex);
 
     return MS_SUCCESS;
 }
@@ -426,7 +459,7 @@ int msCopyClass(classObj *dst, classObj *src, layerObj *layer)
     MS_COPYSTELEM(type);
 
     if (&(src->metadata) != NULL) {
-        //dst->metadata = msCreateHashTable();
+        /* dst->metadata = msCreateHashTable(); */
         msCopyHashTable(&(dst->metadata), &(src->metadata));
     }
 
@@ -466,7 +499,7 @@ int msCopyLabelCacheMember(labelCacheMemberObj *dst,
     MS_COPYSTELEM(tileindex);
     MS_COPYSTELEM(shapeindex);
     MS_COPYPOINT(&(dst->point), &(src->point));
-    //msCopyShape(&(dst->poly), &(src->poly));
+    /* msCopyShape(&(dst->poly), &(src->poly)); */
     MS_COPYSTELEM(status);
 
     return MS_SUCCESS;
@@ -483,7 +516,7 @@ int msCopyMarkerCacheMember(markerCacheMemberObj *dst,
 {  
     MS_COPYSTELEM(id);
   
-    //msCopyShape(&(dst->poly), &(src->poly));
+    /* msCopyShape(&(dst->poly), &(src->poly)); */
     return MS_SUCCESS;
 }
 
@@ -764,16 +797,8 @@ int msCopyLayer(layerObj *dst, layerObj *src)
     MS_COPYSTELEM(connectiontype);
     MS_COPYSTELEM(sameconnection);
 
-    //MS_COPYSTELEM(layerinfo);
-    //MS_COPYSTELEM(ogrlayerinfo); 
-    //MS_COPYSTELEM(wfslayerinfo);
-    MS_COPYSTELEM(numitems);
-
-    for (i = 0; i < dst->numitems; i++) {
-        MS_COPYSTRING(dst->items[i], src->items[i]);
-    }
-
-    MS_COPYSTELEM(iteminfo);
+    /* layerinfo, items, and iteminfo are not copied. these are set when the
+       copied layer is opened */
 
     return_value = msCopyExpression(&(dst->filter), &(src->filter));
     if (return_value != MS_SUCCESS) {
@@ -797,21 +822,22 @@ int msCopyLayer(layerObj *dst, layerObj *src)
     MS_COPYSTELEM(transparency);
     MS_COPYSTELEM(dump);
     MS_COPYSTELEM(debug);
-    MS_COPYSTELEM(numprocessing);
 
-    for (i = 0; i < dst->numprocessing; i++) {
-        MS_COPYSTRING(dst->processing[i], src->processing[i]);
+    /* No need to copy the numprocessing member, as it is incremented by
+       msLayerAddProcessing */
+    for (i = 0; i < src->numprocessing; i++) 
+    {
+        msLayerAddProcessing(dst, msLayerGetProcessing(src, i));
     }
 
     MS_COPYSTELEM(numjoins);
-
-    for (i = 0; i < dst->numprocessing; i++) {
+    for (i = 0; i < dst->numjoins; i++) {
         return_value = msCopyJoin(&(dst->joins[i]), &(src->joins[i]));
         if (return_value != MS_SUCCESS)
             return MS_FAILURE;
     }
 
-		MS_COPYRECT(&(dst->extent), &(src->extent));
+    MS_COPYRECT(&(dst->extent), &(src->extent));
     
     return MS_SUCCESS;
 }
@@ -858,7 +884,7 @@ int msCopyMap(mapObj *dst, mapObj *src)
         return MS_FAILURE;
     }
 
-    //msCopyLabelCache(&(dst->labelcache), &(src->labelcache));
+    /* msCopyLabelCache(&(dst->labelcache), &(src->labelcache)); */
     MS_COPYSTELEM(transparent);
     MS_COPYSTELEM(interlace);
     MS_COPYSTELEM(imagequality);
@@ -895,7 +921,7 @@ int msCopyMap(mapObj *dst, mapObj *src)
         msAppendOutputFormat( dst, 
         msCloneOutputFormat( src->outputformatlist[i]) );
 
-    // set the active output format
+    /* set the active output format */
     MS_COPYSTRING(dst->imagetype, src->imagetype); 
     format = msSelectOutputFormat( dst, dst->imagetype );
     msApplyOutputFormat(&(dst->outputformat), format, MS_NOOVERRIDE, 
