@@ -6,7 +6,7 @@
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
  ******************************************************************************
- * Copyright (c) 1996-2001 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,6 +28,24 @@
  ******************************************************************************
  *
  * $Log: mapdrawgdal.c,v $
+ * Revision 1.45  2005/06/14 16:03:33  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.44  2005/05/06 17:29:30  sdlime
+ * Removed debuging statement from last fix.
+ *
+ * Revision 1.43  2005/05/06 17:19:05  frank
+ * bug 985/1015 - dont classify rasters without expressions
+ *
+ * Revision 1.42  2005/02/18 03:06:45  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.41  2005/01/23 20:40:23  frank
+ * LoadGDALImage() should return int, not CPLErr.
+ *
+ * Revision 1.40  2005/01/19 21:20:13  frank
+ * improve autoscaling - add small amount to scaling range - bug 1168
+ *
  * Revision 1.39  2004/11/16 21:57:49  dan
  * Final pass at updating WMS/WFS client/server interfaces to lookup "ows_*"
  * metadata in addition to default "wms_*"/"wfs_*" metadata (bug 568)
@@ -111,45 +129,6 @@
  *
  * Revision 1.13  2004/01/15 19:49:23  frank
  * ensure geotransform is set on failure of GetGeotransform
- *
- * Revision 1.12  2003/10/29 21:23:03  frank
- * Added special logic to flip coordinate system y axis if the input raster image
- * is ungeoreferenced (transform is 0,1,0,0,0,1) to (0,1,0,ysize,0,-1) effectively
- * making the lower left corner the origin instead of the upper left.
- *
- * Revision 1.11  2003/10/07 14:34:56  frank
- * added (untested) nodata support for greyscale/colormapped images
- *
- * Revision 1.10  2003/09/30 13:08:57  frank
- * ensure only 128 colors are normally used in greyscale mode
- *
- * Revision 1.9  2003/02/28 20:58:41  frank
- * added preliminary support for the COLOR_MATCH_THRESHOLD
- *
- * Revision 1.8  2003/02/26 16:47:43  frank
- * dont crash on files with large color tables
- *
- * Revision 1.7  2003/02/24 21:20:54  frank
- * Added RAW_WINDOW support for use my mapresample.c
- *
- * Revision 1.6  2003/01/31 18:57:12  frank
- * fixed offsite support from pseudocolored or greyscale input: bug 274
- *
- * Revision 1.5  2003/01/21 05:55:05  frank
- * avoid core dumping when classifying 24bit image, fixes colors - bug 270
- *
- * Revision 1.4  2003/01/10 15:10:58  frank
- * fixed a few bugs in support for classified rasters
- *
- * Revision 1.3  2002/11/21 19:51:03  frank
- * avoid warnings
- *
- * Revision 1.2  2002/11/20 05:27:38  frank
- * initial 24 to 8 bit dithering
- *
- * Revision 1.1  2002/11/19 18:30:04  frank
- * New
- *
  */
 
 #include <assert.h>
@@ -157,7 +136,7 @@
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapdrawgdal.c,v 1.39 2004/11/16 21:57:49 dan Exp $")
+MS_CVSID("$Id: mapdrawgdal.c,v 1.45 2005/06/14 16:03:33 dan Exp $")
 
 extern int InvGeoTransform( double *gt_in, double *gt_out );
 
@@ -177,7 +156,7 @@ extern int InvGeoTransform( double *gt_in, double *gt_out );
 #include "gdal_alg.h"
 #endif
 
-static CPLErr 
+static int
 LoadGDALImage( GDALRasterBandH hBand, int iColorIndex, 
                layerObj *layer, 
                int src_xoff, int src_yoff, int src_xsize, int src_ysize, 
@@ -212,9 +191,9 @@ static int allocColorCube(mapObj *map, gdImagePtr img, int *panColorCube);
 
 #define RED_LEVELS 5
 #define RED_DIV 52
-//#define GREEN_LEVELS 7
+/* #define GREEN_LEVELS 7 */
 #define GREEN_LEVELS 5
-//#define GREEN_DIV 37
+/* #define GREEN_DIV 37 */
 #define GREEN_DIV 52
 #define BLUE_LEVELS 5
 #define BLUE_DIV 52
@@ -240,7 +219,7 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
   rectObj copyRect, mapRect;
   unsigned char *pabyRaw1=NULL, *pabyRaw2=NULL, *pabyRaw3=NULL, 
                 *pabyRawAlpha = NULL;
-  int truecolor = FALSE;
+  int truecolor = FALSE, classified = FALSE;
   int red_band=0, green_band=0, blue_band=0, alpha_band=0, cmt=0;
   gdImagePtr gdImg = NULL;
   GDALDatasetH hDS = hDSVoid;
@@ -257,7 +236,7 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
 /*      and then dumped into the SWF or the PDF file.                   */
 /* -------------------------------------------------------------------- */
   
-  //if( MS_RENDERER_GD(map->outputformat) )
+  /* if( MS_RENDERER_GD(map->outputformat) ) */
   if( MS_RENDERER_GD(image->format) )
   {
       gdImg = image->img.gd;
@@ -417,6 +396,19 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
           src_xoff, src_yoff, src_xsize, src_ysize, 
           dst_xoff, dst_yoff, dst_xsize, dst_ysize );
   }
+  
+  /*
+   * Is this image classified?  We consider it classified if there are
+   * classes with an expression string.  
+   */
+  for( i = 0; i < layer->numclasses; i++ )
+  {
+      if( layer->class[i].expression.string != NULL )
+      {
+          classified = TRUE;
+          break;
+      }
+  }
 
   /*
    * Set up the band selection.  We look for a BANDS directive in the 
@@ -445,7 +437,7 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
           alpha_band = 2;
       
       hBand1 = GDALGetRasterBand( hDS, red_band );
-      if( layer->numclasses != 0 
+      if( classified 
           || GDALGetRasterColorTable( hBand1 ) != NULL )
       {
           alpha_band = 0;
@@ -528,7 +520,7 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
    * is sufficiently different than the normal mechanism of loading
    * into an 8bit buffer, that we isolate it into it's own subfunction.
    */
-  if(layer->numclasses > 0 && gdImg 
+  if( classified && gdImg 
      && hBand1 != NULL && GDALGetRasterDataType( hBand1 ) != GDT_Byte ) 
   {
       return msDrawRasterLayerGDAL_16BitClassification( 
@@ -608,7 +600,7 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
    * class colors are provided by the MAP file, or where we use the native
    * color table.
    */
-  if(layer->numclasses > 0 && gdImg ) {
+  if( classified && gdImg ) {
     int c, color_count;
 
     cmap_set = TRUE;
@@ -1029,7 +1021,7 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
 /*      buffer.  The processing options include scaling.                */
 /************************************************************************/
 
-static CPLErr 
+static int
 LoadGDALImage( GDALRasterBandH hBand, int iColorIndex,  layerObj *layer, 
                int src_xoff, int src_yoff, int src_xsize, int src_ysize, 
                GByte *pabyBuffer, int dst_xsize, int dst_ysize )
@@ -1623,8 +1615,6 @@ msDrawRasterLayerGDAL_16BitClassification(
 
 /* -------------------------------------------------------------------- */
 /*      Scan for absolute min/max of this block.                        */
-/*                                                                      */
-/*      TODO: Ignore "nodata".                                          */
 /* -------------------------------------------------------------------- */
     bGotFirstValue = FALSE;
     
@@ -1698,8 +1688,9 @@ msDrawRasterLayerGDAL_16BitClassification(
 /* -------------------------------------------------------------------- */
     else if( dfScaleMin == 0.0 && dfScaleMax == 0.0 )
     {
-        dfScaleMin = fDataMin;
-        dfScaleMax = fDataMax;
+        double dfEpsilon = (fDataMax - fDataMin) / (65536*2);
+        dfScaleMin = fDataMin - dfEpsilon;
+        dfScaleMax = fDataMax + dfEpsilon;
     }
 
 /* -------------------------------------------------------------------- */
@@ -1786,7 +1777,9 @@ msDrawRasterLayerGDAL_16BitClassification(
              * Skip nodata pixels ... no processing.
              */
             if( bGotNoData && fRawValue == fNoDataValue )
+            {
                 continue;
+            }
             
             /*
              * The funny +1/-1 is to avoid odd rounding around zero.
@@ -1795,7 +1788,9 @@ msDrawRasterLayerGDAL_16BitClassification(
             iMapIndex = (int) ((fRawValue - dfScaleMin) * dfScaleRatio+1)-1;
 
             if( iMapIndex >= nBucketCount || iMapIndex < 0 )
+            {
                 continue;
+            }
 
             result = cmap[iMapIndex];
             if( result == -1 )
