@@ -5,7 +5,7 @@
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,36 @@
  ******************************************************************************
  *
  * $Log: maperror.c,v $
+ * Revision 1.68  2005/06/14 16:03:33  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.67  2005/04/07 21:51:22  frank
+ * added SUPPORTS=THREADS to version info
+ *
+ * Revision 1.66  2005/04/07 17:23:16  assefa
+ * Remove #ifdef USE_SVG. It was added during development.
+ *
+ * Revision 1.65  2005/04/07 13:46:46  frank
+ * added report of SVG support
+ *
+ * Revision 1.64  2005/02/22 07:40:27  sdlime
+ * A bunch of updates to GEOS integration. Can move many primatives between MapServer and GEOS, still need to do collections (e.g. multi-point/line/polygon). Added buffer method to mapscript (mapscript/shape.i).
+ *
+ * Revision 1.63  2005/02/18 03:06:45  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.62  2005/02/02 02:21:50  sdlime
+ * Applied Jerry/Dan patch for bug 1194.
+ *
+ * Revision 1.61  2005/01/28 06:16:53  sdlime
+ * Applied patch to make function prototypes ANSI C compliant. Thanks to Petter Reinholdtsen. This fixes but 1181.
+ *
+ * Revision 1.60  2005/01/26 14:42:13  frank
+ * Removed msWebDebug() ... really this time!
+ *
+ * Revision 1.59  2005/01/07 18:51:09  sdlime
+ * Added MS_GMLERR code.
+ *
  * Revision 1.58  2004/10/21 04:30:55  frank
  * Added standardized headers.  Added MS_CVSID().
  *
@@ -50,7 +80,7 @@
 #include <fcntl.h>
 #endif
 
-MS_CVSID("$Id: maperror.c,v 1.58 2004/10/21 04:30:55 frank Exp $")
+MS_CVSID("$Id: maperror.c,v 1.68 2005/06/14 16:03:33 dan Exp $")
 
 static char *ms_errorCodes[MS_NUMERRORCODES] = {"",
 						"Unable to access file.",
@@ -87,7 +117,8 @@ static char *ms_errorCodes[MS_NUMERRORCODES] = {"",
 						"WCS server error.",
 						"GEOS library error.",
 						"Invalid rectangle.",
-						"Date/time error."
+						"Date/time error.",
+						"GML encoding error."
 };
 
 #ifndef USE_THREAD
@@ -180,7 +211,7 @@ errorObj *msGetErrorObj()
 ** the head is moved to the new errorObj, freeing the head errorObj to receive
 ** the new error information.
 */
-static errorObj *msInsertErrorObj()
+static errorObj *msInsertErrorObj(void)
 {
   errorObj *ms_error;
   ms_error = msGetErrorObj();
@@ -257,9 +288,9 @@ char *msGetErrorString(char *delimiter)
 
   if(!delimiter || !error) return(NULL);
 
-  if((errstr = strdup("")) == NULL) return(NULL); // empty at first
+  if((errstr = strdup("")) == NULL) return(NULL); /* empty at first */
   while(error && error->code != MS_NOERR) {
-    if(error->next && error->next->code != MS_NOERR) // (peek ahead) more errors, use delimiter
+    if(error->next && error->next->code != MS_NOERR) /* (peek ahead) more errors, use delimiter */
       snprintf(errbuf, 255, "%s: %s %s%s", error->routine, ms_errorCodes[error->code], error->message, delimiter);
     else
       snprintf(errbuf, 255, "%s: %s %s", error->routine, ms_errorCodes[error->code], error->message);   
@@ -285,8 +316,10 @@ void msSetError(int code, const char *message_fmt, const char *routine, ...)
 
   if(!routine)
     strcpy(ms_error->routine, "");
-  else
+  else {
     strncpy(ms_error->routine, routine, ROUTINELENGTH);
+    ms_error->routine[ROUTINELENGTH-1] = '\0';
+  }
 
   if(!message_fmt)
     strcpy(ms_error->message, "");
@@ -378,8 +411,8 @@ void msWriteErrorImage(mapObj *map, char *filename, int blank) {
   img = gdImageCreate(width, height);
   color = gdImageColorAllocate(img, map->imagecolor.red, 
                                map->imagecolor.green,
-                               map->imagecolor.blue); // BG color
-  nBlack = gdImageColorAllocate(img, 0,0,0); // Text color
+                               map->imagecolor.blue); /* BG color */
+  nBlack = gdImageColorAllocate(img, 0,0,0); /* Text color */
 
   if (map->outputformat && map->outputformat->transparent)
     gdImageColorTransparent(img, 0);
@@ -389,7 +422,7 @@ void msWriteErrorImage(mapObj *map, char *filename, int blank) {
   nWidthTxt  =  nTextLength * font->w;
   nUsableWidth = width - (nMargin*2);
 
-  // Check to see if it all fits on one line. If not, split the text on several lines.
+  /* Check to see if it all fits on one line. If not, split the text on several lines. */
   if(!blank) {
     if (nWidthTxt > nUsableWidth) {
       nMaxCharsPerLine =  nUsableWidth/font->w;
@@ -432,7 +465,7 @@ void msWriteErrorImage(mapObj *map, char *filename, int blank) {
     }
   }
 
-  // actually write the image
+  /* actually write the image */
   if(!filename) 
       msIO_printf("Content-type: %s%c%c", MS_IMAGE_MIME_TYPE(format), 10,10);
   msSaveImageGD(img, filename, format);
@@ -443,7 +476,7 @@ void msWriteErrorImage(mapObj *map, char *filename, int blank) {
 }
 
 char *msGetVersion() {
-  static char version[512];
+  static char version[1024];
 
   sprintf(version, "MapServer version %s", MS_VERSION);
 
@@ -465,6 +498,7 @@ char *msGetVersion() {
 #ifdef USE_MING_FLASH
   strcat(version, " OUTPUT=SWF");
 #endif
+  strcat(version, " OUTPUT=SVG");
 #ifdef USE_PROJ
   strcat(version, " SUPPORTS=PROJ");
 #endif
@@ -489,6 +523,12 @@ char *msGetVersion() {
 #ifdef USE_FASTCGI
   strcat(version, " SUPPORTS=FASTCGI");
 #endif
+#ifdef USE_THREAD
+  strcat(version, " SUPPORTS=THREADS");
+#endif
+#ifdef USE_GEOS
+  strcat(version, " SUPPORTS=GEOS");
+#endif
 #ifdef USE_TIFF
   strcat(version, " INPUT=TIFF");
 #endif
@@ -521,25 +561,6 @@ char *msGetVersion() {
   return(version);
 }
 
-void msWebDebug( const char * pszFormat, ... )
-{
-#ifndef _WIN32
-    va_list args;
-    struct timeval tv;
-
-    msIO_fprintf(stdout, "Content-type: text/html%c%c",10,10);
-
-    msGettimeofday(&tv, NULL);
-    msIO_fprintf(stdout, "[%s].%ld ", chop(ctime(&(tv.tv_sec))), tv.tv_usec);
-
-    va_start(args, pszFormat);
-    msIO_vfprintf(stdout, pszFormat, args);
-    va_end(args);
-
-    exit(0);
-#endif
-}
-
 void msDebug( const char * pszFormat, ... )
 {
 #ifdef ENABLE_STDERR_DEBUG
@@ -555,9 +576,9 @@ void msDebug( const char * pszFormat, ... )
 #endif
 
 #if !defined(USE_FASTCGI) && !defined(_WIN32)
-    // It seems the FastCGI stuff inserts a timestamp anyways, so 
-    // we might as well skip this one.  And the struct timeval doesn't
-    // appear to exist on win32. 
+    /* It seems the FastCGI stuff inserts a timestamp anyways, so  */
+    /* we might as well skip this one.  And the struct timeval doesn't */
+    /* appear to exist on win32.  */
     {
         struct timeval tv;
         msGettimeofday(&tv, NULL);
