@@ -1,12 +1,12 @@
 /******************************************************************************
- * $Id: maperror.h,v 1.31 2004/10/21 04:30:56 frank Exp $
+ * $Id: maperror.h,v 1.33 2005/06/14 16:03:33 dan Exp $
  *
  * Project:  MapServer
  * Purpose:  Declarations for Error and Debug functions.
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: maperror.h,v $
+ * Revision 1.33  2005/06/14 16:03:33  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.32  2005/01/07 18:51:09  sdlime
+ * Added MS_GMLERR code.
+ *
  * Revision 1.31  2004/10/21 04:30:56  frank
  * Added standardized headers.  Added MS_CVSID().
  *
@@ -76,7 +82,8 @@ extern "C" {
 #define MS_GEOSERR 33
 #define MS_RECTERR 34
 #define MS_TIMEERR 36
-#define MS_NUMERRORCODES 37
+#define MS_GMLERR 37
+#define MS_NUMERRORCODES 38
 
 #define MESSAGELENGTH 2048
 #define ROUTINELENGTH 64
