@@ -204,4 +204,14 @@
 
 #define ENCODING 1162
 
+#define WIDTH 1163
+#define MINWIDTH 1164
+#define MAXWIDTH 1165
+
+ /* Color Range support (was Gradient Support)*/
+#define COLORRANGE 1170
+#define ALPHACOLORRANGE 1171
+#define DATARANGE 1172
+#define RANGEITEM 1173
+
 #endif /* MAPFILE_H */
