@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapgraticule.c,v 1.9 2004/11/15 20:35:02 dan Exp $
+ * $Id: mapgraticule.c,v 1.13 2005/06/08 23:57:43 dan Exp $
  *
  * Name:     mapgraticule.c
  * Project:  MapServer
@@ -29,6 +29,19 @@
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  * $Log: mapgraticule.c,v $
+ * Revision 1.13  2005/06/08 23:57:43  dan
+ * Propagate msGetlabelSize() errors in msGraticuleLayerNextShape() (part of
+ * bug 828)
+ *
+ * Revision 1.12  2005/05/19 05:57:08  sdlime
+ * Added explicit DD format for grid labeling. Only shows the number of degrees, nothing more (bug 1256).
+ *
+ * Revision 1.11  2005/05/19 05:32:00  sdlime
+ * Changed default format for graticule labels to %5.2g from %5.2f which should remove trailing zeros. Partially addresses bug 1256.
+ *
+ * Revision 1.10  2005/02/18 03:06:45  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
  * Revision 1.9  2004/11/15 20:35:02  dan
  * Added msLayerIsOpen() to all vector layer types (bug 1051)
  *
@@ -49,7 +62,7 @@
 #include "map.h"
 #include "mapproject.h"
 
-MS_CVSID("$Id: mapgraticule.c,v 1.9 2004/11/15 20:35:02 dan Exp $")
+MS_CVSID("$Id: mapgraticule.c,v 1.13 2005/06/08 23:57:43 dan Exp $")
 
 /**********************************************************************************************************************
  *
@@ -67,59 +80,60 @@ typedef enum
 	lpDefault	= 0,
 	lpDDMMSS	= 1,
 	lpDDMM,
+	lpDD
 } msLabelProcessingType;
 
 void DefineAxis( int iTickCountTarget, double *Min, double *Max, double *Inc );
-static void _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticulePosition ePosition );
+static int _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticulePosition ePosition );
 static void _FormatLabel( layerObj *pLayer, shapeObj *pShape, double dDataToFormat );
 
 #define MAPGRATICULE_ARC_SUBDIVISION_DEFAULT	(256)
 #define MAPGRATICULE_ARC_MINIMUM				 (16)
-#define MAPGRATICULE_FORMAT_STRING_DEFAULT		"%5.2f"
+#define MAPGRATICULE_FORMAT_STRING_DEFAULT		"%5.2g"
 #define MAPGRATICULE_FORMAT_STRING_DDMMSS		"%3d %02d %02d"
 #define MAPGRATICULE_FORMAT_STRING_DDMM			"%3d %02d"
+#define MAPGRATICULE_FORMAT_STRING_DD                   "%3d"
 
 /**********************************************************************************************************************
  *
  */
 int msGraticuleLayerOpen(layerObj *layer) 
 {
-	graticuleObj 		*pInfo	= (graticuleObj *) layer->layerinfo;
-
-	if( pInfo == NULL )
-		return MS_FAILURE;
-
-	pInfo->dincrementlatitude	=   15.0;
-	pInfo->dincrementlongitude	=   15.0;
-	pInfo->dwhichlatitude		=  -90.0;
-	pInfo->dwhichlongitude		= -180.0;
-	pInfo->bvertical			=    1;
-
-	if( layer->class->label.size == -1 )
-		pInfo->blabelaxes		= 0;
-	else
-		pInfo->blabelaxes		= 1;
-
-	if( pInfo->labelformat == NULL )
-	{
-		pInfo->labelformat	= (char *) malloc( strlen( MAPGRATICULE_FORMAT_STRING_DEFAULT ) + 1 );
-		pInfo->ilabeltype		= (int)    lpDefault;
-		strcpy( pInfo->labelformat, MAPGRATICULE_FORMAT_STRING_DEFAULT );
-	}
-	else if( strcmp( pInfo->labelformat, "DDMMSS" )	== 0 )
-	{
-		pInfo->labelformat	= (char *) malloc( strlen( MAPGRATICULE_FORMAT_STRING_DDMMSS ) + 1 );
-		pInfo->ilabeltype		= (int)    lpDDMMSS;
-		strcpy( pInfo->labelformat, MAPGRATICULE_FORMAT_STRING_DDMMSS );
-	}
-	else if( strcmp( pInfo->labelformat, "DDMM" )	== 0 )
-	{
-		pInfo->labelformat	= (char *) malloc( strlen( MAPGRATICULE_FORMAT_STRING_DDMM ) + 1 );
-		pInfo->ilabeltype		= (int)    lpDDMM;
-		strcpy( pInfo->labelformat, MAPGRATICULE_FORMAT_STRING_DDMM );
-	}
-
-	return MS_SUCCESS;
+  graticuleObj *pInfo = (graticuleObj *) layer->layerinfo;
+  
+  if( pInfo == NULL )
+    return MS_FAILURE;
+  
+  pInfo->dincrementlatitude = 15.0;
+  pInfo->dincrementlongitude = 15.0;
+  pInfo->dwhichlatitude = -90.0;
+  pInfo->dwhichlongitude = -180.0;
+  pInfo->bvertical = 1;
+  
+  if( layer->class->label.size == -1 )
+    pInfo->blabelaxes = 0;
+  else
+    pInfo->blabelaxes = 1;
+  
+  if( pInfo->labelformat == NULL ) {
+    pInfo->labelformat = (char *) malloc( strlen( MAPGRATICULE_FORMAT_STRING_DEFAULT ) + 1 );
+    pInfo->ilabeltype = (int) lpDefault;
+    strcpy( pInfo->labelformat, MAPGRATICULE_FORMAT_STRING_DEFAULT );
+  } else if( strcmp( pInfo->labelformat, "DDMMSS" ) == 0 ) {
+    pInfo->labelformat = (char *) malloc( strlen( MAPGRATICULE_FORMAT_STRING_DDMMSS ) + 1 );
+    pInfo->ilabeltype = (int) lpDDMMSS;
+    strcpy( pInfo->labelformat, MAPGRATICULE_FORMAT_STRING_DDMMSS );
+  } else if( strcmp( pInfo->labelformat, "DDMM" )	== 0 ) {
+    pInfo->labelformat = (char *) malloc( strlen( MAPGRATICULE_FORMAT_STRING_DDMM ) + 1 );
+    pInfo->ilabeltype = (int) lpDDMM;
+    strcpy( pInfo->labelformat, MAPGRATICULE_FORMAT_STRING_DDMM );
+  } else if( strcmp( pInfo->labelformat, "DDMM" )   == 0 ) {
+    pInfo->labelformat = (char *) malloc( strlen( MAPGRATICULE_FORMAT_STRING_DD ) + 1 );
+    pInfo->ilabeltype = (int) lpDD;
+    strcpy( pInfo->labelformat, MAPGRATICULE_FORMAT_STRING_DD );
+  }
+  
+  return MS_SUCCESS;
 }
 
 /**********************************************************************************************************************
@@ -127,10 +141,10 @@ int msGraticuleLayerOpen(layerObj *layer)
  */
 int msGraticuleLayerIsOpen(layerObj *layer)
 {
-    if (layer->layerinfo)
-        return MS_TRUE;
-
-    return MS_FALSE;
+  if (layer->layerinfo)
+    return MS_TRUE;
+  
+  return MS_FALSE;
 }
 
 
@@ -139,27 +153,24 @@ int msGraticuleLayerIsOpen(layerObj *layer)
  */
 int msGraticuleLayerClose(layerObj *layer)
 {
-	graticuleObj 		*pInfo			= (graticuleObj *) layer->layerinfo;
+  graticuleObj *pInfo = (graticuleObj *) layer->layerinfo;
+  
+  if( pInfo->labelformat ) {
+    free( pInfo->labelformat );
+    pInfo->labelformat = NULL;
+  }
+	
+  if( pInfo->pboundingpoints ) {
+    free( pInfo->pboundingpoints );
+    pInfo->pboundingpoints = NULL;
+  }
 
-	if( pInfo->labelformat )
-	{
-		free( pInfo->labelformat );
-		pInfo->labelformat			= NULL;
-	}
-
-	if( pInfo->pboundingpoints )
-	{
-		free( pInfo->pboundingpoints );
-		pInfo->pboundingpoints			= NULL;
-	}
-
-	if( pInfo->pboundinglines )
-	{
-		free( pInfo->pboundinglines );
-		pInfo->pboundinglines			= NULL;
-	}
-
-	return MS_SUCCESS;
+  if( pInfo->pboundinglines ) {
+    free( pInfo->pboundinglines );
+    pInfo->pboundinglines = NULL;
+  }
+  
+  return MS_SUCCESS;
 }
 
 /**********************************************************************************************************************
@@ -206,18 +217,18 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect)
 		pInfo->dincrementlongitude	= pInfo->maxincrement;
 		pInfo->dincrementlatitude	= pInfo->maxincrement;
 	}
-//
-//  If using PROJ, project rect back into map system, and generate rect corner points in native system.
-//  These lines will be used when generating labels to get correct placement at arc/rect edge intersections.
-//
+/*
+ * If using PROJ, project rect back into map system, and generate rect corner points in native system.
+ * These lines will be used when generating labels to get correct placement at arc/rect edge intersections.
+ */
 	rectMapCoordinates				= layer->map->extent;
 	pInfo->pboundinglines			= (lineObj *)  malloc( sizeof( lineObj )  * 4 );
 	pInfo->pboundingpoints			= (pointObj *) malloc( sizeof( pointObj ) * 8 );
 	
 	{
-//
-//  top
-//
+/*
+ * top
+ */
 		pInfo->pboundinglines[0].numpoints		= 2;
 		pInfo->pboundinglines[0].point			= &pInfo->pboundingpoints[0];
 		pInfo->pboundinglines[0].point[0].x		= rectMapCoordinates.minx;
@@ -230,9 +241,9 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect)
 		  && layer->projection.numargs > 0 )
 			msProjectLine( &layer->map->projection, &layer->projection, &pInfo->pboundinglines[0] );
 #endif
-//
-//  bottom
-//
+/*
+ * bottom
+ */
 		pInfo->pboundinglines[1].numpoints		= 2;
 		pInfo->pboundinglines[1].point			= &pInfo->pboundingpoints[2];
 		pInfo->pboundinglines[1].point[0].x		= rectMapCoordinates.minx;
@@ -245,9 +256,9 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect)
 		  && layer->projection.numargs > 0 )
 			msProjectLine( &layer->map->projection, &layer->projection, &pInfo->pboundinglines[1] );
 #endif
-//
-//  left
-//
+/*
+ * left
+ */
 		pInfo->pboundinglines[2].numpoints		= 2;
 		pInfo->pboundinglines[2].point			= &pInfo->pboundingpoints[4];
 		pInfo->pboundinglines[2].point[0].x		= rectMapCoordinates.minx;
@@ -260,9 +271,9 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect)
 		  && layer->projection.numargs > 0 )
 			msProjectLine( &layer->map->projection, &layer->projection, &pInfo->pboundinglines[2] );
 #endif
-//
-//  right
-//
+/*
+ * right
+ */
 		pInfo->pboundinglines[3].numpoints		= 2;
 		pInfo->pboundinglines[3].point			= &pInfo->pboundingpoints[6];
 		pInfo->pboundinglines[3].point[0].x		= rectMapCoordinates.maxx;
@@ -299,9 +310,9 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 	shape->line						= (lineObj *) malloc( sizeof( lineObj ) );
 
 	shape->line->numpoints			= (int) pInfo->maxsubdivides;	
-//
-//  Subdivide and draw current arc, rendering the arc labels first
-//
+/*
+ * Subdivide and draw current arc, rendering the arc labels first
+ */
 	if( pInfo->bvertical )
 	{
 		int		iPointIndex;
@@ -312,7 +323,7 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 		switch( pInfo->ilabelstate )
 		{
 			case 0:
-				if( ! pInfo->blabelaxes )		//  Bottom
+				if( ! pInfo->blabelaxes )		/* Bottom */
 				{
 					pInfo->ilabelstate++;
 					shape->numlines				= 0;
@@ -330,13 +341,14 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 				shape->line->point[1].y			= dStartY + dArcDelta;
 
 				_FormatLabel( layer, shape, shape->line->point[0].x );
-				_AdjustLabelPosition( layer, shape, posBottom );
+				if (_AdjustLabelPosition( layer, shape, posBottom ) != MS_SUCCESS)
+                                    return MS_FAILURE;
 
 				pInfo->ilabelstate++;
 				return MS_SUCCESS;
 
 			case 1:
-				if( ! pInfo->blabelaxes )		//  Top
+				if( ! pInfo->blabelaxes )		/* Top */
 				{
 					pInfo->ilabelstate++;
 					shape->numlines				= 0;
@@ -354,7 +366,8 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 				shape->line->point[1].y			= dStartY;
 
 				_FormatLabel( layer, shape, shape->line->point[0].x );
-				_AdjustLabelPosition( layer, shape, posTop );
+				if (_AdjustLabelPosition( layer, shape, posTop ) != MS_SUCCESS)
+                                    return MS_FAILURE;
 
 				pInfo->ilabelstate++;
 				return MS_SUCCESS;
@@ -393,7 +406,7 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 		switch( pInfo->ilabelstate )
 		{
 			case 0:
-				if( ! pInfo->blabelaxes )			//  Left  side
+				if( ! pInfo->blabelaxes )			/* Left  side */
 				{
 					pInfo->ilabelstate++;
 					shape->numlines				= 0;
@@ -411,13 +424,14 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 				shape->line->point[1].y			= pInfo->dwhichlatitude;
 
 				_FormatLabel( layer, shape, shape->line->point[0].y );
-				_AdjustLabelPosition( layer, shape, posLeft );
+				if (_AdjustLabelPosition( layer, shape, posLeft ) != MS_SUCCESS)
+                                    return MS_FAILURE;
 
 				pInfo->ilabelstate++;
 				return MS_SUCCESS;
 
 			case 1:
-				if( ! pInfo->blabelaxes )			//  Right side
+				if( ! pInfo->blabelaxes )			/* Right side */
 				{
 					pInfo->ilabelstate++;
 					shape->numlines				= 0;
@@ -435,7 +449,8 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 				shape->line->point[1].y			= pInfo->dwhichlatitude;
 
 				_FormatLabel( layer, shape, shape->line->point[0].y );
-				_AdjustLabelPosition( layer, shape, posRight );
+				if (_AdjustLabelPosition( layer, shape, posRight ) != MS_SUCCESS)
+                                    return MS_FAILURE;
 
 				pInfo->ilabelstate++;
 				return MS_SUCCESS;
@@ -463,9 +478,9 @@ int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape)
 				break;
 		}
 	}
-//
-//  Increment and move to next arc
-//
+/*
+ * Increment and move to next arc
+ */
 	pInfo->dwhichlatitude		   += pInfo->dincrementlatitude;
 
 	if( pInfo->dwhichlatitude > pInfo->dendlatitude )
@@ -563,50 +578,50 @@ int msGraticuleLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c, int
  */
 static void _FormatLabel( layerObj *pLayer, shapeObj *pShape, double dDataToFormat )
 {
-	graticuleObj 		*pInfo	= (graticuleObj  *) pLayer->layerinfo;
-	char				cBuffer[32];
-	int					iDegrees, iMinutes;
+  graticuleObj *pInfo = (graticuleObj  *) pLayer->layerinfo;
+  char cBuffer[32];
+  int iDegrees, iMinutes;
 	
-	switch( pInfo->ilabeltype )
-	{
-		case lpDDMMSS:
-			iDegrees		= (int) dDataToFormat;
-			dDataToFormat	= fabs( dDataToFormat - (double) iDegrees );
-			
-			iMinutes		= (int) (dDataToFormat * 60.0);
-			dDataToFormat	= dDataToFormat - (((double) iMinutes) / 60.0);
-			
-			sprintf( cBuffer, pInfo->labelformat, iDegrees, iMinutes, (int) (dDataToFormat * 3600.0) );
-			break;
-			
-		case lpDDMM:
-			iDegrees		= (int) dDataToFormat;
-			dDataToFormat	= fabs( dDataToFormat - (double) iDegrees );
-			
-			sprintf( cBuffer, pInfo->labelformat, iDegrees, (int) (dDataToFormat * 60.0) );
-			break;
-
-		case lpDefault:
-		default:
-			sprintf( cBuffer, pInfo->labelformat, dDataToFormat );
-	}
+  switch( pInfo->ilabeltype ) {
+  case lpDDMMSS:
+    iDegrees = (int) dDataToFormat;
+    dDataToFormat = fabs( dDataToFormat - (double) iDegrees );    
+    iMinutes = (int) (dDataToFormat * 60.0);
+    dDataToFormat = dDataToFormat - (((double) iMinutes) / 60.0);    
+    sprintf( cBuffer, pInfo->labelformat, iDegrees, iMinutes, (int) (dDataToFormat * 3600.0) );
+    break;
+  case lpDDMM:
+    iDegrees = (int) dDataToFormat;
+    dDataToFormat = fabs( dDataToFormat - (double) iDegrees );    
+    sprintf( cBuffer, pInfo->labelformat, iDegrees, (int) (dDataToFormat * 60.0) );
+    break;
+  case lpDD:
+    iDegrees = (int) dDataToFormat;
+    sprintf( cBuffer, pInfo->labelformat, iDegrees);
+    break;  
+  case lpDefault:
+  default:
+    sprintf( cBuffer, pInfo->labelformat, dDataToFormat );
+  }
 	
-	pShape->text			= strdup( cBuffer );
+  pShape->text = strdup( cBuffer );
 }
 
 /**********************************************************************************************************************
  *
  *  Move label position into display area by adjusting underlying shape line.
  */
-static void _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticulePosition ePosition )
+static int _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticulePosition ePosition )
 {
 	graticuleObj 		*pInfo	= (graticuleObj  *) pLayer->layerinfo;
 	rectObj				 rectLabel;
 	pointObj			 ptPoint;
 
-	if( pInfo == NULL
-	  || pShape == NULL )
-		return;
+	if( pInfo == NULL || pShape == NULL )
+    {
+        msSetError(MS_MISCERR, "Assertion failed: Null shape or layerinfo!, ", "_AdjustLabelPosition()");
+		return MS_FAILURE;
+    }
 	
 	ptPoint			= pShape->line->point[0];
 
@@ -619,7 +634,8 @@ static void _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticul
     if( pLayer->transform ) 
 		msTransformShapeToPixel( pShape, pLayer->map->extent, pLayer->map->cellsize );
 
-	msGetLabelSize( pShape->text, &pLayer->class[0].label, &rectLabel, &pLayer->map->fontset, 1.0 );
+	if (msGetLabelSize( pShape->text, &pLayer->class[0].label, &rectLabel, &pLayer->map->fontset, 1.0 ) != 0)
+        return MS_FAILURE;  /* msSetError already called */
 
 	switch( ePosition )
 	{
@@ -667,29 +683,31 @@ static void _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticul
 			pShape->line->point[0].y = ptPoint.y;
 			break;
 	}
+
+    return MS_SUCCESS;
 }
 
 /**********************************************************************************************************************
- **********************************************************************************************************************/
-//  DefineAxes - Copyright (c) 2000, Michael P.D. Bramley.
-//
-//  Permission is granted to use this code without restriction as long as
-//  this copyright notice appears in all source files.
-//
-//  Minor tweaks to incrment calculations - jnovak
-//
+ **********************************************************************************************************************
+ * DefineAxes - Copyright (c) 2000, Michael P.D. Bramley.
+ *
+ * Permission is granted to use this code without restriction as long as
+ * this copyright notice appears in all source files.
+ *
+ * Minor tweaks to incrment calculations - jnovak
+ */
 void DefineAxis( int iTickCountTarget, double *Min, double *Max, double *Inc )
 {
-	// define local variables...
+	/* define local variables... */
 
-	double Test_inc,              // candidate increment value
-		Test_min,              // minimum scale value
-		Test_max,              // maximum scale value
-		Range = *Max - *Min ;  // range of data
+	double Test_inc,              /* candidate increment value */
+		Test_min,              /* minimum scale value */
+		Test_max,              /* maximum scale value */
+		Range = *Max - *Min ;  /* range of data */
 
-	int i = 0 ;                   // counter
+	int i = 0 ;                   /* counter */
 
-	// don't create problems -- solve them
+	/* don't create problems -- solve them */
 
 	if( Range < 0 ) 
 	{
@@ -697,7 +715,7 @@ void DefineAxis( int iTickCountTarget, double *Min, double *Max, double *Inc )
 		return ;
 	}
 
-	// handle special case of repeated values
+	/* handle special case of repeated values */
 
 	else if( Range == 0) 
 	{        
@@ -707,18 +725,18 @@ void DefineAxis( int iTickCountTarget, double *Min, double *Max, double *Inc )
 		return ;
 	}
 
-	// compute candidate for increment
+	/* compute candidate for increment */
 
 	Test_inc = pow( 10.0, ceil( log10( Range/10 ) ) ) ;
 
-	// establish maximum scale value...
+	/* establish maximum scale value... */
 
 	Test_max = ( (long)(*Max / Test_inc)) * Test_inc ;
 
 	if( Test_max < *Max )
 		Test_max += Test_inc ;
 
-	// establish minimum scale value...
+	/* establish minimum scale value... */
 
 	Test_min = Test_max ;
 
@@ -729,15 +747,15 @@ void DefineAxis( int iTickCountTarget, double *Min, double *Max, double *Inc )
 	}
 	while( Test_min > *Min ) ;
 
-	// subtracting small values can screw up the scale limits,
-	// eg: if DefineAxis is called with (min,max)=(0.01, 0.1),
-	// then the calculated scale is 1.0408E17 TO 0.05 BY 0.01.
-	// the following if statment corrects for this...
+	/* subtracting small values can screw up the scale limits, */
+	/* eg: if DefineAxis is called with (min,max)=(0.01, 0.1), */
+	/* then the calculated scale is 1.0408E17 TO 0.05 BY 0.01. */
+	/* the following if statment corrects for this... */
 
-	//if( fabs(Test_min) < 1E-10 )
-	//   Test_min = 0 ;
+	/* if( fabs(Test_min) < 1E-10 ) */
+	/* Test_min = 0 ; */
 
-	// adjust for too few tick marks
+	/* adjust for too few tick marks */
 
 	if( iTickCountTarget <= 0 )
 		iTickCountTarget	= MAPGRATICULE_ARC_MINIMUM;
@@ -757,7 +775,7 @@ void DefineAxis( int iTickCountTarget, double *Min, double *Max, double *Inc )
 			Test_max -= Test_inc ;
 	}
 
-	// pass back axis definition to caller
+	/* pass back axis definition to caller */
 
 	*Min = Test_min ;
 	*Max = Test_max ;
