@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: maphttp.c,v 1.17 2004/10/21 04:30:54 frank Exp $
+ * $Id: maphttp.c,v 1.19 2005/05/13 17:23:34 dan Exp $
  *
  * Project:  MapServer
  * Purpose:  Utility functions to access files via HTTP (requires libcurl)
@@ -27,6 +27,13 @@
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  * $Log: maphttp.c,v $
+ * Revision 1.19  2005/05/13 17:23:34  dan
+ * First pass at properly handling XML exceptions from CONNECTIONTYPE WMS
+ * layers. Still needs some work. (bug 1246)
+ *
+ * Revision 1.18  2005/02/18 03:06:45  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
  * Revision 1.17  2004/10/21 04:30:54  frank
  * Added standardized headers.  Added MS_CVSID().
  *
@@ -92,7 +99,7 @@
 #include "maperror.h"
 #include "mapows.h"
 
-MS_CVSID("$Id: maphttp.c,v 1.17 2004/10/21 04:30:54 frank Exp $")
+MS_CVSID("$Id: maphttp.c,v 1.19 2005/05/13 17:23:34 dan Exp $")
 
 #include <time.h>
 #ifndef _WIN32
@@ -344,7 +351,7 @@ int msHTTPExecuteRequests(httpRequestObj *pasReqInfo, int numRequests,
             fp = fopen(pasReqInfo[i].pszOutputFile, "r");
             if (fp)
             {
-                // File already there, don't download again.
+                /* File already there, don't download again. */
                 if (pasReqInfo[i].debug)
                     msDebug("HTTP request: id=%d, found in cache, skipping.\n",
                             pasReqInfo[i].nLayerId);
@@ -419,7 +426,7 @@ int msHTTPExecuteRequests(httpRequestObj *pasReqInfo, int numRequests,
             curl_easy_setopt(http_handle, CURLOPT_POSTFIELDS, 
                              pasReqInfo[i].pszPostRequest);
             curl_easy_setopt(http_handle, CURLOPT_HTTPHEADER, headers);
-            //curl_slist_free_all(headers); /* free the header list */
+            /* curl_slist_free_all(headers); */ /* free the header list */
         }
         /* Add to multi handle */
         curl_multi_add_handle(multi_handle, http_handle);
@@ -532,7 +539,7 @@ int msHTTPExecuteRequests(httpRequestObj *pasReqInfo, int numRequests,
         psReq = &(pasReqInfo[i]);
 
         if (psReq->nStatus == 242)
-            continue;  // Nothing to do here, this file was in cache already
+            continue;  /* Nothing to do here, this file was in cache already */
 
         if (psReq->fp)
             fclose(psReq->fp);
@@ -544,18 +551,29 @@ int msHTTPExecuteRequests(httpRequestObj *pasReqInfo, int numRequests,
             curl_easy_getinfo(http_handle,
                               CURLINFO_HTTP_CODE, &lVal) == CURLE_OK)
         {
+            char *pszContentType = NULL;
+
             psReq->nStatus = lVal;
+
+            /* Fetch content type of response */
+            if (curl_easy_getinfo(http_handle, 
+                                  CURLINFO_CONTENT_TYPE, 
+                                  &pszContentType) == CURLE_OK &&
+                pszContentType != NULL)
+            {
+                psReq->pszContentType = strdup(pszContentType);
+            }
         }
 
         if (!MS_HTTP_SUCCESS(psReq->nStatus))
         {
-            // Set status to MS_DONE to indicate that transfers were 
-            // completed but may not be succesfull
+            /* Set status to MS_DONE to indicate that transfers were  */
+            /* completed but may not be succesfull */
             nStatus = MS_DONE;
 
             if (psReq->nStatus == -(CURLE_OPERATION_TIMEOUTED))
             {
-                // Timeout isn't a fatal error
+                /* Timeout isn't a fatal error */
                 if (psReq->debug)
                     msDebug("HTTP: TIMEOUT of %d seconds exceeded for %s\n",
                             nTimeout, psReq->pszGetUrl );
@@ -573,7 +591,7 @@ int msHTTPExecuteRequests(httpRequestObj *pasReqInfo, int numRequests,
             }
             else if (psReq->nStatus > 0)
             {
-                // Got an HTTP Error, e.g. 404, etc.
+                /* Got an HTTP Error, e.g. 404, etc. */
 
                 if (psReq->debug)
                     msDebug("HTTP: HTTP GET request failed with status %d (%s)"
@@ -589,7 +607,7 @@ int msHTTPExecuteRequests(httpRequestObj *pasReqInfo, int numRequests,
             }
             else
             {
-                // Got a curl error
+                /* Got a curl error */
 
                 if (psReq->debug)
                     msDebug("HTTP: request failed with curl error "
