@@ -5,7 +5,7 @@
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,24 @@
  ******************************************************************************
  *
  * $Log: maplayer.c,v $
+ * Revision 1.105  2005/06/14 16:03:33  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.104  2005/04/25 06:41:56  sdlime
+ * Applied Bill's newest gradient patch, more concise in the mapfile and potential to use via MapScript.
+ *
+ * Revision 1.103  2005/04/15 17:10:36  sdlime
+ * Applied Bill Benko's patch for bug 1305, gradient support.
+ *
+ * Revision 1.102  2005/02/18 03:06:46  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.101  2005/01/12 21:11:21  frank
+ * removed LABELS_ROTATE_WITH_MAP, rotate labels if angle!=0 or labelangleitem
+ *
+ * Revision 1.100  2005/01/11 00:24:59  frank
+ * added msLayerLabelsRotateWithMap
+ *
  * Revision 1.99  2004/11/16 19:18:43  assefa
  * Make sure that the timestring is complete for pg layers (Bug 837)
  *
@@ -47,7 +65,7 @@
 #include "map.h"
 #include "maptime.h"
 
-MS_CVSID("$Id: maplayer.c,v 1.99 2004/11/16 19:18:43 assefa Exp $")
+MS_CVSID("$Id: maplayer.c,v 1.105 2005/06/14 16:03:33 dan Exp $")
 
 /*
 ** Iteminfo is a layer parameter that holds information necessary to retrieve an individual item for
@@ -70,17 +88,17 @@ static int layerInitItemInfo(layerObj *layer)
       return(MS_FAILURE);
     }
 
-    // iteminfo needs to be a bit more complex, a list of indexes plus the length of the list
+    /* iteminfo needs to be a bit more complex, a list of indexes plus the length of the list */
     layer->iteminfo = (int *) msDBFGetItemIndexes(shpfile->hDBF, layer->items, layer->numitems);
     if(!layer->iteminfo) return(MS_FAILURE);
     return(MS_SUCCESS);
     break;
   case(MS_TILED_SHAPEFILE):
-    // iteminfo needs to be a bit more complex, a list of indexes plus the length of the list
+    /* iteminfo needs to be a bit more complex, a list of indexes plus the length of the list */
     return(msTiledSHPLayerInitItemInfo(layer));
     break;
   case(MS_INLINE):
-    return(MS_SUCCESS); // inline shapes have no items
+    return(MS_SUCCESS); /* inline shapes have no items */
     break;
   case(MS_OGR):
     return(msOGRLayerInitItemInfo(layer));
@@ -171,9 +189,9 @@ int msLayerOpen(layerObj *layer)
 
   switch(layer->connectiontype) {
   case(MS_SHAPEFILE):
-    if(layer->layerinfo) return(MS_SUCCESS); // layer already open
+    if(layer->layerinfo) return(MS_SUCCESS); /* layer already open */
 
-    // allocate space for a shapefileObj using layer->layerinfo	
+    /* allocate space for a shapefileObj using layer->layerinfo	 */
     shpfile = (shapefileObj *) malloc(sizeof(shapefileObj));
     if(!shpfile) {
       msSetError(MS_MEMERR, "Error allocating shapefileObj structure.", "msLayerOpen()");
@@ -196,7 +214,7 @@ int msLayerOpen(layerObj *layer)
     return(msTiledSHPOpenFile(layer));
     break;
   case(MS_INLINE):
-    layer->currentfeature = layer->features; // point to the begining of the feature list
+    layer->currentfeature = layer->features; /* point to the begining of the feature list */
     return(MS_SUCCESS);
     break;
   case(MS_OGR):
@@ -309,7 +327,7 @@ int msLayerWhichShapes(layerObj *layer, rectObj rect)
     status = msSHPWhichShapes(shpfile, rect, layer->debug);
     if(status != MS_SUCCESS) return(status);
 
-    // now apply the maxshapes criteria (NOTE: this ignores the filter so you could get less than maxfeatures)
+    /* now apply the maxshapes criteria (NOTE: this ignores the filter so you could get less than maxfeatures) */
     if(layer->maxfeatures > 0) {
       for(i=0; i<shpfile->numshapes; i++)
         n1 += msGetBit(shpfile->status,i);
@@ -387,12 +405,12 @@ int msLayerNextShape(layerObj *layer, shapeObj *shape)
 
     do {
       i = shpfile->lastshape + 1;
-      while(i<shpfile->numshapes && !msGetBit(shpfile->status,i)) i++; // next "in" shape
+      while(i<shpfile->numshapes && !msGetBit(shpfile->status,i)) i++; /* next "in" shape */
       shpfile->lastshape = i;
 
-      if(i == shpfile->numshapes) return(MS_DONE); // nothing else to read
+      if(i == shpfile->numshapes) return(MS_DONE); /* nothing else to read */
 
-      filter_passed = MS_TRUE;  // By default accept ANY shape
+      filter_passed = MS_TRUE;  /* By default accept ANY shape */
       if(layer->numitems > 0 && layer->iteminfo) {
         values = msDBFGetValueList(shpfile->hDBF, i, layer->iteminfo, layer->numitems);
         if(!values) return(MS_FAILURE);
@@ -401,11 +419,11 @@ int msLayerNextShape(layerObj *layer, shapeObj *shape)
             values = NULL;
         }
       }
-    } while(!filter_passed);  // Loop until both spatial and attribute filters match
+    } while(!filter_passed);  /* Loop until both spatial and attribute filters match */
 
-    msSHPReadShape(shpfile->hSHP, i, shape); // ok to read the data now
+    msSHPReadShape(shpfile->hSHP, i, shape); /* ok to read the data now */
 
-    // skip NULL shapes (apparently valid for shapefiles, at least ArcView doesn't care)
+    /* skip NULL shapes (apparently valid for shapefiles, at least ArcView doesn't care) */
     if(shape->type == MS_SHAPE_NULL) return(msLayerNextShape(layer, shape));
 
     shape->values = values;
@@ -415,7 +433,7 @@ int msLayerNextShape(layerObj *layer, shapeObj *shape)
   case(MS_TILED_SHAPEFILE):
     return(msTiledSHPNextShape(layer, shape));
   case(MS_INLINE):
-    if(!(layer->currentfeature)) return(MS_DONE); // out of features    
+    if(!(layer->currentfeature)) return(MS_DONE); /* out of features     */
     msCopyShape(&(layer->currentfeature->shape), shape);
     layer->currentfeature = layer->currentfeature->next;
     return(MS_SUCCESS);
@@ -446,9 +464,9 @@ int msLayerNextShape(layerObj *layer, shapeObj *shape)
     break;
   }
 
-  // TO DO! This is where dynamic joins will happen. Joined attributes will be
-  // tagged on to the main attributes with the naming scheme [join name].[item name].
-  // We need to leverage the iteminfo (I think) at this point
+  /* TO DO! This is where dynamic joins will happen. Joined attributes will be */
+  /* tagged on to the main attributes with the naming scheme [join name].[item name]. */
+  /* We need to leverage the iteminfo (I think) at this point */
 
   return(MS_FAILURE);
 }
@@ -470,7 +488,7 @@ int msLayerGetShape(layerObj *layer, shapeObj *shape, int tile, long record)
       return(MS_FAILURE);
     }
 
-    // msSHPReadShape *should* return success or failure so we don't have to test here
+    /* msSHPReadShape *should* return success or failure so we don't have to test here */
     if(record < 0 || record >= shpfile->numshapes) {
       msSetError(MS_MISCERR, "Invalid feature id.", "msLayerGetShape()");
       return(MS_FAILURE);
@@ -488,8 +506,8 @@ int msLayerGetShape(layerObj *layer, shapeObj *shape, int tile, long record)
     return(msTiledSHPGetShape(layer, shape, tile, record));
   case(MS_INLINE):
     return msINLINELayerGetShape(layer, shape, record);
-    //msSetError(MS_MISCERR, "Cannot retrieve inline shapes randomly.", "msLayerGetShape()");
-    //return(MS_FAILURE);
+    /* msSetError(MS_MISCERR, "Cannot retrieve inline shapes randomly.", "msLayerGetShape()"); */
+    /* return(MS_FAILURE); */
     break;
   case(MS_OGR):
   case(MS_WFS):
@@ -517,8 +535,8 @@ int msLayerGetShape(layerObj *layer, shapeObj *shape, int tile, long record)
     break;
   }
 
-  // TO DO! This is where dynamic joins will happen. Joined attributes will be
-  // tagged on to the main attributes with the naming scheme [join name].[item name].
+  /* TO DO! This is where dynamic joins will happen. Joined attributes will be */
+  /* tagged on to the main attributes with the naming scheme [join name].[item name]. */
 
   return(MS_FAILURE);
 }
@@ -530,7 +548,7 @@ void msLayerClose(layerObj *layer)
 {
   shapefileObj *shpfile;
 
-  // no need for items once the layer is closed
+  /* no need for items once the layer is closed */
   layerFreeItemInfo(layer);
   if(layer->items) {
     msFreeCharArray(layer->items, layer->numitems);
@@ -541,7 +559,7 @@ void msLayerClose(layerObj *layer)
   switch(layer->connectiontype) {
   case(MS_SHAPEFILE):
     shpfile = layer->layerinfo;
-    if(!shpfile) return; // nothing to do
+    if(!shpfile) return; /* nothing to do */
     msSHPCloseFile(shpfile);
     free(layer->layerinfo);
     layer->layerinfo = NULL;
@@ -568,8 +586,8 @@ void msLayerClose(layerObj *layer)
     msMYGISLayerClose(layer);
     break;
   case(MS_SDE):
-    // using pooled connections for SDE, closed when map file is closed
-    // msSDELayerClose(layer); 
+    /* using pooled connections for SDE, closed when map file is closed */
+    /* msSDELayerClose(layer);  */
     break;
   case(MS_ORACLESPATIAL):
     msOracleSpatialLayerClose(layer);
@@ -594,7 +612,7 @@ int msLayerGetItems(layerObj *layer)
 {
   shapefileObj *shpfile;
 
-  // clean up any previously allocated instances
+  /* clean up any previously allocated instances */
   layerFreeItemInfo(layer);
   if(layer->items) {
     msFreeCharArray(layer->items, layer->numitems);
@@ -621,7 +639,7 @@ int msLayerGetItems(layerObj *layer)
     return(msTiledSHPLayerGetItems(layer));
     break;
   case(MS_INLINE):
-    return(MS_SUCCESS); // inline shapes have no items
+    return(MS_SUCCESS); /* inline shapes have no items */
     break;
   case(MS_OGR):
     return(msOGRLayerGetItems(layer));
@@ -651,7 +669,7 @@ int msLayerGetItems(layerObj *layer)
     break;
   }
 
-  // TO DO! Need to add any joined itemd on to the core layer items, one long list! 
+  /* TO DO! Need to add any joined itemd on to the core layer items, one long list!  */
 
   return(MS_FAILURE);
 }
@@ -734,19 +752,19 @@ static int string2list(char **list, int *listsize, char *string)
 
   for(i=0; i<(*listsize); i++)
     if(strcmp(list[i], string) == 0) { 
-        // printf("string2list (duplicate): %s %d\n", string, i);
+        /* printf("string2list (duplicate): %s %d\n", string, i); */
       return(i);
     }
 
   list[i] = strdup(string);
   (*listsize)++;
 
-  // printf("string2list: %s %d\n", string, i);
+  /* printf("string2list: %s %d\n", string, i); */
 
   return(i);
 }
 
-// TO DO: this function really needs to use the lexer
+/* TO DO: this function really needs to use the lexer */
 static void expression2list(char **list, int *listsize, expressionObj *expression) 
 {
   int i, j, l;
@@ -772,12 +790,12 @@ static void expression2list(char **list, int *listsize, expressionObj *expressio
       tmpstr2[j+1] = '\0';
       string2list(expression->items, &(expression->numitems), tmpstr2);
 
-      if(tmpint != expression->numitems) { // not a duplicate, so no need to calculate the index
+      if(tmpint != expression->numitems) { /* not a duplicate, so no need to calculate the index */
         tmpstr1[j-1] = '\0';
         expression->indexes[expression->numitems - 1] = string2list(list, listsize, tmpstr1);
       }
 
-      j = 0; // reset
+      j = 0; /* reset */
 
       continue;
     }
@@ -794,16 +812,16 @@ int msLayerWhichItemsNew(layerObj *layer, int classify, int annotate, char *meta
 {
   int status;
   int numchars;
-//  int i;
+/* int i; */
 
-  status = msLayerGetItems(layer); // get a list of all attributes available for this layer (including JOINs)
+  status = msLayerGetItems(layer); /* get a list of all attributes available for this layer (including JOINs) */
   if(status != MS_SUCCESS) return(status);
 
-  // allocate space for the various item lists
+  /* allocate space for the various item lists */
   if(classify && layer->filter.type == MS_EXPRESSION) { 
     numchars = countChars(layer->filter.string, '[');
     if(numchars > 0) {
-      layer->filter.items = (char **)calloc(numchars, sizeof(char *)); // should be more than enough space
+      layer->filter.items = (char **)calloc(numchars, sizeof(char *)); /* should be more than enough space */
       if(!(layer->filter.items)) {
 	msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
 	return(MS_FAILURE);
@@ -817,7 +835,7 @@ int msLayerWhichItemsNew(layerObj *layer, int classify, int annotate, char *meta
     }
   }
 
-  // for(i=0; i<layer->numitems; i++) { }
+  /* for(i=0; i<layer->numitems; i++) { } */
 
   return(MS_SUCCESS);
 }
@@ -834,18 +852,18 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
   int i, j;
   int nt=0, ne=0;
   
-  //
-  // TO DO! I have a better algorithm.
-  // 
-  // 1) call msLayerGetItems to get a complete list (including joins)
-  // 2) loop though item-based parameters and expressions to identify items to keep
-  // 3) based on 2) build a list of items
-  //
-  // This is more straight forward and robust, fixes the problem of [...] regular expressions
-  // embeded in logical expressions. It also opens up to using dynamic joins anywhere...
-  //
+  /*  */
+  /* TO DO! I have a better algorithm. */
+  /*  */
+  /* 1) call msLayerGetItems to get a complete list (including joins) */
+  /* 2) loop though item-based parameters and expressions to identify items to keep */
+  /* 3) based on 2) build a list of items */
+  /*  */
+  /* This is more straight forward and robust, fixes the problem of [...] regular expressions */
+  /* embeded in logical expressions. It also opens up to using dynamic joins anywhere... */
+  /*  */
 
-  // Cleanup any previous item selection
+  /* Cleanup any previous item selection */
   layerFreeItemInfo(layer);
   if(layer->items) {
     msFreeCharArray(layer->items, layer->numitems);
@@ -861,6 +879,7 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
       for(j=0; j<layer->class[i].numstyles; j++) {
         if(layer->class[i].styles[j].angleitem) nt++;
 	if(layer->class[i].styles[j].sizeitem) nt++;
+	if(layer->class[i].styles[j].rangeitem) nt++;
       }
     }
 
@@ -868,7 +887,7 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
     if(layer->filter.type == MS_EXPRESSION) {
       ne = countChars(layer->filter.string, '[');
       if(ne > 0) {
-	layer->filter.items = (char **)calloc(ne, sizeof(char *)); // should be more than enough space
+	layer->filter.items = (char **)calloc(ne, sizeof(char *)); /* should be more than enough space */
 	if(!(layer->filter.items)) {
 	  msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
 	  return(MS_FAILURE);
@@ -895,7 +914,7 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
     if(classify && layer->class[i].expression.type == MS_EXPRESSION) { 
       ne = countChars(layer->class[i].expression.string, '[');
       if(ne > 0) {
-	layer->class[i].expression.items = (char **)calloc(ne, sizeof(char *)); // should be more than enough space
+	layer->class[i].expression.items = (char **)calloc(ne, sizeof(char *)); /* should be more than enough space */
 	if(!(layer->class[i].expression.items)) {
 	  msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
 	  return(MS_FAILURE);
@@ -914,7 +933,7 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
     if(annotate && layer->class[i].text.type == MS_EXPRESSION) { 
       ne = countChars(layer->class[i].text.string, '[');
       if(ne > 0) {
-	layer->class[i].text.items = (char **)calloc(ne, sizeof(char *)); // should be more than enough space
+	layer->class[i].text.items = (char **)calloc(ne, sizeof(char *)); /* should be more than enough space */
 	if(!(layer->class[i].text.items)) {
 	  msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
 	  return(MS_FAILURE);
@@ -930,20 +949,20 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
     }
   }
 
-  // TODO: it would be nice to move this into the SDE code itself, feels wrong here...
+  /* TODO: it would be nice to move this into the SDE code itself, feels wrong here... */
   if(layer->connectiontype == MS_SDE) {
-    layer->items = (char **)calloc(nt+2, sizeof(char *)); // should be more than enough space, SDE always needs a couple of additional items
+    layer->items = (char **)calloc(nt+2, sizeof(char *)); /* should be more than enough space, SDE always needs a couple of additional items */
     if(!layer->items) {
       msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
       return(MS_FAILURE);
     }
-    layer->items[0] = msSDELayerGetRowIDColumn(layer); // row id
+    layer->items[0] = msSDELayerGetRowIDColumn(layer); /* row id */
     layer->items[1] = msSDELayerGetSpatialColumn(layer);
     layer->numitems = 2;
   } else {
-    //if(nt == 0) return(MS_SUCCESS);
+    /* if(nt == 0) return(MS_SUCCESS); */
     if(nt > 0) {
-      layer->items = (char **)calloc(nt, sizeof(char *)); // should be more than enough space
+      layer->items = (char **)calloc(nt, sizeof(char *)); /* should be more than enough space */
       if(!layer->items) {
         msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
         return(MS_FAILURE);
@@ -961,6 +980,7 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
 	for(j=0; j<layer->class[i].numstyles; j++) {
 	  if(layer->class[i].styles[j].angleitem) layer->class[i].styles[j].angleitemindex = string2list(layer->items, &(layer->numitems), layer->class[i].styles[j].angleitem);
 	  if(layer->class[i].styles[j].sizeitem) layer->class[i].styles[j].sizeitemindex = string2list(layer->items, &(layer->numitems), layer->class[i].styles[j].sizeitem); 
+	  if(layer->class[i].styles[j].rangeitem) layer->class[i].styles[j].rangeitemindex = string2list(layer->items, &(layer->numitems), layer->class[i].styles[j].rangeitem); 
 	}
       }
 
@@ -1005,7 +1025,7 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
     }
   }
 
-  // populate the iteminfo array
+  /* populate the iteminfo array */
   if(layer->numitems == 0)
     return(MS_SUCCESS);
 
@@ -1019,7 +1039,7 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
 int msLayerSetItems(layerObj *layer, char **items, int numitems)
 {
   int i;
-  // Cleanup any previous item selection
+  /* Cleanup any previous item selection */
   layerFreeItemInfo(layer);
   if(layer->items) {
     msFreeCharArray(layer->items, layer->numitems);
@@ -1027,7 +1047,7 @@ int msLayerSetItems(layerObj *layer, char **items, int numitems)
     layer->numitems = 0;
   }
 
-  // now allocate and set the layer item parameters 
+  /* now allocate and set the layer item parameters  */
   layer->items = (char **)malloc(sizeof(char *)*numitems);
   if(!layer->items) {
     msSetError(MS_MEMERR, NULL, "msLayerSetItems()");
@@ -1038,7 +1058,7 @@ int msLayerSetItems(layerObj *layer, char **items, int numitems)
     layer->items[i] = strdup(items[i]);
   layer->numitems = numitems;
 
-  // populate the iteminfo array
+  /* populate the iteminfo array */
   return(layerInitItemInfo(layer));
 
   return(MS_SUCCESS);
@@ -1194,7 +1214,6 @@ int msLayerClearProcessing( layerObj *layer ) {
     return layer->numprocessing;
 }
 
-
 int msPOSTGISLayerSetTimeFilter(layerObj *lp, 
                                 const char *timestring, 
                                 const char *timefield)
@@ -1212,7 +1231,7 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
       return MS_FALSE;
 
     if (strstr(timestring, ",") == NULL && 
-        strstr(timestring, "/") == NULL) //discrete time
+        strstr(timestring, "/") == NULL) /* discrete time */
       tmpstimestring = strdup(timestring);
     else
     {
@@ -1223,12 +1242,12 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
         if (numtimes >= 1)
         {
             tokens = split(atimes[0],  '/', &ntmp);
-            if (ntmp == 2) //ranges
+            if (ntmp == 2) /* ranges */
             {
                 tmpstimestring = strdup(tokens[0]);
                 msFreeCharArray(tokens, ntmp);
             }
-            else if (ntmp == 1) //multiple times
+            else if (ntmp == 1) /* multiple times */
             {
                 tmpstimestring = strdup(atimes[0]);
             }
@@ -1277,9 +1296,9 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
     if (!timeresolution)
       return MS_FALSE;
 
-    //where date_trunc('month', _cwctstamp) = '2004-08-01'
+    /* where date_trunc('month', _cwctstamp) = '2004-08-01' */
     if (strstr(timestring, ",") == NULL && 
-        strstr(timestring, "/") == NULL) //discrete time
+        strstr(timestring, "/") == NULL) /* discrete time */
     {
         if(lp->filteritem) free(lp->filteritem);
         lp->filteritem = strdup(timefield);
@@ -1299,12 +1318,12 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
         strcat(buffer, " = ");
         strcat(buffer,  "'");
         strcat(buffer, timestring);
-        //make sure that the timestring is complete and acceptable
-        //to the date_trunc function :
-        // - if the resolution is year (2004) or month (2004-01), 
-        //a complete string for time would be 2004-01-01
-        // - if the resolluion is hour or minute (2004-01-01 15), a 
-        // complete time is 2004-01-01 15:00:00
+        /* make sure that the timestring is complete and acceptable */
+        /* to the date_trunc function : */
+        /* - if the resolution is year (2004) or month (2004-01),  */
+        /* a complete string for time would be 2004-01-01 */
+        /* - if the resolluion is hour or minute (2004-01-01 15), a  */
+        /* complete time is 2004-01-01 15:00:00 */
         if (strcasecmp(timeresolution, "year")==0)
         {
             nlength = strlen(timestring);
@@ -1343,7 +1362,7 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
 
         strcat(buffer, ")");
         
-        //loadExpressionString(&lp->filter, (char *)timestring);
+        /* loadExpressionString(&lp->filter, (char *)timestring); */
         loadExpressionString(&lp->filter, buffer);
 
         free(timeresolution);
@@ -1356,9 +1375,9 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
 
     if (numtimes >= 1)
     {
-        //check to see if we have ranges by parsing the first entry
+        /* check to see if we have ranges by parsing the first entry */
         tokens = split(atimes[0],  '/', &ntmp);
-        if (ntmp == 2) //ranges
+        if (ntmp == 2) /* ranges */
         {
             msFreeCharArray(tokens, ntmp);
             for (i=0; i<numtimes; i++)
@@ -1384,10 +1403,10 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
                     strcat(buffer,  "'");
 
                     strcat(buffer, tokens[0]);
-                    // - if the resolution is year (2004) or month (2004-01), 
-                    //a complete string for time would be 2004-01-01
-                    // - if the resolluion is hour or minute (2004-01-01 15), a 
-                    // complete time is 2004-01-01 15:00:00
+                    /* - if the resolution is year (2004) or month (2004-01),  */
+                    /* a complete string for time would be 2004-01-01 */
+                    /* - if the resolluion is hour or minute (2004-01-01 15), a  */
+                    /* complete time is 2004-01-01 15:00:00 */
                     if (strcasecmp(timeresolution, "year")==0)
                     {
                         nlength = strlen(tokens[0]);
@@ -1436,10 +1455,10 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
                     strcat(buffer,  "'");
                     strcat(buffer, tokens[1]);
 
-                    // - if the resolution is year (2004) or month (2004-01), 
-                    //a complete string for time would be 2004-01-01
-                    // - if the resolluion is hour or minute (2004-01-01 15), a 
-                    // complete time is 2004-01-01 15:00:00
+                    /* - if the resolution is year (2004) or month (2004-01),  */
+                    /* a complete string for time would be 2004-01-01 */
+                    /* - if the resolluion is hour or minute (2004-01-01 15), a  */
+                    /* complete time is 2004-01-01 15:00:00 */
                     if (strcasecmp(timeresolution, "year")==0)
                     {
                         nlength = strlen(tokens[1]);
@@ -1482,7 +1501,7 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
             if (strlen(buffer) > 0)
               strcat(buffer, ")");
         }
-        else if (ntmp == 1) //multiple times
+        else if (ntmp == 1) /* multiple times */
         {
             msFreeCharArray(tokens, ntmp);
             strcat(buffer, "(");
@@ -1505,12 +1524,12 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
 
                 strcat(buffer, atimes[i]);
                 
-                //make sure that the timestring is complete and acceptable
-                //to the date_trunc function :
-                // - if the resolution is year (2004) or month (2004-01), 
-                //a complete string for time would be 2004-01-01
-                // - if the resolluion is hour or minute (2004-01-01 15), a 
-                // complete time is 2004-01-01 15:00:00
+                /* make sure that the timestring is complete and acceptable */
+                /* to the date_trunc function : */
+                /* - if the resolution is year (2004) or month (2004-01),  */
+                /* a complete string for time would be 2004-01-01 */
+                /* - if the resolluion is hour or minute (2004-01-01 15), a  */
+                /* complete time is 2004-01-01 15:00:00 */
                 if (strcasecmp(timeresolution, "year")==0)
                 {
                     nlength = strlen(atimes[i]);
@@ -1557,7 +1576,7 @@ int msPOSTGISLayerSetTimeFilter(layerObj *lp,
 
         msFreeCharArray(atimes, numtimes);
 
-        //load the string to the filter
+        /* load the string to the filter */
         if (strlen(buffer) > 0)
         {
             if(lp->filteritem) 
@@ -1600,8 +1619,8 @@ int msLayerSetTimeFilter(layerObj *lp, const char *timestring,
         return msPOSTGISLayerSetTimeFilter(lp,timestring, timefield);
     }
 
-    //for shape and ogr files time expressions are
-    //delimited using backtics (ex `[TIME]` eq `2004-01-01`)
+    /* for shape and ogr files time expressions are */
+    /* delimited using backtics (ex `[TIME]` eq `2004-01-01`) */
     if (lp->connectiontype == MS_SHAPEFILE ||
         lp->connectiontype == MS_OGR ||
         lp->connectiontype == MS_TILED_SHAPEFILE ||
@@ -1611,12 +1630,12 @@ int msLayerSetTimeFilter(layerObj *lp, const char *timestring,
       addtimebacktics = 0;
 
     
-    //parse the time string. We support dicrete times (eg 2004-09-21), 
-    //multiple times (2004-09-21, 2004-09-22, ...)
-    //and range(s) (2004-09-21/2004-09-25, 2004-09-27/2004-09-29)
+    /* parse the time string. We support dicrete times (eg 2004-09-21),  */
+    /* multiple times (2004-09-21, 2004-09-22, ...) */
+    /* and range(s) (2004-09-21/2004-09-25, 2004-09-27/2004-09-29) */
 
     if (strstr(timestring, ",") == NULL && 
-        strstr(timestring, "/") == NULL) //discrete time
+        strstr(timestring, "/") == NULL) /* discrete time */
     {
         if(lp->filteritem) free(lp->filteritem);
         lp->filteritem = strdup(timefield);
@@ -1651,7 +1670,7 @@ int msLayerSetTimeFilter(layerObj *lp, const char *timestring,
 
         strcat(buffer, ")");
         
-        //loadExpressionString(&lp->filter, (char *)timestring);
+        /* loadExpressionString(&lp->filter, (char *)timestring); */
         loadExpressionString(&lp->filter, buffer);
 
         return MS_TRUE;
@@ -1663,9 +1682,9 @@ int msLayerSetTimeFilter(layerObj *lp, const char *timestring,
 
     if (numtimes >= 1)
     {
-        //check to see if we have ranges by parsing the first entry
+        /* check to see if we have ranges by parsing the first entry */
         tokens = split(atimes[0],  '/', &ntmp);
-        if (ntmp == 2) //ranges
+        if (ntmp == 2) /* ranges */
         {
                 
             msFreeCharArray(tokens, ntmp);
@@ -1735,7 +1754,7 @@ int msLayerSetTimeFilter(layerObj *lp, const char *timestring,
              if (strlen(buffer) > 0)
                strcat(buffer, ")");
         }
-        else if (ntmp == 1) //multiple times
+        else if (ntmp == 1) /* multiple times */
         {
              msFreeCharArray(tokens, ntmp);
              strcat(buffer, "(");
@@ -1780,7 +1799,7 @@ int msLayerSetTimeFilter(layerObj *lp, const char *timestring,
 
         msFreeCharArray(atimes, numtimes);
 
-        //load the string to the filter
+        /* load the string to the filter */
         if (strlen(buffer) > 0)
         {
             if(lp->filteritem) 
