@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapogcfilter.h,v 1.7 2004/07/28 22:16:17 assefa Exp $
+ * $Id: mapogcfilter.h,v 1.11 2005/05/12 17:38:54 assefa Exp $
  *
  * Name:     mapogcfilter.h
  * Project:  MapServer
@@ -29,6 +29,18 @@
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  * $Log: mapogcfilter.h,v $
+ * Revision 1.11  2005/05/12 17:38:54  assefa
+ * prototype for FLTGetShape changed : Now parsing unit value.
+ *
+ * Revision 1.10  2005/04/25 06:41:56  sdlime
+ * Applied Bill's newest gradient patch, more concise in the mapfile and potential to use via MapScript.
+ *
+ * Revision 1.9  2005/03/29 22:53:14  assefa
+ * Initial support to improve WFS filter performance for DB layers (Bug 1292).
+ *
+ * Revision 1.8  2005/01/28 06:16:54  sdlime
+ * Applied patch to make function prototypes ANSI C compliant. Thanks to Petter Reinholdtsen. This fixes but 1181.
+ *
  * Revision 1.7  2004/07/28 22:16:17  assefa
  * Add support for spatial filters inside an SLD. (Bug 782).
  *
@@ -114,7 +126,7 @@ typedef struct
 /*      prototypes.                                                     */
 /* -------------------------------------------------------------------- */
 FilterEncodingNode *FLTParseFilterEncoding(char *szXMLString);
-FilterEncodingNode *FLTCreateFilterEncodingNode();
+FilterEncodingNode *FLTCreateFilterEncodingNode(void);
 int FLTApplyFilterToLayer(FilterEncodingNode *psNode, mapObj *map, 
                          int iLayerIndex, int bOnlySpatialFilter);
 int FLTApplySpatialFilterToLayer(FilterEncodingNode *psNode, mapObj *map, 
@@ -148,11 +160,28 @@ char *FLTGetMapserverExpressionClassItem(FilterEncodingNode *psFilterNode);
 char *FLTGetNodeExpression(FilterEncodingNode *psFilterNode);
 char *FLTGetBBOX(FilterEncodingNode *psFilterNode, rectObj *psRect);
 
-shapeObj *FLTGetShape(FilterEncodingNode *psFilterNode, double *pdfDistance);
+shapeObj *FLTGetShape(FilterEncodingNode *psFilterNode, double *pdfDistance,
+                      int *pnUnit);
 
 char *FLTGetLogicalComparisonExpresssion(FilterEncodingNode *psFilterNode);
 char *FLTGetBinaryComparisonExpresssion(FilterEncodingNode *psFilterNode);
 char *FLTGetIsBetweenComparisonExpresssion(FilterEncodingNode *psFilterNode);
 char *FLTGetIsLikeComparisonExpression(FilterEncodingNode *psFilterNode);
-int FTLHasSpatialFilter(FilterEncodingNode *psFilterNode);
+int FLTHasSpatialFilter(FilterEncodingNode *psFilterNode);
+
+
+/*SQL expressions related functions.*/
+void FLTApplySimpleSQLFilter(FilterEncodingNode *psNode, mapObj *map, 
+                          int iLayerIndex);
+
+char *FLTGetSQLExpression(FilterEncodingNode *psFilterNode,int connectiontype);
+char *FLTGetBinaryComparisonSQLExpresssion(FilterEncodingNode *psFilterNode);
+char *FLTGetIsBetweenComparisonSQLExpresssion(FilterEncodingNode *psFilterNode);
+char *FLTGetIsLikeComparisonSQLExpression(FilterEncodingNode *psFilterNode,
+                                       int connectiontype);
+char *FLTGetLogicalComparisonSQLExpresssion(FilterEncodingNode *psFilterNode,
+                                            int connectiontype);
+int FLTIsSimpleFilter(FilterEncodingNode *psFilterNode);
+
+
 #endif
