@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapogr.cpp,v 1.79 2004/11/17 20:40:55 dan Exp $
+ * $Id: mapogr.cpp,v 1.87 2005/06/14 16:03:33 dan Exp $
  *
  * Name:     mapogr.cpp
  * Project:  MapServer
@@ -8,7 +8,7 @@
  * Author:   Daniel Morissette, DM Solutions Group (morissette@dmsolutions.ca)
  *
  **********************************************************************
- * Copyright (c) 2000-2003, Daniel Morissette, DM Solutions Group Inc
+ * Copyright (c) 2000-2005, Daniel Morissette, DM Solutions Group Inc
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,32 @@
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  * $Log: mapogr.cpp,v $
+ * Revision 1.87  2005/06/14 16:03:33  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.86  2005/04/21 15:09:28  julien
+ * Bug1244: Replace USE_SHAPE_Z_M by USE_POINT_Z_M
+ *
+ * Revision 1.85  2005/04/14 15:17:14  julien
+ * Bug 1244: Remove Z and M from point by default to gain performance.
+ *
+ * Revision 1.84  2005/03/29 17:56:40  frank
+ * Insure that tile index reading is restarted in msOGRLayerInitItemInfo()
+ * or else fastcgi repeat requests for a layer may fail on subsequent
+ * renders.
+ *
+ * Revision 1.83  2005/03/25 07:01:27  frank
+ * construct proper polygon spatial filter
+ *
+ * Revision 1.82  2005/03/25 05:43:43  frank
+ * use msAddLineDirectly() to avoid extra alloc/copy of points
+ *
+ * Revision 1.81  2004/12/13 07:57:58  frank
+ * bug 1126: support passing FILTER to OGR if prefixed with WHERE
+ *
+ * Revision 1.80  2004/12/13 03:57:17  frank
+ * modifed to handle all ogr geometry types in ogrGeomPoints per bug 1124
+ *
  * Revision 1.79  2004/11/17 20:40:55  dan
  * Pass config option GML_FIELDTYPES=ALWAYS_STRING to OGR so that all
  * GML attributes are returned as strings to MapServer. (bug 1043)
@@ -173,7 +199,7 @@
 #  include "cpl_string.h"
 #endif
 
-MS_CVSID("$Id: mapogr.cpp,v 1.79 2004/11/17 20:40:55 dan Exp $")
+MS_CVSID("$Id: mapogr.cpp,v 1.87 2005/06/14 16:03:33 dan Exp $")
 
 #ifdef USE_OGR
 
@@ -204,6 +230,10 @@ typedef struct ms_ogr_file_info_t
 
 /**********************************************************************
  *                     ogrPointsAddPoint()
+ *
+ * NOTE: This function assumes the line->point array already has been
+ * allocated large enough for the point to be added, but that numpoints
+ * does not include this new point. 
  **********************************************************************/
 static void ogrPointsAddPoint(lineObj *line, double dX, double dY, 
                               int lineindex, rectObj *bounds)
@@ -224,6 +254,10 @@ static void ogrPointsAddPoint(lineObj *line, double dX, double dY,
 
     line->point[line->numpoints].x = dX;
     line->point[line->numpoints].y = dY;
+#ifdef USE_POINT_Z_M
+    line->point[line->numpoints].z = 0.0;
+    line->point[line->numpoints].m = 0.0;
+#endif
     line->numpoints++;
 }
 
@@ -234,11 +268,68 @@ static int ogrGeomPoints(OGRGeometry *poGeom, shapeObj *outshp)
 {
   int   i;
   int   numpoints;
-  lineObj line={0,NULL};
 
   if (poGeom == NULL)   
       return 0;
 
+/* -------------------------------------------------------------------- */
+/*      Container types result in recursive invocation on each          */
+/*      subobject to add a set of points to the current list.           */
+/* -------------------------------------------------------------------- */
+  switch( wkbFlatten(poGeom->getGeometryType()) )
+  {
+      case wkbGeometryCollection:
+      case wkbMultiLineString:
+      case wkbMultiPolygon:
+      {
+          OGRGeometryCollection *poCollection = (OGRGeometryCollection *)
+              poGeom;
+          
+          for (int iGeom=0; iGeom < poCollection->getNumGeometries(); iGeom++ )
+          {
+              if( ogrGeomPoints( poCollection->getGeometryRef( iGeom ), 
+                                 outshp ) == -1 )
+                  return -1;
+          }
+
+          return 0;							
+      }
+      break;
+
+      case wkbPolygon:
+      {
+          OGRPolygon *poPoly = (OGRPolygon *)poGeom;
+
+          if( ogrGeomPoints( poPoly->getExteriorRing(), outshp ) == -1 )
+              return -1;
+          
+          for (int iRing=0; iRing < poPoly->getNumInteriorRings(); iRing++)
+          {
+              if( ogrGeomPoints( poPoly->getInteriorRing(iRing), outshp )
+                  == -1 )
+                  return -1;
+          }
+          return 0;
+      }
+      break;
+
+      case wkbPoint:
+      case wkbMultiPoint:
+      case wkbLineString:
+      case wkbLinearRing:
+          /* We will handle these directly */
+          break;
+
+      default:
+          /* There shouldn't be any more cases should there? */
+          msSetError(MS_OGRERR, 
+                     "OGRGeometry type `%s' not supported yet.", 
+                     "ogrGeomPoints()",
+                     poGeom->getGeometryName() );
+          return(-1);
+  }
+
+
 /* ------------------------------------------------------------------
  * Count total number of points
  * ------------------------------------------------------------------ */
@@ -246,26 +337,11 @@ static int ogrGeomPoints(OGRGeometry *poGeom, shapeObj *outshp)
   {
       numpoints = 1;
   }
-  else if ( wkbFlatten(poGeom->getGeometryType()) == wkbLineString )
+  else if ( wkbFlatten(poGeom->getGeometryType()) == wkbLineString 
+            || wkbFlatten(poGeom->getGeometryType()) == wkbLinearRing )
   {
       numpoints = ((OGRLineString*)poGeom)->getNumPoints();
   }
-  else if ( wkbFlatten(poGeom->getGeometryType()) == wkbPolygon )
-  {
-      OGRPolygon *poPoly = (OGRPolygon *)poGeom;
-      OGRLinearRing *poRing = poPoly->getExteriorRing();
-
-      numpoints=0;
-      if (poRing)
-          numpoints = poRing->getNumPoints();
-
-      for (int iRing=0; iRing < poPoly->getNumInteriorRings(); iRing++)
-      {
-          poRing = poPoly->getInteriorRing(iRing);
-          if (poRing)
-              numpoints += poRing->getNumPoints();
-      }
-  }
   else if ( wkbFlatten(poGeom->getGeometryType()) == wkbMultiPoint )
   {
       numpoints = ((OGRMultiPoint*)poGeom)->getNumGeometries();
@@ -273,56 +349,59 @@ static int ogrGeomPoints(OGRGeometry *poGeom, shapeObj *outshp)
   else
   {
       msSetError(MS_OGRERR, 
-                 (char*)CPLSPrintf("OGRGeometry type `%s' not supported yet.", 
-                                   poGeom->getGeometryName()),
-                 "ogrGeomPoints()");
+                 "OGRGeometry type `%s' not supported yet.", 
+                 "ogrGeomPoints()",
+                 poGeom->getGeometryName() );
       return(-1);
   }
 
 /* ------------------------------------------------------------------
- * alloc buffer and filter/transform points
+ * Do we need to allocate a line object to contain all our points? 
  * ------------------------------------------------------------------ */
-  line.numpoints = 0;
-  line.point = (pointObj *)malloc(sizeof(pointObj)*numpoints);
-  if(!line.point) 
+  if( outshp->numlines == 0 )
+  {
+      lineObj newline;
+      
+      newline.numpoints = 0;
+      newline.point = NULL;
+      msAddLine(outshp, &newline);
+  }
+  
+/* ------------------------------------------------------------------
+ * Extend the point array for the new of points to add from the 
+ * current geometry.
+ * ------------------------------------------------------------------ */
+  lineObj *line = outshp->line + outshp->numlines-1;
+
+  if( line->numpoints == 0 )
+      line->point = (pointObj *) malloc(sizeof(pointObj) * numpoints);
+  else
+      line->point = (pointObj *) 
+          realloc(line->point,sizeof(pointObj) * (numpoints+line->numpoints));
+  
+  if(!line->point) 
   {
       msSetError(MS_MEMERR, "Unable to allocate temporary point cache.", 
                  "ogrGeomPoints()");
       return(-1);
   }
    
+/* ------------------------------------------------------------------
+ * alloc buffer and filter/transform points
+ * ------------------------------------------------------------------ */
   if( wkbFlatten(poGeom->getGeometryType()) == wkbPoint )
   {
       OGRPoint *poPoint = (OGRPoint *)poGeom;
-      ogrPointsAddPoint(&line, poPoint->getX(), poPoint->getY(), 
-                        outshp->numlines, &(outshp->bounds));
+      ogrPointsAddPoint(line, poPoint->getX(), poPoint->getY(), 
+                        outshp->numlines-1, &(outshp->bounds));
   }
-  else if( wkbFlatten(poGeom->getGeometryType()) == wkbLineString )
+  else if( wkbFlatten(poGeom->getGeometryType()) == wkbLineString 
+           || wkbFlatten(poGeom->getGeometryType()) == wkbLinearRing )
   {
       OGRLineString *poLine = (OGRLineString *)poGeom;
       for(i=0; i<numpoints; i++)
-          ogrPointsAddPoint(&line, poLine->getX(i), poLine->getY(i),
-                            outshp->numlines, &(outshp->bounds));
-  }
-  else if (wkbFlatten(poGeom->getGeometryType()) == wkbPolygon )
-  {
-      OGRPolygon *poPoly = (OGRPolygon *)poGeom;
-      OGRLinearRing *poRing;
-
-      for (int iRing=-1; iRing < poPoly->getNumInteriorRings(); iRing++)
-      {
-          if (iRing == -1)
-              poRing = poPoly->getExteriorRing();
-          else
-              poRing = poPoly->getInteriorRing(iRing);
-
-          if (poRing)
-          {
-              for(i=0; i<poRing->getNumPoints(); i++)
-                  ogrPointsAddPoint(&line, poRing->getX(i), poRing->getY(i),
-                                    outshp->numlines, &(outshp->bounds));
-          }
-      }
+          ogrPointsAddPoint(line, poLine->getX(i), poLine->getY(i),
+                            outshp->numlines-1, &(outshp->bounds));
   }
   else if( wkbFlatten(poGeom->getGeometryType()) == wkbMultiPoint )
   {
@@ -331,14 +410,11 @@ static int ogrGeomPoints(OGRGeometry *poGeom, shapeObj *outshp)
       for(i=0; i<numpoints; i++)
       {
           poPoint = (OGRPoint *)poMultiPoint->getGeometryRef( i );
-          ogrPointsAddPoint(&line, poPoint->getX(), poPoint->getY(), 
-                            outshp->numlines, &(outshp->bounds));
+          ogrPointsAddPoint(line, poPoint->getX(), poPoint->getY(), 
+                            outshp->numlines-1, &(outshp->bounds));
       }
   }
 
-  msAddLine(outshp, &line);
-  free(line.point);
-
   outshp->type = MS_SHAPE_POINT;
 
   return(0);
@@ -462,8 +538,7 @@ static int ogrGeomLine(OGRGeometry *poGeom, shapeObj *outshp,
           line.numpoints++;
       }
 
-      msAddLine(outshp, &line);
-      free(line.point);
+      msAddLineDirectly(outshp, &line);
   }
   else
   {
@@ -1077,19 +1152,44 @@ static int msOGRFileWhichShapes(layerObj *layer, rectObj rect,
  * region and matches the layer's FILTER expression, but there is currently
  * no _efficient_ way to do that with OGR.
  * ------------------------------------------------------------------ */
-  OGRLinearRing oSpatialFilter;
+  OGRPolygon oSpatialFilter;
+  OGRLinearRing oRing;
 
-  oSpatialFilter.setNumPoints(5);
-  oSpatialFilter.setPoint(0, rect.minx, rect.miny);
-  oSpatialFilter.setPoint(1, rect.maxx, rect.miny);
-  oSpatialFilter.setPoint(2, rect.maxx, rect.maxy);
-  oSpatialFilter.setPoint(3, rect.minx, rect.maxy);
-  oSpatialFilter.setPoint(4, rect.minx, rect.miny);
+  oRing.setNumPoints(5);
+  oRing.setPoint(0, rect.minx, rect.miny);
+  oRing.setPoint(1, rect.maxx, rect.miny);
+  oRing.setPoint(2, rect.maxx, rect.maxy);
+  oRing.setPoint(3, rect.minx, rect.maxy);
+  oRing.setPoint(4, rect.minx, rect.miny);
+
+  oSpatialFilter.addRing( &oRing );
 
   psInfo->poLayer->SetSpatialFilter( &oSpatialFilter );
 
   psInfo->rect = rect;
 
+/* ------------------------------------------------------------------
+ * Apply an attribute filter if we have one prefixed with a WHERE 
+ * keyword in the filter string.  Otherwise, ensure the attribute
+ * filter is clear. 
+ * ------------------------------------------------------------------ */
+  if( layer->filter.string && EQUALN(layer->filter.string,"WHERE ",6) )
+  {
+      CPLErrorReset();
+      if( psInfo->poLayer->SetAttributeFilter( layer->filter.string+6 )
+          != OGRERR_NONE )
+      {
+          msSetError(MS_OGRERR,
+                     "SetAttributeFilter(%s) failed on layer %s.\n%s", 
+                     "msOGRFileWhichShapes()",
+                     layer->filter.string+6, layer->name, 
+                     CPLGetLastErrorMsg() );
+          return MS_FAILURE;
+      }
+  }
+  else
+      psInfo->poLayer->SetAttributeFilter( NULL );
+
 /* ------------------------------------------------------------------
  * Reset current feature pointer
  * ------------------------------------------------------------------ */
@@ -1184,8 +1284,11 @@ msOGRFileNextShape(layerObj *layer, shapeObj *shape,
           }
       }
 
-      if (msEvalExpression(&(layer->filter), layer->filteritemindex, 
-                           shape->values, layer->numitems) == MS_TRUE)
+      // Check the expression unless it is a WHERE clause already 
+      // handled by OGR. 
+      if( (layer->filter.string && EQUALN(layer->filter.string,"WHERE ",6))
+          || msEvalExpression(&(layer->filter), layer->filteritemindex, 
+                              shape->values, layer->numitems) == MS_TRUE)
       {
           // Feature matched filter expression... process geometry
           // shape->type will be set if geom is compatible with layer type
@@ -1312,6 +1415,16 @@ int msOGRFileReadTile( layerObj *layer, msOGRFileInfo *psInfo,
         psInfo->poCurTile = NULL;
     }
 
+/* -------------------------------------------------------------------- */
+/*      If -2 is passed, then seek reset reading of the tileindex.      */
+/*      We want to start from the beginning even if this file is        */
+/*      shared between layers or renders.                               */
+/* -------------------------------------------------------------------- */
+    if( targetTile == -2 )
+    {
+        psInfo->poLayer->ResetReading();
+    }
+        
 /* -------------------------------------------------------------------- */
 /*      Get the name (connection string really) of the next tile.       */
 /* -------------------------------------------------------------------- */
@@ -1323,8 +1436,8 @@ int msOGRFileReadTile( layerObj *layer, msOGRFileInfo *psInfo,
 #ifndef IGNORE_MISSING_DATA
   NextFile:
 #endif
-    
-    if( targetTile == -1 )
+
+    if( targetTile < 0 )
         poFeature = psInfo->poLayer->GetNextFeature();
     
     else
@@ -1670,7 +1783,7 @@ int msOGRLayerInitItemInfo(layerObj *layer)
   if( layer->tileindex != NULL )
   {
       if( psInfo->poCurTile == NULL 
-          && msOGRFileReadTile( layer, psInfo ) != MS_SUCCESS )
+          && msOGRFileReadTile( layer, psInfo, -2 ) != MS_SUCCESS )
           return MS_FAILURE;
       
       psInfo = psInfo->poCurTile;
