@@ -27,6 +27,18 @@
  ******************************************************************************
  *
  * $Log: mapoutput.c,v $
+ * Revision 1.34  2005/04/07 17:23:16  assefa
+ * Remove #ifdef USE_SVG. It was added during development.
+ *
+ * Revision 1.33  2005/03/08 19:53:25  frank
+ * adjusted png24 mimetype per bug 1231
+ *
+ * Revision 1.32  2005/02/08 21:36:23  frank
+ * Return actual error code.
+ *
+ * Revision 1.31  2005/02/03 00:06:57  assefa
+ * Add SVG function prototypes and related calls.
+ *
  * Revision 1.30  2004/11/18 20:55:38  frank
  * In msOutputFormatValidate() we now ensure that GD/JPEG does not have
  * alpha enabled, or transparent turned on.   Bug 1073.
@@ -127,7 +139,7 @@
 #include <assert.h>
 #include "map.h"
 
-MS_CVSID("$Id: mapoutput.c,v 1.30 2004/11/18 20:55:38 frank Exp $")
+MS_CVSID("$Id: mapoutput.c,v 1.34 2005/04/07 17:23:16 assefa Exp $")
 
 static outputFormatObj *msAllocOutputFormat( mapObj *map, const char *name, 
                                              const char *driver );
@@ -267,7 +279,7 @@ outputFormatObj *msCreateDefaultOutputFormat( mapObj *map,
     if( strcasecmp(driver,"GD/PNG24") == 0 )
     {
         format = msAllocOutputFormat( map, "png24", "GD/PNG" );
-        format->mimetype = strdup("image/png");
+        format->mimetype = strdup("image/png; mode=24bit");
         format->imagemode = MS_IMAGEMODE_RGB;
         format->extension = strdup("png");
         format->renderer = MS_RENDER_WITH_GD;
@@ -343,7 +355,14 @@ outputFormatObj *msCreateDefaultOutputFormat( mapObj *map,
         format->imagemode = 0;
         format->renderer = MS_RENDER_WITH_IMAGEMAP;
     }
-
+    if( strcasecmp(driver,"svg") == 0 )
+    {
+        format = msAllocOutputFormat( map, "svg", driver );
+        format->mimetype = strdup("image/svg+xml");
+        format->imagemode = 0;
+        format->extension = strdup("svg");
+        format->renderer = MS_RENDER_WITH_SVG;
+    }
     if( format != NULL )
         format->inmapfile = MS_FALSE;
 
@@ -391,6 +410,9 @@ void msApplyDefaultOutputFormats( mapObj *map )
     if( msSelectOutputFormat( map, "GTiff" ) == NULL )
         msCreateDefaultOutputFormat( map, "GDAL/GTiff" );
 
+    if( msSelectOutputFormat( map, "svg" ) == NULL )
+        msCreateDefaultOutputFormat( map, "svg" );
+
     if( map->imagetype != NULL )
         free( map->imagetype );
     map->imagetype = saved_imagetype;
@@ -965,5 +987,5 @@ int msOutputFormatValidate( outputFormatObj *format )
         && format->renderer != MS_RENDER_WITH_RAWDATA )
         format->renderer = MS_RENDER_WITH_RAWDATA;
 
-    return MS_TRUE;
+    return result;
 }
