@@ -1,10 +1,47 @@
 /**********************************************************************
- * $Id: mapows.h,v 1.47 2004/11/25 06:19:05 dan Exp $
+ * $Id: mapows.h,v 1.59 2005/06/02 20:32:01 sdlime Exp $
  *
  * mapows.h - OGC Web Services (WMS, WFS, WCS) support function definitions
  *
  **********************************************************************
  * $Log: mapows.h,v $
+ * Revision 1.59  2005/06/02 20:32:01  sdlime
+ * Changed metadata reference from ...getfeature_collection to ...feature_collection.
+ *
+ * Revision 1.58  2005/06/02 20:25:04  sdlime
+ * Updated WFS output to not use wfs:FeatureCollection as the main container for GML3 output. A default container of msFeatureCollection is provided or the user may define one explicitly.
+ *
+ * Revision 1.57  2005/05/31 18:24:49  sdlime
+ * Updated GML3 writer to use the new gmlGeometryListObj. This allows you to package geometries from WFS in a pretty flexible manner. Will port GML2 writer once testing on GML3 code is complete.
+ *
+ * Revision 1.56  2005/05/31 05:49:31  sdlime
+ * Added geometry metadata processing functions to mapgml.c.
+ *
+ * Revision 1.55  2005/05/31 05:21:17  sdlime
+ * Added couple structures for managing GML/WFS geometry types.
+ *
+ * Revision 1.54  2005/05/24 18:52:45  julien
+ * Bug 1149: From WMS 1.1.1, SRS are given in individual tags.
+ *
+ * Revision 1.53  2005/05/23 17:41:37  sdlime
+ * Added constants for the 2 types of schemas WFS servers will support. A default schema that references GML 2.1 geometries, and another for schemas complient with the GML profile for simple feature exchange document recently published (formerly known as GML3L0).
+ *
+ * Revision 1.52  2005/05/23 17:31:27  sdlime
+ * Move GML metadata structures and functions prototypes to mapows.h since they will be needed by mapwfs.c as well.
+ *
+ * Revision 1.51  2005/05/13 17:23:34  dan
+ * First pass at properly handling XML exceptions from CONNECTIONTYPE WMS
+ * layers. Still needs some work. (bug 1246)
+ *
+ * Revision 1.50  2005/04/21 21:10:38  sdlime
+ * Adjusted WFS support to allow for a new output format (GML3).
+ *
+ * Revision 1.49  2005/02/18 03:06:46  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.48  2004/12/29 22:49:57  sdlime
+ * Added GML3 writing capabilities to mapgml.c. Not hooked up to anything yet.
+ *
  * Revision 1.47  2004/11/25 06:19:05  dan
  * Add trailing "?" or "&" to connection string when required in WFS
  * client layers using GET method (bug 1082)
@@ -145,7 +182,7 @@ typedef struct http_request_info
     int         nTimeout;
     rectObj     bbox;
     int         nStatus;       /* 200=success, value < 0 if request failed */
-    char      * pszContentType;
+    char      * pszContentType;/* Content-Type of the response */
     char      * pszErrBuf;     /* Buffer where curl can write errors */
     char        *pszPostRequest;     /* post request content (NULL for GET) */
     char        *pszPostContentType; /* post request MIME type */
@@ -166,8 +203,8 @@ typedef  struct
   char *pszTypeName;
   char *pszFilter;
   int nMaxFeatures;
-  char *pszBbox; //only used with a Get Request
-  char *pszOutputFormat; //only used with DescibeFeatureType
+  char *pszBbox; /* only used with a Get Request */
+  char *pszOutputFormat; /* only used with DescibeFeatureType */
 
 } wfsParamsObj;
 
@@ -207,7 +244,7 @@ MS_DLL_EXPORT char *msOWSTerminateOnlineResource(const char *src_url);
 MS_DLL_EXPORT char *msOWSGetOnlineResource(mapObj *map, const char *namespaces, const char *metadata_name, cgiRequestObj *req);
 MS_DLL_EXPORT const char *msOWSGetSchemasLocation(mapObj *map);
 
-// Constants for OWS Service version numbers
+/* Constants for OWS Service version numbers */
 #define OWS_0_1_2   0x000102
 #define OWS_0_1_4   0x000104
 #define OWS_0_1_6   0x000106
@@ -218,18 +255,18 @@ MS_DLL_EXPORT const char *msOWSGetSchemasLocation(mapObj *map);
 #define OWS_1_0_8   0x010008
 #define OWS_1_1_0   0x010100
 #define OWS_1_1_1   0x010101
-#define OWS_VERSION_MAXLEN   20  // Buffer size for msOWSGetVersionString()
+#define OWS_VERSION_MAXLEN   20  /* Buffer size for msOWSGetVersionString() */
 
 MS_DLL_EXPORT int msOWSParseVersionString(const char *pszVersion);
 MS_DLL_EXPORT const char *msOWSGetVersionString(int nVersion, char *pszBuffer);
 
 
-// OWS_NOERR and OWS_WARN passed as action_if_not_found to printMetadata()
+/* OWS_NOERR and OWS_WARN passed as action_if_not_found to printMetadata() */
 #define OWS_NOERR   0
 #define OWS_WARN    1
 
-// OWS_WMS and OWS_WFS used for functions that differ in behavior between
-// WMS and WFS services (e.g. msOWSPrintLatLonBoundingBox())
+/* OWS_WMS and OWS_WFS used for functions that differ in behavior between */
+/* WMS and WFS services (e.g. msOWSPrintLatLonBoundingBox()) */
 #define OWS_WMS     1
 #define OWS_WFS     2
 
@@ -279,6 +316,11 @@ int msOWSPrintEncodeMetadataList(FILE *stream, hashTableObj *metadata,
                                  const char *startTag, 
                                  const char *endTag, const char *itemFormat,
                                  const char *default_value);
+int msOWSPrintEncodeParamList(FILE *stream, const char *name, 
+                              const char *value, int action_if_not_found, 
+                              char delimiter, const char *startTag, 
+                              const char *endTag, const char *format, 
+                              const char *default_value);
 void msOWSPrintLatLonBoundingBox(FILE *stream, const char *tabspace, 
                                  rectObj *extent, projectionObj *srcproj,
                                  int nService);
@@ -303,13 +345,65 @@ const char *msOWSGetEPSGProj(projectionObj *proj, hashTableObj *metadata, const
 /*====================================================================
  *   mapgml.c
  *====================================================================*/
+#define OWS_GML2 0 /* Supported GML formats */
+#define OWS_GML3 1
+
+#define OWS_WFS_FEATURE_COLLECTION_NAME "msFeatureCollection"
+#define OWS_GML_DEFAULT_GEOMETRY_NAME "msGeometry"
+#define OWS_GML_OCCUR_UNBOUNDED -1
+
+#if defined(USE_WMS_SVR) || defined (USE_WFS_SVR)
+typedef struct {
+  char *name;     /* name of the item */
+  char *alias;    /* is the item aliased for presentation? (NULL if not) */
+  char *type;     /* raw type for these item (NULL for a "string") (TODO: should this be a lookup table instead?) */
+  int encode;     /* should the value be HTML encoded? Default is MS_TRUE */
+  int visible;    /* should this item be output, default is MS_FALSE */  
+} gmlItemObj;
+
+typedef struct {
+  gmlItemObj *items;
+  int numitems;
+} gmlItemListObj;
+
+typedef struct {
+  char *name;     /* name of the geometry (type of GML property) */
+  char *type;     /* raw type for these geometries (point|multipoint|line|multiline|polygon|multipolygon */
+  int occurmin, occurmax;   /* number of occurances (default 0,1) */
+} gmlGeometryObj;
+
+typedef struct {
+  gmlGeometryObj *geometries;
+  int numgeometries;
+} gmlGeometryListObj;
+
+typedef struct {
+  char *name;     /* name of the group */
+  char **items;   /* list of items in the group */
+  int numitems;   /* number of items */
+} gmlGroupObj;
+
+typedef struct {
+  gmlGroupObj *groups;
+  int numgroups;
+} gmlGroupListObj;
+
+MS_DLL_EXPORT int msItemInGroups(gmlItemObj *item, gmlGroupListObj *groupList);
+MS_DLL_EXPORT gmlItemListObj *msGMLGetItems(layerObj *layer);
+MS_DLL_EXPORT void msGMLFreeItems(gmlItemListObj *itemList);
+MS_DLL_EXPORT gmlGeometryListObj *msGMLGetGeometries(layerObj *layer);
+MS_DLL_EXPORT void msGMLFreeGeometries(gmlGeometryListObj *geometryList);
+MS_DLL_EXPORT gmlGroupListObj *msGMLGetGroups(layerObj *layer);
+MS_DLL_EXPORT void msGMLFreeGroups(gmlGroupListObj *groupList);
+#endif
+
 #ifdef USE_WMS_SVR
-// export to fix bug 851
+/* export to fix bug 851 */
 MS_DLL_EXPORT int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces);
 #endif
 
 #ifdef USE_WFS_SVR
-MS_DLL_EXPORT int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *);
+MS_DLL_EXPORT int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *wfs_namespace, int outputformat);
 #endif
 
 
@@ -343,6 +437,11 @@ MS_DLL_EXPORT char *msWMSGetFeatureInfoURL(mapObj *map, layerObj *lp,
 /*====================================================================
  *   mapwfs.c
  *====================================================================*/
+
+/* Supported DescribeFeature formats */
+#define OWS_DEFAULT_SCHEMA 0 /* basically a GML 2.1 schema */
+#define OWS_SFE_SCHEMA 1 /* GML for simple feature exchange (formerly GML3L0) */
+
 int msWFSDispatch(mapObj *map, cgiRequestObj *requestobj);
 void msWFSParseRequest(cgiRequestObj *, wfsParamsObj *);
 wfsParamsObj *msWFSCreateParamsObj(void);
@@ -383,7 +482,7 @@ MS_DLL_EXPORT int msLoadMapContextURL(mapObj *map, char *urlfilename, int unique
  *   mapwcs.c
  *====================================================================*/
 
-int msWCSDispatch(mapObj *map, cgiRequestObj *requestobj); // only 1 public function
+int msWCSDispatch(mapObj *map, cgiRequestObj *requestobj); /* only 1 public function */
 
 #endif /* MAPOWS_H */
 
