@@ -115,8 +115,8 @@
 #include "map.h" /* for TRUE/FALSE and REGEX includes */
 #include "maptime.h" /* for time comparison routines */
 
-extern int msyylex(); /* lexer globals */
-extern int msyyerror();
+extern int msyylex(void); /* lexer globals */
+extern int msyyerror(const char *);
 
 int msyyresult;
 
@@ -1254,17 +1254,17 @@ yyreduce:
   case 16:
 #line 140 "mapparser.y"
     {
-                                         regex_t re;
+                                         ms_regex_t re;
 
-                                         if(regcomp(&re, yyvsp[0].strval, REG_EXTENDED|REG_NOSUB) != 0) 
+                                         if(ms_regcomp(&re, yyvsp[0].strval, MS_REG_EXTENDED|MS_REG_NOSUB) != 0) 
                                            yyval.intval = MS_FALSE;
 
-                                         if(regexec(&re, yyvsp[-2].strval, 0, NULL, 0) == 0)
+                                         if(ms_regexec(&re, yyvsp[-2].strval, 0, NULL, 0) == 0)
                                   	   yyval.intval = MS_TRUE;
 			                 else
 			                   yyval.intval = MS_FALSE;
 
-                                         regfree(&re);
+                                         ms_regfree(&re);
                                        ;}
     break;
 
