@@ -1,12 +1,12 @@
 /******************************************************************************
- * $Id: mappool.c,v 1.4 2004/10/21 04:30:55 frank Exp $
+ * $Id: mappool.c,v 1.10.2.1 2005/07/07 14:51:16 frank Exp $
  *
  * Project:  MapServer
  * Purpose:  Implement new style connection pooling.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
  ******************************************************************************
- * Copyright (c) 1996-2001 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,6 +28,27 @@
  ******************************************************************************
  *
  * $Log: mappool.c,v $
+ * Revision 1.10.2.1  2005/07/07 14:51:16  frank
+ * bug1402: any thread can release a connection
+ *
+ * Revision 1.10  2005/06/14 16:03:34  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.9  2005/02/22 00:35:25  frank
+ * More fixes for locking.
+ *
+ * Revision 1.8  2005/02/22 00:26:52  frank
+ * fixed some lock acquisition flaws
+ *
+ * Revision 1.7  2005/02/18 03:06:46  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.6  2005/02/03 04:54:47  frank
+ * Fixed lock releaseing in msConnPoolRelease().
+ *
+ * Revision 1.5  2005/02/02 17:57:48  frank
+ * Added multithreading safety support for pool
+ *
  * Revision 1.4  2004/10/21 04:30:55  frank
  * Added standardized headers.  Added MS_CVSID().
  *
@@ -145,8 +166,9 @@ o Currently the connection pooling API makes not attempt to address
  ****************************************************************************/
 
 #include "map.h"
+#include "mapthread.h"
 
-MS_CVSID("$Id: mappool.c,v 1.4 2004/10/21 04:30:55 frank Exp $")
+MS_CVSID("$Id: mappool.c,v 1.10.2.1 2005/07/07 14:51:16 frank Exp $")
 
 /* defines for lifetime.  
    A positive number is a time-from-last use in seconds */
@@ -160,6 +182,7 @@ typedef struct {
 
     int   lifespan;
     int   ref_count;
+    int   thread_id;
     int   debug;
 
     time_t last_used;
@@ -219,6 +242,8 @@ void msConnPoolRegister( layerObj *layer,
 /* -------------------------------------------------------------------- */
 /*      Grow the array of connection information objects if needed.     */
 /* -------------------------------------------------------------------- */
+    msAcquireLock( TLOCK_POOL );
+
     if( connectionCount == connectionMax )
     {
         connectionMax += 10;
@@ -228,6 +253,7 @@ void msConnPoolRegister( layerObj *layer,
         if( connections == NULL )
         {
             msSetError(MS_MEMERR, NULL, "msConnPoolRegister()");
+            msReleaseLock( TLOCK_POOL );
             return;
         }
     }
@@ -243,6 +269,7 @@ void msConnPoolRegister( layerObj *layer,
     conn->connection = strdup( layer->connection );
     conn->close = close_func;
     conn->ref_count = 1;
+    conn->thread_id = msGetThreadId();
     conn->last_used = time(NULL);
     conn->conn_handle = conn_handle;
     conn->debug = layer->debug;
@@ -271,6 +298,8 @@ void msConnPoolRegister( layerObj *layer,
                     close_connection );
         conn->lifespan = MS_LIFE_ZEROREF;
     }
+
+    msReleaseLock( TLOCK_POOL );
 }
 
 /************************************************************************/
@@ -306,7 +335,6 @@ static void msConnPoolClose( int conn_index )
         conn->close( conn->conn_handle );
 
     /* free malloced() stuff in this connection */
-
     free( conn->connection );
 
     /* move the last connection in place of our now closed one */
@@ -324,7 +352,6 @@ static void msConnPoolClose( int conn_index )
         free( connections );
         connections = NULL;
     }
-    
 }
 
 /************************************************************************/
@@ -344,14 +371,19 @@ void *msConnPoolRequest( layerObj *layer )
     if( layer->connection == NULL )
         return NULL;
 
+    msAcquireLock( TLOCK_POOL );
     for( i = 0; i < connectionCount; i++ )
     {
         connectionObj *conn = connections + i;
 
         if( layer->connectiontype == conn->connectiontype
-            && strcasecmp( layer->connection, conn->connection ) == 0 )
+            && strcasecmp( layer->connection, conn->connection ) == 0 
+            && (conn->ref_count == 0 || conn->thread_id == msGetThreadId()) )
         {
+            void *conn_handle = NULL;
+
             conn->ref_count++;
+            conn->thread_id = msGetThreadId();
             conn->last_used = time(NULL);
 
             if( layer->debug )
@@ -361,10 +393,15 @@ void *msConnPoolRequest( layerObj *layer )
                 conn->debug = layer->debug;
             }
 
-            return conn->conn_handle;
+            conn_handle = conn->conn_handle;
+
+            msReleaseLock( TLOCK_POOL );
+            return conn_handle;
         }
     }
 
+    msReleaseLock( TLOCK_POOL );
+
     return NULL;
 }
 
@@ -373,7 +410,8 @@ void *msConnPoolRequest( layerObj *layer )
 /*                                                                      */
 /*      Release the passed connection for the given layer.              */
 /*      Internally the reference count is dropped, and the              */
-/*      connection may be closed.                                       */
+/*      connection may be closed.  We assume the caller has already     */
+/*      acquired the pool lock.                                         */
 /************************************************************************/
 
 void msConnPoolRelease( layerObj *layer, void *conn_handle )
@@ -388,38 +426,31 @@ void msConnPoolRelease( layerObj *layer, void *conn_handle )
     if( layer->connection == NULL )
         return;
 
+    msAcquireLock( TLOCK_POOL );
     for( i = 0; i < connectionCount; i++ )
     {
         connectionObj *conn = connections + i;
 
         if( layer->connectiontype == conn->connectiontype
-            && strcasecmp( layer->connection, conn->connection ) == 0 )
+            && strcasecmp( layer->connection, conn->connection ) == 0 
+            && conn->conn_handle == conn_handle )
         {
-            if( conn->conn_handle != conn_handle )
-            {
-                msDebug( "%s: Handle confusion on layer %s, passed in %p, but expected %p.\n", 
-                         "msConnPoolRelease()",
-                         layer->name,
-                         conn_handle, 
-                         conn->conn_handle );
-
-                msSetError( MS_MISCERR, 
-                            "Handle confusion on layer %s.", 
-                            "msConnPoolRelease()",
-                            layer->name );
-                return;
-            }
-
             conn->ref_count--;
             conn->last_used = time(NULL);
 
+            if( conn->ref_count == 0 )
+                conn->thread_id = 0;
+
             if( conn->ref_count == 0 && conn->lifespan == MS_LIFE_ZEROREF )
                 msConnPoolClose( i );
 
+            msReleaseLock( TLOCK_POOL );
             return;
         }
     }
 
+    msReleaseLock( TLOCK_POOL );
+
     msDebug( "%s: Unable to find handle for layer '%s'.\n",
              "msConnPoolRelease()",
              layer->name );
@@ -441,16 +472,22 @@ void msConnPoolCloseUnreferenced()
 {
     int  i;
 
-    // this really needs to be commented out before commiting. 
-    // msDebug( "msConnPoolCloseUnreferenced()\n" );
+    /* this really needs to be commented out before commiting.  */
+    /* msDebug( "msConnPoolCloseUnreferenced()\n" ); */
 
+    msAcquireLock( TLOCK_POOL );
     for( i = connectionCount - 1; i >= 0; i-- )
     {
         connectionObj *conn = connections + i;
 
         if( conn->ref_count == 0 )
+        {
+            /* for now we don't assume the locks are re-entrant, so release */
+            /* it so msConnPoolClose() can get it.  */
             msConnPoolClose( i );
+        }
     }
+    msReleaseLock( TLOCK_POOL );
 }
 
 /************************************************************************/
@@ -463,9 +500,11 @@ void msConnPoolCloseUnreferenced()
 void msConnPoolFinalCleanup()
 
 {
-    // this really needs to be commented out before commiting. 
-    // msDebug( "msConnPoolFinalCleanup()\n" );
+    /* this really needs to be commented out before commiting.  */
+    /* msDebug( "msConnPoolFinalCleanup()\n" ); */
 
+    msAcquireLock( TLOCK_POOL );
     while( connectionCount > 0 )
         msConnPoolClose( 0 );
+    msReleaseLock( TLOCK_POOL );
 }
