@@ -5,7 +5,7 @@
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,39 @@
  ******************************************************************************
  *
  * $Log: mapprimitive.c,v $
+ * Revision 1.53  2005/06/14 16:03:34  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.52  2005/05/19 05:57:08  sdlime
+ * Added GEOS geometry clean up code to msFreeShape...
+ *
+ * Revision 1.51  2005/04/21 15:09:28  julien
+ * Bug1244: Replace USE_SHAPE_Z_M by USE_POINT_Z_M
+ *
+ * Revision 1.50  2005/04/14 15:17:14  julien
+ * Bug 1244: Remove Z and M from point by default to gain performance.
+ *
+ * Revision 1.49  2005/03/25 05:42:58  frank
+ * added msAddLineDirectly(), msAddLine() uses memcpy()
+ *
+ * Revision 1.48  2005/03/24 22:27:20  frank
+ * optimized msTransformShapeToPixel - avoid division
+ *
+ * Revision 1.47  2005/03/24 17:50:04  frank
+ * optimized msClipPoly{gon/line}Rect for all-inside case
+ *
+ * Revision 1.46  2005/02/22 07:40:27  sdlime
+ * A bunch of updates to GEOS integration. Can move many primatives between MapServer and GEOS, still need to do collections (e.g. multi-point/line/polygon). Added buffer method to mapscript (mapscript/shape.i).
+ *
+ * Revision 1.45  2005/02/18 03:06:46  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.44  2005/02/13 22:16:06  dan
+ * Use double as second arg to pow() (bug 1235)
+ *
+ * Revision 1.43  2004/12/14 21:30:43  sdlime
+ * Moved functions to build lists of inner and outer rings to mapprimitive.c from mapgml.c. They are needed to covert between MapServer polygons and GEOS gemometries (bug 771).
+ *
  * Revision 1.42  2004/11/12 20:23:16  frank
  * include z and m in formatted pointObj, fmt is const!
  *
@@ -38,7 +71,7 @@
 #include "map.h"
 #include "mapprimitive.h"
 
-MS_CVSID("$Id: mapprimitive.c,v 1.42 2004/11/12 20:23:16 frank Exp $")
+MS_CVSID("$Id: mapprimitive.c,v 1.53 2005/06/14 16:03:34 dan Exp $")
 
 typedef enum {CLIP_LEFT, CLIP_MIDDLE, CLIP_RIGHT} CLIP_STATE;
 
@@ -65,22 +98,26 @@ void msPrintShape(shapeObj *p)
 
 void msInitShape(shapeObj *shape)
 {
-  // spatial component
+  /* spatial component */
   shape->line = NULL;
   shape->numlines = 0;
   shape->type = MS_SHAPE_NULL;
   shape->bounds.minx = shape->bounds.miny = -1;
   shape->bounds.maxx = shape->bounds.maxy = -1;
   
-  // attribute component
+  /* attribute component */
   shape->values = NULL;
   shape->numvalues = 0;
 
-  // annotation component
+#ifdef USE_GEOS
+  shape->geometry = NULL;
+#endif
+
+  /* annotation component */
   shape->text = NULL;
 
-  // bookkeeping component
-  shape->classindex = 0; // default class
+  /* bookkeeping component */
+  shape->classindex = 0; /* default class */
   shape->tileindex = shape->index = -1;
 }
 
@@ -90,7 +127,7 @@ int msCopyShape(shapeObj *from, shapeObj *to) {
   if(!from || !to) return(-1);
 
   for(i=0; i<from->numlines; i++)
-    msAddLine(to, &(from->line[i])); // copy each line
+    msAddLine(to, &(from->line[i])); /* copy each line */
 
   to->type = from->type;
 
@@ -112,6 +149,8 @@ int msCopyShape(shapeObj *from, shapeObj *to) {
     to->numvalues = from->numvalues;
   }
 
+  /* TODO add GEOS copy, or at least initialize to->geometry to NULL */
+
   return(0);
 }
 
@@ -126,7 +165,11 @@ void msFreeShape(shapeObj *shape)
   if(shape->values) msFreeCharArray(shape->values, shape->numvalues);
   if(shape->text) free(shape->text);
   
-  msInitShape(shape); // now reset
+#ifdef USE_GEOS
+  msGEOSFreeGeometry(shape);
+#endif
+
+  msInitShape(shape); /* now reset */
 }
 
 void msComputeBounds(shapeObj *shape)
@@ -149,9 +192,68 @@ void msComputeBounds(shapeObj *shape)
   }
 }
 
+/* checks to see if ring r is an outer ring of shape */
+static int isOuterRing(shapeObj *shape, int r) 
+{
+  pointObj point; /* a point in the ring */
+  shapeObj ring;
+
+  if(shape->numlines == 1) return(MS_TRUE);
+
+  msInitShape(&ring); /* convert the ring of interest into its own shape */
+  msAddLine(&ring, &(shape->line[r]));
+
+  msPolygonLabelPoint(&ring, &point, -1); /* generate a point in that ring */
+  msFreeShape(&ring); /* done with it */
+
+  return(msIntersectPointPolygon(&point, shape)); /* test the point against the main shape */
+}
+
+/*
+** Returns a list of outer rings for shape (the list has one entry for each ring, 
+** MS_TRUE for outer rings).
+*/
+int *msGetOuterList(shapeObj *shape) 
+{
+  int i;
+  int *list;
+
+  list = (int *)malloc(sizeof(int)*shape->numlines);
+  if(!list) return(NULL);
+
+  for(i=0; i<shape->numlines; i++)
+    list[i] = isOuterRing(shape, i);
+
+  return(list);
+}
+
+/*
+** Returns a list of inner rings for ring r in shape (given a list of outer rings).
+*/
+int *msGetInnerList(shapeObj *shape, int r, int *outerlist) 
+{
+  int i;
+  int *list;
+
+  list = (int *)malloc(sizeof(int)*shape->numlines);
+  if(!list) return(NULL);
+
+  for(i=0; i<shape->numlines; i++) { /* test all rings against the ring */
+
+    if(outerlist[i] == MS_TRUE) { /* ring is an outer and can't be an inner */
+      list[i] = MS_FALSE;
+      continue;
+    }
+
+    list[i] = msPointInPolygon(&(shape->line[i].point[0]), &(shape->line[r]));
+  }
+
+  return(list);
+}
+
 int msAddLine(shapeObj *p, lineObj *new_line)
 {
-  int c, v;
+  int c;
   lineObj *extended_line;
 
   /* Create an extended line array */
@@ -172,8 +274,46 @@ int msAddLine(shapeObj *p, lineObj *new_line)
     return(MS_FAILURE);
   }
 
-  for (v= 0; v < new_line->numpoints; v++)
-    extended_line[c].point[v]= new_line->point[v];
+  memcpy( extended_line[c].point, new_line->point, 
+          sizeof(pointObj) * new_line->numpoints );
+
+  /* Dispose of the old line */
+  if(p->line) free(p->line);
+
+  /* Update the polygon information */
+  p->numlines++;
+  p->line = extended_line;
+
+  return(MS_SUCCESS);
+}
+
+/*
+** Same as msAddLine(), except that this version "seizes" the points
+** array from the passed in line and uses it instead of copying it.  
+*/
+int msAddLineDirectly(shapeObj *p, lineObj *new_line)
+{
+  int c;
+  lineObj *extended_line;
+
+  /* Create an extended line array */
+  if((extended_line = (lineObj *)malloc((p->numlines + 1) * sizeof(lineObj))) == NULL) {
+    msSetError(MS_MEMERR, NULL, "msAddLine()");
+    return(MS_FAILURE);
+  }
+
+  /* Copy the old line into the extended line array */
+  for (c= 0; c < p->numlines; c++)
+    extended_line[c]= p->line[c];
+
+  /* Copy the new line onto the end of the extended line array */
+  c= p->numlines;
+  extended_line[c].numpoints = new_line->numpoints;  
+  extended_line[c].point = new_line->point;
+
+  /* strip points reference off the passed in lineObj */
+  new_line->point = NULL;
+  new_line->numpoints = 0;
 
   /* Dispose of the old line */
   if(p->line) free(p->line);
@@ -213,7 +353,7 @@ void msRectToPolygon(rectObj rect, shapeObj *poly)
   line.numpoints = 5;
   
   msAddLine(poly, &line);
-  if(poly->numlines == 1) { // poly was empty to begin with
+  if(poly->numlines == 1) { /* poly was empty to begin with */
     poly->type = MS_SHAPE_POLYGON;
     poly->bounds = rect;
   } else
@@ -311,6 +451,21 @@ void msClipPolylineRect(shapeObj *shape, rectObj rect)
   if(shape->numlines == 0) /* nothing to clip */
     return;
 
+  /*
+  ** Don't do any clip processing of shapes completely within the
+  ** clip rectangle based on a comparison of bounds.   We could do 
+  ** something similar for completely outside, but that rarely occurs
+  ** since the spatial query at the layer read level has generally already
+  ** discarded all shapes completely outside the rect.
+  */
+  if( shape->bounds.maxx <= rect.maxx
+      && shape->bounds.minx >= rect.minx
+      && shape->bounds.maxy <= rect.maxy
+      && shape->bounds.miny >= rect.miny )
+  {
+      return;
+  }
+
   for(i=0; i<shape->numlines; i++) {
 
     line.point = (pointObj *)malloc(sizeof(pointObj)*shape->line[i].numpoints);
@@ -375,7 +530,22 @@ void msClipPolygonRect(shapeObj *shape, rectObj rect)
   
   if(shape->numlines == 0) /* nothing to clip */
     return;
-   
+
+  /*
+  ** Don't do any clip processing of shapes completely within the
+  ** clip rectangle based on a comparison of bounds.   We could do 
+  ** something similar for completely outside, but that rarely occurs
+  ** since the spatial query at the layer read level has generally already
+  ** discarded all shapes completely outside the rect.
+  */
+  if( shape->bounds.maxx <= rect.maxx
+      && shape->bounds.minx >= rect.minx
+      && shape->bounds.maxy <= rect.maxy
+      && shape->bounds.miny >= rect.miny )
+  {
+      return;
+  }
+
   for(j=0; j<shape->numlines; j++) {
 
     line.point = (pointObj *)malloc(sizeof(pointObj)*2*shape->line[j].numpoints+1); /* worst case scenario, +1 allows us to duplicate the 1st and last point */
@@ -482,7 +652,7 @@ void msClipPolygonRect(shapeObj *shape, rectObj rect)
     }
 
     if(line.numpoints > 0) {
-      line.point[line.numpoints].x = line.point[0].x; // force closure
+      line.point[line.numpoints].x = line.point[0].x; /* force closure */
       line.point[line.numpoints].y = line.point[0].y;
       line.numpoints++;
       msAddLine(&tmp, &line);
@@ -506,21 +676,21 @@ void msClipPolygonRect(shapeObj *shape, rectObj rect)
 void msTransformShapeToPixel(shapeObj *shape, rectObj extent, double cellsize)
 {
   int i,j,k; /* loop counters */
+  double inv_cs = 1.0 / cellsize; /* invert and multiply much faster */
 
+  if(shape->numlines == 0) return; /* nothing to transform */
 
-  if(shape->numlines == 0) return; // nothing to transform
-
-  if(shape->type == MS_SHAPE_LINE || shape->type == MS_SHAPE_POLYGON) { // remove co-linear vertices
+  if(shape->type == MS_SHAPE_LINE || shape->type == MS_SHAPE_POLYGON) { /* remove co-linear vertices */
   
-    for(i=0; i<shape->numlines; i++) { // for each part
+    for(i=0; i<shape->numlines; i++) { /* for each part */
       
-      shape->line[i].point[0].x = MS_MAP2IMAGE_X(shape->line[i].point[0].x, extent.minx, cellsize);
-      shape->line[i].point[0].y = MS_MAP2IMAGE_Y(shape->line[i].point[0].y, extent.maxy, cellsize);
+      shape->line[i].point[0].x = MS_MAP2IMAGE_X_IC(shape->line[i].point[0].x, extent.minx, inv_cs);
+      shape->line[i].point[0].y = MS_MAP2IMAGE_Y_IC(shape->line[i].point[0].y, extent.maxy, inv_cs);
       
       for(j=1, k=1; j < shape->line[i].numpoints; j++ ) {
 	
-	shape->line[i].point[k].x = MS_MAP2IMAGE_X(shape->line[i].point[j].x, extent.minx, cellsize);
-	shape->line[i].point[k].y = MS_MAP2IMAGE_Y(shape->line[i].point[j].y, extent.maxy, cellsize);
+	shape->line[i].point[k].x = MS_MAP2IMAGE_X_IC(shape->line[i].point[j].x, extent.minx, inv_cs);
+	shape->line[i].point[k].y = MS_MAP2IMAGE_Y_IC(shape->line[i].point[j].y, extent.maxy, inv_cs);
 
 	if(k == 1) {
 	  if((shape->line[i].point[0].x != shape->line[i].point[1].x) || (shape->line[i].point[0].y != shape->line[i].point[1].y))
@@ -536,13 +706,13 @@ void msTransformShapeToPixel(shapeObj *shape, rectObj extent, double cellsize)
 	  }
 	}
       }
-      shape->line[i].numpoints = k; // save actual number kept
+      shape->line[i].numpoints = k; /* save actual number kept */
     }
-  } else { // points or untyped shapes
-    for(i=0; i<shape->numlines; i++) { // for each part
+  } else { /* points or untyped shapes */
+    for(i=0; i<shape->numlines; i++) { /* for each part */
       for(j=1; j < shape->line[i].numpoints; j++ ) {
-	shape->line[i].point[j].x = MS_MAP2IMAGE_X(shape->line[i].point[j].x, extent.minx, cellsize);
-	shape->line[i].point[j].y = MS_MAP2IMAGE_Y(shape->line[i].point[j].y, extent.maxy, cellsize);
+	shape->line[i].point[j].x = MS_MAP2IMAGE_X_IC(shape->line[i].point[j].x, extent.minx, inv_cs);
+	shape->line[i].point[j].y = MS_MAP2IMAGE_Y_IC(shape->line[i].point[j].y, extent.maxy, inv_cs);
 	  }
 	}
   }
@@ -554,11 +724,11 @@ void msTransformPixelToShape(shapeObj *shape, rectObj extent, double cellsize)
 {
 	int i,j; /* loop counters */
 
-	if(shape->numlines == 0) return; // nothing to transform
+	if(shape->numlines == 0) return; /* nothing to transform */
 
-	if(shape->type == MS_SHAPE_LINE || shape->type == MS_SHAPE_POLYGON)  // remove co-linear vertices
+	if(shape->type == MS_SHAPE_LINE || shape->type == MS_SHAPE_POLYGON)  /* remove co-linear vertices */
 	{
-		for(i=0; i<shape->numlines; i++)  // for each part
+		for(i=0; i<shape->numlines; i++)  /* for each part */
 		{ 
 			for(j=0; j < shape->line[i].numpoints; j++ ) 
 			{
@@ -567,9 +737,9 @@ void msTransformPixelToShape(shapeObj *shape, rectObj extent, double cellsize)
 			}
 		}
 	}
-	else  // points or untyped shapes
+	else  /* points or untyped shapes */
 	{
-		for(i=0; i<shape->numlines; i++)  // for each part
+		for(i=0; i<shape->numlines; i++)  /* for each part */
 		{
 			for(j=1; j < shape->line[i].numpoints; j++ ) 
 			{
@@ -619,7 +789,7 @@ void bufferPolyline(shapeObj *p, shapeObj *op, int w)
     outside.point = (pointObj *)malloc(sizeof(pointObj)*p->line[i].numpoints);
     inside.numpoints = outside.numpoints = p->line[i].numpoints;    
 
-    angle = asin(MS_ABS(p->line[i].point[1].x - p->line[i].point[0].x)/sqrt((pow((p->line[i].point[1].x - p->line[i].point[0].x),2) + pow((p->line[i].point[1].y - p->line[i].point[0].y),2))));
+    angle = asin(MS_ABS(p->line[i].point[1].x - p->line[i].point[0].x)/sqrt((pow((p->line[i].point[1].x - p->line[i].point[0].x),2.0) + pow((p->line[i].point[1].y - p->line[i].point[0].y),2.0))));
     if(p->line[i].point[0].x < p->line[i].point[1].x)
       dy = sin(angle) * (w/2);
     else
@@ -641,7 +811,7 @@ void bufferPolyline(shapeObj *p, shapeObj *op, int w)
 
     for(j=2; j<p->line[i].numpoints; j++) {
 
-      angle = asin(MS_ABS(p->line[i].point[j].x - p->line[i].point[j-1].x)/sqrt((pow((p->line[i].point[j].x - p->line[i].point[j-1].x),2) + pow((p->line[i].point[j].y - p->line[i].point[j-1].y),2))));
+      angle = asin(MS_ABS(p->line[i].point[j].x - p->line[i].point[j-1].x)/sqrt((pow((p->line[i].point[j].x - p->line[i].point[j-1].x),2.0) + pow((p->line[i].point[j].y - p->line[i].point[j-1].y),2.0))));
       if(p->line[i].point[j-1].x < p->line[i].point[j].x)
 	dy = sin(angle) * (w/2);
       else
@@ -664,7 +834,7 @@ void bufferPolyline(shapeObj *p, shapeObj *op, int w)
       outside.point[j-1] = generateLineIntersection(outside.point[j-2], outside.point[j-1], a, outside.point[j]);
     }
 
-    // need a touch of code if 1st point equals last point in p (find intersection)
+    /* need a touch of code if 1st point equals last point in p (find intersection) */
 
     msAddLine(op, &inside);
     msAddLine(op, &outside);
@@ -676,7 +846,7 @@ void bufferPolyline(shapeObj *p, shapeObj *op, int w)
   return;
 }
 
-// Currently unused.
+/* Currently unused. */
 #ifdef notdef
 static int get_centroid(shapeObj *p, pointObj *lp, double *miny, double *maxy)
 {
@@ -744,7 +914,7 @@ int msPolygonLabelPoint(shapeObj *p, pointObj *lp, int min_dimension)
   if(min_dimension != -1)
     if(MS_MIN(maxx-minx,maxy-miny) < min_dimension) return(MS_FAILURE);
 
-  //if(get_centroid(p, lp, &miny, &maxy) == -1) return(MS_FAILURE);
+  /* if(get_centroid(p, lp, &miny, &maxy) == -1) return(MS_FAILURE); */
   lp->x = (maxx+minx)/2.0;
   lp->y = (maxy+miny)/2.0;
 
@@ -866,7 +1036,7 @@ int msPolylineLabelPoint(shapeObj *p, pointObj *lp, int min_length, double *angl
     line_length = 0;
     max_segment_length = 0;
     for(j=1;j<p->line[i].numpoints;j++) {
-      segment_length = sqrt((pow((p->line[i].point[j].x-p->line[i].point[j-1].x),2) + pow((p->line[i].point[j].y-p->line[i].point[j-1].y),2)));
+      segment_length = sqrt((pow((p->line[i].point[j].x-p->line[i].point[j-1].x),2.0) + pow((p->line[i].point[j].y-p->line[i].point[j-1].y),2.0)));
       line_length += segment_length;
       if(segment_length > max_segment_length) {
 	max_segment_length = segment_length;
@@ -889,7 +1059,7 @@ int msPolylineLabelPoint(shapeObj *p, pointObj *lp, int min_length, double *angl
   if((min_length != -1) && (total_length < min_length)) /* too short to label */
     return(MS_FAILURE);
 
-  // ok, now we know which line and which segment within that line
+  /* ok, now we know which line and which segment within that line */
   i = line_index;
   j = segment_index;
 
@@ -898,7 +1068,7 @@ int msPolylineLabelPoint(shapeObj *p, pointObj *lp, int min_length, double *angl
   lp->x = (p->line[i].point[j].x + p->line[i].point[j-1].x)/2.0;
   lp->y = (p->line[i].point[j].y + p->line[i].point[j-1].y)/2.0;
  
-  theta = asin(MS_ABS(p->line[i].point[j].x - p->line[i].point[j-1].x)/sqrt((pow((p->line[i].point[j].x - p->line[i].point[j-1].x),2) + pow((p->line[i].point[j].y - p->line[i].point[j-1].y),2))));
+  theta = asin(MS_ABS(p->line[i].point[j].x - p->line[i].point[j-1].x)/sqrt((pow((p->line[i].point[j].x - p->line[i].point[j-1].x),2.0) + pow((p->line[i].point[j].y - p->line[i].point[j-1].y),2.0))));
 
   if(p->line[i].point[j-1].x < p->line[i].point[j].x) { /* i.e. to the left */
     if(p->line[i].point[j-1].y < p->line[i].point[j].y) /* i.e. below */
@@ -929,6 +1099,10 @@ void msRectToFormattedString(rectObj *rect, char *format, char *buffer,
 void msPointToFormattedString(pointObj *point, const char *format, 
                               char *buffer, int buffer_length) 
 {
+#ifdef USE_POINT_Z_M
     snprintf(buffer, buffer_length, format, point->x, point->y, point->z, point->m);
+#else
+    snprintf(buffer, buffer_length, format, point->x, point->y);
+#endif
 }
 
