@@ -5,7 +5,7 @@
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,18 @@
  ******************************************************************************
  *
  * $Log: mapprimitive.h,v $
+ * Revision 1.18  2005/06/14 16:03:34  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.17  2005/04/21 15:09:28  julien
+ * Bug1244: Replace USE_SHAPE_Z_M by USE_POINT_Z_M
+ *
+ * Revision 1.16  2005/04/14 15:17:14  julien
+ * Bug 1244: Remove Z and M from point by default to gain performance.
+ *
+ * Revision 1.15  2005/02/18 03:06:46  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
  * Revision 1.14  2004/11/29 15:09:34  sean
  * hide itemObj and geotransformObj from swig, these are not ready for use
  *
@@ -51,8 +63,10 @@ typedef struct {
 typedef struct {
   double x;
   double y;
+#ifdef USE_POINT_Z_M
   double z;  
   double m;  
+#endif
 } pointObj;
 
 typedef struct {
@@ -79,7 +93,7 @@ typedef struct {
 #endif
 
   rectObj bounds;
-  int type; // MS_SHAPE_TYPE
+  int type; /* MS_SHAPE_TYPE */
   long index;
   int tileindex;
   int classindex;
@@ -108,8 +122,8 @@ typedef struct {
 typedef struct {
   int  need_geotransform;
   double rotation_angle;  
-  double geotransform[6];    // Pixel/line to georef.
-  double invgeotransform[6]; // georef to pixel/line  
+  double geotransform[6];    /* Pixel/line to georef. */
+  double invgeotransform[6]; /* georef to pixel/line */
 } geotransformObj;
 #endif
 
