@@ -5,7 +5,7 @@
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,21 @@
  ******************************************************************************
  *
  * $Log: mapproject.c,v $
+ * Revision 1.42  2005/06/14 16:03:34  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.41  2005/04/21 15:09:28  julien
+ * Bug1244: Replace USE_SHAPE_Z_M by USE_POINT_Z_M
+ *
+ * Revision 1.40  2005/04/14 15:17:14  julien
+ * Bug 1244: Remove Z and M from point by default to gain performance.
+ *
+ * Revision 1.39  2005/03/08 18:08:01  frank
+ * fixed msProjectPoint() to return MS_FAILURE in missing case: bug 1273
+ *
+ * Revision 1.38  2005/02/18 03:06:46  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
  * Revision 1.37  2004/11/16 21:56:18  dan
  * msGetEPSGProj() obsolete, replaced by msOWSGetEPSGProj() (bug 568)
  *
@@ -41,7 +56,7 @@
 #include "map.h"
 #include "mapproject.h"
 
-MS_CVSID("$Id: mapproject.c,v 1.37 2004/11/16 21:56:18 dan Exp $")
+MS_CVSID("$Id: mapproject.c,v 1.42 2005/06/14 16:03:34 dan Exp $")
 
 #ifdef USE_PROJ
 static int msTestNeedWrap( pointObj pt1, pointObj pt2, pointObj pt2_geo,
@@ -132,6 +147,9 @@ int msProjectPoint(projectionObj *in, projectionObj *out, pointObj *point)
           }
       }
 
+      if( p.u == HUGE_VAL || p.v == HUGE_VAL )
+          return MS_FAILURE;
+
       point->x = p.u;
       point->y = p.v;
   }
@@ -205,6 +223,10 @@ int msProjectRect(projectionObj *in, projectionObj *out, rectObj *rect)
 
   prj_point.x = rect->minx;
   prj_point.y = rect->miny;
+#ifdef USE_POINT_Z_M
+  prj_point.z = 0.0;
+  prj_point.m = 0.0;
+#endif /* USE_POINT_Z_M */
 
   msProjectGrowRect(in,out,&prj_rect,&rect_initialized,&prj_point,
                     &failure);
@@ -400,7 +422,7 @@ int msProjectionsDiffer( projectionObj *proj1, projectionObj *proj2 )
     if( proj1->numargs != proj2->numargs )
         return MS_TRUE;
 
-    // This test should be more rigerous.
+    /* This test should be more rigerous. */
     if( proj1->gt.need_geotransform 
         || proj2->gt.need_geotransform )
         return MS_TRUE;
@@ -597,7 +619,7 @@ char *msGetProjectionString(projectionObj *proj)
                 continue;
             if (pszProjString[0] == '\0')
             {
-                // no space at beginning of line
+                /* no space at beginning of line */
                 if (proj->args[i][0] != '+')
                     strcat(pszProjString, "+");
             }
