@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapprojhack.c,v 1.5 2004/10/21 04:30:55 frank Exp $
+ * $Id: mapprojhack.c,v 1.7 2005/02/18 03:06:47 dan Exp $
  *
  * Project:  MapServer
  * Purpose:  Implementation of utility functions related to the PROJ4 library
@@ -27,6 +27,12 @@
  **********************************************************************
  *
  * $Log: mapprojhack.c,v $
+ * Revision 1.7  2005/02/18 03:06:47  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.6  2005/01/24 14:30:14  frank
+ * Avoid pj_units init warnings.
+ *
  * Revision 1.5  2004/10/21 04:30:55  frank
  * Added standardized headers.  Added MS_CVSID().
  *
@@ -108,28 +114,28 @@ static int ConvertProjUnitStringToMS(const char *pszProjUnit)
 #ifdef USE_PROJ 
 struct PJ_UNITS
 pj_units_copy[] = {
-	"km",	"1000.",	"Kilometer",
-	"m",	"1.",		"Meter",
-	"dm",	"1/10",		"Decimeter",
-	"cm",	"1/100",	"Centimeter",
-	"mm",	"1/1000",	"Millimeter",
-	"kmi",	"1852.0",	"International Nautical Mile",
-	"in",	"0.0254",	"International Inch",
-	"ft",	"0.3048",	"International Foot",
-	"yd",	"0.9144",	"International Yard",
-	"mi",	"1609.344",	"International Statute Mile",
-	"fath",	"1.8288",	"International Fathom",
-	"ch",	"20.1168",	"International Chain",
-	"link",	"0.201168",	"International Link",
-	"us-in",	"1./39.37",	"U.S. Surveyor's Inch",
-	"us-ft",	"0.304800609601219",	"U.S. Surveyor's Foot",
-	"us-yd",	"0.914401828803658",	"U.S. Surveyor's Yard",
-	"us-ch",	"20.11684023368047",	"U.S. Surveyor's Chain",
-	"us-mi",	"1609.347218694437",	"U.S. Surveyor's Statute Mile",
-	"ind-yd",	"0.91439523",	"Indian Yard",
-	"ind-ft",	"0.30479841",	"Indian Foot",
-	"ind-ch",	"20.11669506",	"Indian Chain",
-(char *)0, (char *)0, (char *)0
+       {"km",  "1000.",        "Kilometer"},
+       {"m",   "1.",           "Meter"},
+       {"dm",  "1/10",         "Decimeter"},
+       {"cm",  "1/100",        "Centimeter"},
+       {"mm",  "1/1000",       "Millimeter"},
+       {"kmi", "1852.0",       "International Nautical Mile"},
+       {"in",  "0.0254",       "International Inch"},
+       {"ft",  "0.3048",       "International Foot"},
+       {"yd",  "0.9144",       "International Yard"},
+       {"mi",  "1609.344",     "International Statute Mile"},
+       {"fath",        "1.8288",       "International Fathom"},
+       {"ch",  "20.1168",      "International Chain"},
+       {"link",        "0.201168",     "International Link"},
+       {"us-in",       "1./39.37",     "U.S. Surveyor's Inch"},
+       {"us-ft",       "0.304800609601219",    "U.S. Surveyor's Foot"},
+       {"us-yd",       "0.914401828803658",    "U.S. Surveyor's Yard"},
+       {"us-ch",       "20.11684023368047",    "U.S. Surveyor's Chain"},
+       {"us-mi",       "1609.347218694437",    "U.S. Surveyor's Statute Mile"},
+       {"ind-yd",      "0.91439523",   "Indian Yard"},
+       {"ind-ft",      "0.30479841",   "Indian Foot"},
+       {"ind-ch",      "20.11669506",  "Indian Chain"},
+       {(char *)0, (char *)0, (char *)0}
 };
 #endif /* def USE_PROJ */
 
@@ -148,7 +154,7 @@ int GetMapserverUnitUsingProj(projectionObj *psProj)
         if (psProj->proj->is_latlong)
             return MS_DD;
 
-        //psProj->proj->to_meter;
+        /* psProj->proj->to_meter; */
         for (lu = pj_units_copy; lu->id ; ++lu)
         {
             if (strtod(lu->to_meter, NULL) == psProj->proj->to_meter)
