@@ -5,7 +5,7 @@
  * Author:   Frank Warmerdam, warmerdam@pobox.com
  *
  ******************************************************************************
- * Copyright (c) 1996-2001 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,12 @@
  ******************************************************************************
  *
  * $Log: mapresample.c,v $
+ * Revision 1.54.2.1  2005/07/21 22:22:19  frank
+ * bug 1372: fixed resampling multiband raw rasters
+ *
+ * Revision 1.54  2005/06/14 16:03:34  dan
+ * Updated copyright date to 2005
+ *
  * Revision 1.53  2004/10/21 04:30:56  frank
  * Added standardized headers.  Added MS_CVSID().
  *
@@ -211,7 +217,7 @@
 #include <assert.h>
 #include "mapresample.h"
 
-MS_CVSID("$Id: mapresample.c,v 1.53 2004/10/21 04:30:56 frank Exp $")
+MS_CVSID("$Id: mapresample.c,v 1.54.2.1 2005/07/21 22:22:19 frank Exp $")
 
 /* The amount of "extra" resolution we will load for our resampling source */
 #define RES_RATIO	2.0
@@ -301,7 +307,7 @@ msSimpleRasterResampler( imageObj *psSrcImage, colorObj offsite,
         for( nDstX = 0; nDstX < nDstXSize; nDstX++ )
         {
             int		nSrcX, nSrcY;
-            int		nValue;
+            int		nValue = 0;
 
             if( !panSuccess[nDstX] )
             {
@@ -357,51 +363,66 @@ msSimpleRasterResampler( imageObj *psSrcImage, colorObj offsite,
             }
             else if( MS_RENDERER_RAWDATA(psSrcImage->format) )
             {
-                if( psSrcImage->format->imagemode == MS_IMAGEMODE_INT16 )
+                int band;
+
+                for( band = 0; band < psSrcImage->format->bands; band++ )
                 {
-                    int	nValue;
+                    if( psSrcImage->format->imagemode == MS_IMAGEMODE_INT16 )
+                    {
+                        int	nValue;
 
-                    nValue = psSrcImage->img.raw_16bit[
-                        nSrcX + nSrcY * psSrcImage->width];
+                        nValue = psSrcImage->img.raw_16bit[
+                            nSrcX + nSrcY * psSrcImage->width 
+                            + band*psSrcImage->width*psSrcImage->height];
 
-                    if( nValue == offsite.red )
-                        continue;
+                        if( nValue == offsite.red )
+                            continue;
                     
-                    nSetPoints++;
-                    psDstImage->img.raw_16bit[
-                        nDstX + nDstY * psDstImage->width] = nValue;
-                }
-                else if( psSrcImage->format->imagemode == MS_IMAGEMODE_FLOAT32)
-                {
-                    float fValue;
+                        nSetPoints++;
+                        psDstImage->img.raw_16bit[
+                            nDstX + nDstY * psDstImage->width
+                            + band*psDstImage->width*psDstImage->height] 
+                            = nValue;
+                    }
+                    else if( psSrcImage->format->imagemode 
+                             == MS_IMAGEMODE_FLOAT32)
+                    {
+                        float fValue;
 
-                    fValue = psSrcImage->img.raw_float[
-                        nSrcX + nSrcY * psSrcImage->width];
+                        fValue = psSrcImage->img.raw_float[
+                            nSrcX + nSrcY * psSrcImage->width 
+                            + band*psSrcImage->width*psSrcImage->height];
 
-                    if( fValue == offsite.red )
-                        continue;
+                        if( fValue == offsite.red )
+                            continue;
                     
-                    nSetPoints++;
-                    psDstImage->img.raw_float[
-                        nDstX + nDstY * psDstImage->width] = fValue;
-                }
-                else if( psSrcImage->format->imagemode == MS_IMAGEMODE_BYTE)
-                {
-                    int nValue;
+                        nSetPoints++;
+                        psDstImage->img.raw_float[
+                            nDstX + nDstY * psDstImage->width
+                            + band*psDstImage->width*psDstImage->height] 
+                            = nValue;
+                    }
+                    else if(psSrcImage->format->imagemode == MS_IMAGEMODE_BYTE)
+                    {
+                        int nValue;
 
-                    nValue = psSrcImage->img.raw_byte[
-                        nSrcX + nSrcY * psSrcImage->width];
+                        nValue = psSrcImage->img.raw_byte[
+                            nSrcX + nSrcY * psSrcImage->width 
+                            + band*psSrcImage->width*psSrcImage->height];
 
-                    if( nValue == offsite.red )
-                        continue;
+                        if( nValue == offsite.red )
+                            continue;
                     
-                    nSetPoints++;
-                    psDstImage->img.raw_byte[nDstX + nDstY * psDstImage->width]
-                        = (unsigned char) nValue;
-                }
-                else
-                {
-                    assert( 0 );
+                        nSetPoints++;
+                        psDstImage->img.raw_byte[
+                            nDstX + nDstY * psDstImage->width
+                            + band*psDstImage->width*psDstImage->height]
+                            = (unsigned char) nValue;
+                    }
+                    else
+                    {
+                        assert( 0 );
+                    }
                 }
             }
         }
