@@ -4,9 +4,20 @@
  *
  * This file was originally based on the SWIG interface file mapscript.i
  *
- * $Id: mapscript_i.c,v 1.90 2004/11/12 18:43:01 assefa Exp $
+ * $Id: mapscript_i.c,v 1.93 2005/06/06 05:48:20 dan Exp $
  *
  * $Log: mapscript_i.c,v $
+ * Revision 1.93  2005/06/06 05:48:20  dan
+ * Added $layerObj->removeClass() (was already in SWIG MapScript)
+ * Added $layerObj->removeClass() to PHP MapScript (was already in SWIG
+ * MapScript, bug 1373)
+ *
+ * Revision 1.92  2005/04/21 15:09:29  julien
+ * Bug1244: Replace USE_SHAPE_Z_M by USE_POINT_Z_M
+ *
+ * Revision 1.91  2005/04/14 15:17:15  julien
+ * Bug 1244: Remove Z and M from point by default to gain performance.
+ *
  * Revision 1.90  2004/11/12 18:43:01  assefa
  * rectObj memebers are initialized ot -1 (Bug 788).
  *
@@ -593,6 +604,10 @@ int layerObj_moveClassDown(layerObj *self, int index)
     return msMoveClassDown(self, index);
 }
 
+classObj *layerObj_removeClass(layerObj *self, int index)
+{
+    return msRemoveClass(self, index);
+}
 
 /**********************************************************************
  * class extensions for classObj, always within the context of a layer
@@ -788,7 +803,9 @@ int lineObj_add(lineObj *self, pointObj *p) {
 
     self->point[self->numpoints].x = p->x;
     self->point[self->numpoints].y = p->y;
+#ifdef USE_POINT_Z_M
     self->point[self->numpoints].m = p->m;
+#endif
     self->numpoints++;
 
     return 0;
