@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: php_mapscript.h,v 1.47 2004/11/02 21:03:36 assefa Exp $
+ * $Id: php_mapscript.h,v 1.51 2005/06/14 16:03:36 dan Exp $
  *
  * Name:     php_mapscript.c
  * Project:  PHP/MapScript extension for MapServer.
@@ -8,7 +8,7 @@
  * Author:   Daniel Morissette, morissette@dmsolutions.ca
  *
  **********************************************************************
- * Copyright (c) 2000-2002, Daniel Morissette, DM Solutions Group
+ * Copyright (c) 2000-2005, Daniel Morissette, DM Solutions Group
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -30,6 +30,21 @@
  **********************************************************************
  *
  * $Log: php_mapscript.h,v $
+ * Revision 1.51  2005/06/14 16:03:36  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.50  2005/06/06 05:48:20  dan
+ * Added $layerObj->removeClass() (was already in SWIG MapScript)
+ * Added $layerObj->removeClass() to PHP MapScript (was already in SWIG
+ * MapScript, bug 1373)
+ *
+ * Revision 1.49  2005/05/27 15:00:12  dan
+ * New regex wrappers to solve issues with previous version (bug 1354)
+ *
+ * Revision 1.48  2005/05/25 21:02:06  dan
+ * Added a regex wrapper, allowing MapServer to build with PHP compiled
+ * with its builtin regex (bug 1354)
+ *
  * Revision 1.47  2004/11/02 21:03:36  assefa
  * Add a 2nd optional argument to LoadMapContext function (Bug 1023).
  *
@@ -81,7 +96,25 @@
 #ifndef _PHP_MAPSCRIPT_H_INCLUDED_
 #define _PHP_MAPSCRIPT_H_INCLUDED_
 
-//#include "map.h"
+#ifdef USE_PHP_REGEX
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <math.h>
+
+#if defined(_WIN32) && !defined(__CYGWIN__)
+#include <direct.h>
+#include <memory.h>
+#include <malloc.h>
+#else
+#include <unistd.h>
+#endif
+
+#include "map.h"
+#include "mapregex.h"
+#endif /* USE_PHP_REGEX */
+
+
 #include "maptemplate.h"
 #include "mapogcsld.h"
 
@@ -194,6 +227,7 @@ int             layerObj_applySLDURL(layerObj *self, char *sld, char *stylelayer
 char            *layerObj_generateSLD(layerObj *self);
 int             layerObj_moveClassUp(layerObj *self, int index);
 int             layerObj_moveClassDown(layerObj *self, int index);
+classObj        *layerObj_removeClass(layerObj *self, int index);
 
 
 classObj       *classObj_new(layerObj *layer, classObj *class);
