@@ -129,6 +129,7 @@ class fontSetObj(object):
         return "<C fontSetObj instance at %s>" % (self.this,)
     filename = property(_mapscript.fontSetObj_filename_get)
     numfonts = property(_mapscript.fontSetObj_numfonts_get)
+    fonts = property(_mapscript.fontSetObj_fonts_get, _mapscript.fontSetObj_fonts_set)
     def __init__(self, *args):
         newobj = _mapscript.new_fontSetObj(*args)
         self.this = newobj.this
@@ -299,11 +300,20 @@ class styleObj(object):
     color = property(_mapscript.styleObj_color_get, _mapscript.styleObj_color_set)
     backgroundcolor = property(_mapscript.styleObj_backgroundcolor_get, _mapscript.styleObj_backgroundcolor_set)
     outlinecolor = property(_mapscript.styleObj_outlinecolor_get, _mapscript.styleObj_outlinecolor_set)
+    mincolor = property(_mapscript.styleObj_mincolor_get, _mapscript.styleObj_mincolor_set)
+    maxcolor = property(_mapscript.styleObj_maxcolor_get, _mapscript.styleObj_maxcolor_set)
+    minvalue = property(_mapscript.styleObj_minvalue_get, _mapscript.styleObj_minvalue_set)
+    maxvalue = property(_mapscript.styleObj_maxvalue_get, _mapscript.styleObj_maxvalue_set)
+    rangeitem = property(_mapscript.styleObj_rangeitem_get, _mapscript.styleObj_rangeitem_set)
+    rangeitemindex = property(_mapscript.styleObj_rangeitemindex_get, _mapscript.styleObj_rangeitemindex_set)
     symbol = property(_mapscript.styleObj_symbol_get, _mapscript.styleObj_symbol_set)
     symbolname = property(_mapscript.styleObj_symbolname_get, _mapscript.styleObj_symbolname_set)
     size = property(_mapscript.styleObj_size_get, _mapscript.styleObj_size_set)
     minsize = property(_mapscript.styleObj_minsize_get, _mapscript.styleObj_minsize_set)
     maxsize = property(_mapscript.styleObj_maxsize_get, _mapscript.styleObj_maxsize_set)
+    width = property(_mapscript.styleObj_width_get, _mapscript.styleObj_width_set)
+    minwidth = property(_mapscript.styleObj_minwidth_get, _mapscript.styleObj_minwidth_set)
+    maxwidth = property(_mapscript.styleObj_maxwidth_get, _mapscript.styleObj_maxwidth_set)
     offsetx = property(_mapscript.styleObj_offsetx_get, _mapscript.styleObj_offsetx_set)
     offsety = property(_mapscript.styleObj_offsety_get, _mapscript.styleObj_offsety_set)
     angle = property(_mapscript.styleObj_angle_get, _mapscript.styleObj_angle_set)
@@ -928,6 +938,31 @@ class mapObjPtr(mapObj):
         self.__class__ = mapObj
 _mapscript.mapObj_swigregister(mapObjPtr)
 
+class SVGObj(object):
+    def __repr__(self):
+        return "<C SVGObj instance at %s>" % (self.this,)
+    map = property(_mapscript.SVGObj_map_get, _mapscript.SVGObj_map_set)
+    stream = property(_mapscript.SVGObj_stream_get, _mapscript.SVGObj_stream_set)
+    filename = property(_mapscript.SVGObj_filename_get, _mapscript.SVGObj_filename_set)
+    streamclosed = property(_mapscript.SVGObj_streamclosed_get, _mapscript.SVGObj_streamclosed_set)
+    compressed = property(_mapscript.SVGObj_compressed_get, _mapscript.SVGObj_compressed_set)
+    def __init__(self, *args):
+        newobj = _mapscript.new_SVGObj(*args)
+        self.this = newobj.this
+        self.thisown = 1
+        del newobj.thisown
+    def __del__(self, destroy=_mapscript.delete_SVGObj):
+        try:
+            if self.thisown: destroy(self)
+        except: pass
+
+class SVGObjPtr(SVGObj):
+    def __init__(self, this):
+        self.this = this
+        if not hasattr(self,"thisown"): self.thisown = 0
+        self.__class__ = SVGObj
+_mapscript.SVGObj_swigregister(SVGObjPtr)
+
 class imageObj(object):
     def __repr__(self):
         return "<C imageObj instance at %s>" % (self.this,)
@@ -943,6 +978,7 @@ class imageObj(object):
             if self.thisown: destroy(self)
         except: pass
     def save(*args): return _mapscript.imageObj_save(*args)
+    def getBytes(*args): return _mapscript.imageObj_getBytes(*args)
     def __init__(self, *args):
         newobj = _mapscript.new_imageObj(*args)
         self.this = newobj.this
@@ -963,6 +999,8 @@ msSaveImage = _mapscript.msSaveImage
 
 msFreeImage = _mapscript.msFreeImage
 
+msSetup = _mapscript.msSetup
+
 msCleanup = _mapscript.msCleanup
 class rectObj(object):
     def __repr__(self):
@@ -1036,8 +1074,6 @@ class pointObj(object):
         return "<C pointObj instance at %s>" % (self.this,)
     x = property(_mapscript.pointObj_x_get, _mapscript.pointObj_x_set)
     y = property(_mapscript.pointObj_y_get, _mapscript.pointObj_y_set)
-    z = property(_mapscript.pointObj_z_get, _mapscript.pointObj_z_set)
-    m = property(_mapscript.pointObj_m_get, _mapscript.pointObj_m_set)
     def __init__(self, *args):
         newobj = _mapscript.new_pointObj(*args)
         self.this = newobj.this
@@ -1123,6 +1159,8 @@ class shapeObj(object):
     def setBounds(*args): return _mapscript.shapeObj_setBounds(*args)
     def clone(*args): return _mapscript.shapeObj_clone(*args)
     def copy(*args): return _mapscript.shapeObj_copy(*args)
+    def buffer(*args): return _mapscript.shapeObj_buffer(*args)
+    def convexHull(*args): return _mapscript.shapeObj_convexHull(*args)
     def getValue(*args): return _mapscript.shapeObj_getValue(*args)
     def contains(*args): return _mapscript.shapeObj_contains(*args)
     def distanceToPoint(*args): return _mapscript.shapeObj_distanceToPoint(*args)
@@ -1391,6 +1429,7 @@ MS_WCSERR = _mapscript.MS_WCSERR
 MS_GEOSERR = _mapscript.MS_GEOSERR
 MS_RECTERR = _mapscript.MS_RECTERR
 MS_TIMEERR = _mapscript.MS_TIMEERR
+MS_GMLERR = _mapscript.MS_GMLERR
 MS_NUMERRORCODES = _mapscript.MS_NUMERRORCODES
 MESSAGELENGTH = _mapscript.MESSAGELENGTH
 ROUTINELENGTH = _mapscript.ROUTINELENGTH
