@@ -1,4 +1,4 @@
-# $Id: labeltest.py,v 1.2 2004/10/04 22:09:06 sean Exp $
+# $Id: labeltest.py,v 1.3 2005/01/30 02:18:27 sean Exp $
 #
 # Project:  MapServer
 # Purpose:  xUnit style Python mapscript tests of Map
@@ -46,7 +46,7 @@ class LabelCacheMemberTestCase(MapTestCase):
     def testCacheMemberText(self):
         """string attribute has been renamed to 'text' (bug 852)"""
         img = self.map.draw()
-        assert self.map.labelcache.numlabels == 2
+        assert self.map.labelcache.numlabels == 2, self.map.labelcache.numlabels
         label = self.map.nextLabel()
         assert label.text == 'A Point', label.text
         
