@@ -5,7 +5,7 @@
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  * Copyright (c) 1998 Todd C. Miller <Todd.Miller@courtesan.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -28,6 +28,15 @@
  ******************************************************************************
  *
  * $Log: mapstring.c,v $
+ * Revision 1.33  2005/06/14 16:03:34  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.32  2005/02/18 03:06:47  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.31  2005/02/04 14:06:46  sdlime
+ * Fixed bug 1207, thread safety issue in mapstring.c
+ *
  * Revision 1.30  2004/11/19 19:28:13  hobu
  * check for null inputs in strncasecmp and strcasecmp
  *
@@ -44,7 +53,7 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: mapstring.c,v 1.30 2004/11/19 19:28:13 hobu Exp $")
+MS_CVSID("$Id: mapstring.c,v 1.33 2005/06/14 16:03:34 dan Exp $")
 
 #include <ctype.h>
 
@@ -181,14 +190,14 @@ int strcasecmp(const char *s1, const char *s2)
 #endif
 
 char *long2string(long value) {
-  static char buffer[256]; // plenty of space
+  char buffer[256]; /* plenty of space */
 
   sprintf(buffer, "%ld", value);
   return(strdup(buffer));
 }
 
 char *double2string(double value) {
-  static char buffer[256]; // plenty of space
+  char buffer[256]; /* plenty of space */
 
   sprintf(buffer, "%g", value);
   return(strdup(buffer));
@@ -421,7 +430,7 @@ char *msBuildPath(char *pszReturnPath, const char *abs_path, const char *path)
       return NULL;
   }
 
-  // Check if path is absolute
+  /* Check if path is absolute */
   if((abs_path == NULL) || (abslen == 0) || 
        (path[0] == '\\') || (path[0] == '/') || 
          (pathlen > 1 && (path[1] == ':')))
@@ -430,7 +439,7 @@ char *msBuildPath(char *pszReturnPath, const char *abs_path, const char *path)
       return(pszReturnPath);
   }
 
-  // else return abs_path/path
+  /* else return abs_path/path */
   if((abs_path[abslen-1] == '/') || (abs_path[abslen-1] == '\\'))
   {
       sprintf(pszReturnPath, "%s%s", abs_path, path);
@@ -618,8 +627,8 @@ char *msEncodeHTMLEntities(const char *string)
     if(string == NULL)
         return NULL;
 
-    // Start with 100 extra chars for replacements... 
-    // should be good enough for most cases
+    /* Start with 100 extra chars for replacements...  */
+    /* should be good enough for most cases */
     buflen = strlen(string) + 100;
     newstring = (char*)malloc(buflen+1*sizeof(char*));
     if (newstring == NULL)
@@ -630,11 +639,11 @@ char *msEncodeHTMLEntities(const char *string)
 
     for(i=0, c=string; *c != '\0'; c++)
     {
-        // Need to realloc buffer?
+        /* Need to realloc buffer? */
         if (i+6 > buflen)
         {
-            // If we had to realloc then this string must contain several
-            // entities... so let's go with twice the previous buffer size
+            /* If we had to realloc then this string must contain several */
+            /* entities... so let's go with twice the previous buffer size */
             buflen *= 2;
             newstring = (char*)realloc(newstring, buflen+1*sizeof(char*));
             if (newstring == NULL)
@@ -663,7 +672,7 @@ char *msEncodeHTMLEntities(const char *string)
             i += 6;
             break;
           case '\'':
-            strcpy(newstring+i, "&#39;"); // changed from &apos; and i += 6 (bug 1040)
+            strcpy(newstring+i, "&#39;"); /* changed from &apos; and i += 6 (bug 1040) */
             i += 5;
             break;
           default:
@@ -699,7 +708,7 @@ void msDecodeHTMLEntities(const char *string)
 
     while((pszAmp = strchr(pszBuffer, '&')) != NULL)
     {
-        // Get the &...;
+        /* Get the &...; */
         strcpy(pszReplace, pszAmp);
         pszSemiColon = strchr(pszReplace, ';');
         if(pszSemiColon == NULL)
@@ -707,12 +716,12 @@ void msDecodeHTMLEntities(const char *string)
         else
             pszSemiColon++;
 
-        // Get everything after the &...;
+        /* Get everything after the &...; */
         strcpy(pszEnd, pszSemiColon);
 
         pszReplace[pszSemiColon-pszReplace] = '\0';
 
-        // Replace the &...;
+        /* Replace the &...; */
         if(strcasecmp(pszReplace, "&amp;") == 0)
         {
             pszBuffer[pszAmp - pszBuffer] = '&';
@@ -790,11 +799,11 @@ char *strcatalloc(char *pszDest, char *pszSrc)
    if (pszSrc == NULL)
       return pszDest;
 
-   // if destination is null, allocate memory
+   /* if destination is null, allocate memory */
    if (pszDest == NULL) {
       pszDest = strdup(pszSrc);
    }
-   else { // if dest is not null, reallocate memory
+   else { /* if dest is not null, reallocate memory */
       char *pszTemp;
 
       nLen = strlen(pszDest) + strlen(pszSrc);
@@ -835,7 +844,7 @@ char *msJoinStrings(char **array, int arrayLength, const char *delimeter)
     strcat(string, array[i]);
     strcat(string, delimeter);
   }
-  strcat(string, array[i]); // add last element, no delimiter
+  strcat(string, array[i]); /* add last element, no delimiter */
 
   return string;
 }
@@ -854,7 +863,7 @@ char *msHashString(const char *pszStr)
     pszOutBuf = (char*)malloc( (HASH_SIZE*2+1)*sizeof(char) );
     if (pszOutBuf == NULL)
     {
-        // msSetError(MS_MEMERR, ...);
+        /* msSetError(MS_MEMERR, ...); */
     }
 
     for(i=0; pszStr && pszStr[i]; i++)
