@@ -5,7 +5,7 @@
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,18 @@
  ******************************************************************************
  *
  * $Log: mapsymbol.c,v $
+ * Revision 1.83  2005/06/14 16:03:35  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.82  2005/05/10 13:18:25  dan
+ * Fixed several issues with writeSymbol() (bug 1344)
+ *
+ * Revision 1.81  2005/02/18 03:06:47  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.80  2005/01/28 06:16:54  sdlime
+ * Applied patch to make function prototypes ANSI C compliant. Thanks to Petter Reinholdtsen. This fixes but 1181.
+ *
  * Revision 1.79  2004/11/19 17:38:41  sean
  * final fix to bug 1074, image transparency preserved through msSymbolSetImageGD
  * and symbolObj::setImage.
@@ -62,10 +74,10 @@
 #include "mapcopy.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapsymbol.c,v 1.79 2004/11/19 17:38:41 sean Exp $")
+MS_CVSID("$Id: mapsymbol.c,v 1.83 2005/06/14 16:03:35 dan Exp $")
 
-extern int msyylex(); /* lexer globals */
-extern void msyyrestart();
+extern int msyylex(void); /* lexer globals */
+extern void msyyrestart(FILE *);
 extern double msyynumber;
 extern char *msyytext;
 extern int msyylineno;
@@ -74,6 +86,12 @@ extern FILE *msyyin;
 extern unsigned char PNGsig[8];
 extern unsigned char JPEGsig[3];
 
+/*
+** Symbol to string static arrays needed for writing map files.
+** Must be kept in sync with enumerations and defines found in map.h.
+*/
+extern char *msLabelPositions[]; /* Defined in mapfile.c */
+static char *msCapsJoinsCorners[7]={"NONE", "BEVEL", "BUTT", "MITER", "ROUND", "SQUARE", "TRIANGLE"};
 
 void freeImageCache(struct imageCacheObj *ic)
 {
@@ -147,7 +165,7 @@ void initSymbol(symbolObj *s)
   s->type = MS_SYMBOL_VECTOR;
   s->transparent = MS_FALSE;
   s->transparentcolor = 0;
-  s->stylelength = 0; // solid line
+  s->stylelength = 0; /* solid line */
   s->sizex = 0;
   s->sizey = 0;
   s->filled = MS_FALSE;
@@ -223,8 +241,8 @@ int loadSymbol(symbolObj *s, char *symbolpath)
       if((getInteger(&s->gap)) == -1) return(-1);
       break;
     case(POSITION):
-      // if((s->position = getSymbol(3, MS_UC,MS_CC,MS_LC)) == -1) 
-      //   return(-1);
+      /* if((s->position = getSymbol(3, MS_UC,MS_CC,MS_LC)) == -1)  */
+      /* return(-1); */
       if((s->position = getSymbol(9, MS_UL,MS_UC,MS_UR,MS_CL,MS_CC,MS_CR,MS_LL,MS_LC,MS_LR)) == -1) 
 	return(-1);
       break;
@@ -243,12 +261,12 @@ int loadSymbol(symbolObj *s, char *symbolpath)
 	return(-1);
       }
       
-      // Set imagepath
+      /* Set imagepath */
       s->imagepath = strdup(msyytext);
 
           
-      fread(bytes,8,1,stream); // read some bytes to try and identify the file
-      rewind(stream); // reset the image for the readers
+      fread(bytes,8,1,stream); /* read some bytes to try and identify the file */
+      rewind(stream); /* reset the image for the readers */
       if (memcmp(bytes,"GIF8",4)==0) 
       {
 #ifdef USE_GD_GIF
@@ -362,7 +380,7 @@ int loadSymbol(symbolObj *s, char *symbolpath)
       if((s->type = getSymbol(6,MS_SYMBOL_VECTOR,MS_SYMBOL_ELLIPSE,MS_SYMBOL_PIXMAP,MS_SYMBOL_SIMPLE,MS_SYMBOL_CARTOLINE,MS_SYMBOL_HATCH)) == -1)
 	return(-1);
 #endif
-      if(s->type == MS_TRUETYPE) // TrueType keyword is valid several place in map files and symbol files, this simplifies the lexer
+      if(s->type == MS_TRUETYPE) /* TrueType keyword is valid several place in map files and symbol files, this simplifies the lexer */
 	s->type = MS_SYMBOL_TRUETYPE;
       break;
     default:
@@ -385,31 +403,31 @@ void writeSymbol(symbolObj *s, FILE *stream)
   
   switch (s->type) {
   case(MS_SYMBOL_HATCH):
-    // todo!
+    /* todo! */
     break;
   case(MS_SYMBOL_PIXMAP):
     fprintf(stream, "    TYPE PIXMAP\n");
-    if(s->imagepath != NULL) fprintf(stream, "    IMAGE %s\n", s->imagepath);
+    if(s->imagepath != NULL) fprintf(stream, "    IMAGE \"%s\"\n", s->imagepath);
     fprintf(stream, "    TRANSPARENT %d\n", s->transparentcolor);
     break;
   case(MS_SYMBOL_TRUETYPE):
     fprintf(stream, "    TYPE TRUETYPE\n");
     if(s->antialias == MS_TRUE) fprintf(stream, "    ANTIALIAS TRUE\n");
-    fprintf(stream, "    CHARACTER %s\n", s->character);
+    if (s->character != NULL) fprintf(stream, "    CHARACTER \"%s\"\n", s->character);
     fprintf(stream, "    GAP %d\n", s->gap);
-    fprintf(stream, "    FONT %s\n", s->font);
-    fprintf(stream, "    POSITION %d\n", s->position);
+    if (s->font != NULL) fprintf(stream, "    FONT \"%s\"\n", s->font);
+    fprintf(stream, "    POSITION %s\n", msLabelPositions[s->position]);
     break;
   case(MS_SYMBOL_CARTOLINE):
     fprintf(stream, "    TYPE CARTOLINE\n");
-    fprintf(stream, "    LINECAP %d\n", s->linecap);
-    fprintf(stream, "    LINEJOIN %d\n", s->linejoin);
+    fprintf(stream, "    LINECAP %s\n", msCapsJoinsCorners[s->linecap]);
+    fprintf(stream, "    LINEJOIN %s\n", msCapsJoinsCorners[s->linejoin]);
     fprintf(stream, "    LINEJOINMAXSIZE %g\n", s->linejoinmaxsize);
     break;
   case(MS_SYMBOL_SIMPLE):
     break;
   case(MS_SYMBOL_ELLIPSE):
-    //default = MS_SYMBOL_VECTOR
+    /* default = MS_SYMBOL_VECTOR */
   default:
     if(s->type == MS_SYMBOL_ELLIPSE)
       fprintf(stream, "    TYPE ELLIPSE\n");
@@ -418,7 +436,7 @@ void writeSymbol(symbolObj *s, FILE *stream)
     
     if(s->filled == MS_TRUE) fprintf(stream, "    FILLED TRUE\n");
     
-    // POINT
+    /* POINT */
     if(s->numpoints != 0) {
       fprintf(stream, "    POINTS\n");
       for(i=0; i<s->numpoints; i++) {
@@ -426,7 +444,7 @@ void writeSymbol(symbolObj *s, FILE *stream)
       }
       fprintf(stream, "    END\n");
     }
-    // STYLE
+    /* STYLE */
     if(s->stylelength != 0) {
       fprintf(stream, "    STYLE\n     ");
       for(i=0; i<s->stylelength; i++) {
@@ -458,7 +476,7 @@ int msAddImageSymbol(symbolSetObj *symbolset, char *filename)
 
   if(!filename || strlen(filename) == 0) return(-1);
 
-  if(symbolset->numsymbols == MS_MAXSYMBOLS) { // no room
+  if(symbolset->numsymbols == MS_MAXSYMBOLS) { /* no room */
     msSetError(MS_SYMERR, "Maximum number of symbols reached.", "msAddImageSymbol()");
     return(-1);
   }
@@ -484,8 +502,8 @@ int msAddImageSymbol(symbolSetObj *symbolset, char *filename)
 
   initSymbol(&symbolset->symbol[i]);
 
-  fread(bytes,8,1,stream); // read some bytes to try and identify the file
-  rewind(stream); // reset the image for the readers
+  fread(bytes,8,1,stream); /* read some bytes to try and identify the file */
+  rewind(stream); /* reset the image for the readers */
   if (memcmp(bytes,"GIF8",4)==0) {
 #ifdef USE_GD_GIF
     symbolset->symbol[i].img = gdImageCreateFromGif(stream);
@@ -565,8 +583,8 @@ int msLoadSymbolSet(symbolSetObj *symbolset, mapObj *map)
 
 int loadSymbolSet(symbolSetObj *symbolset, mapObj *map)
 {
-//  char old_path[MS_PATH_LENGTH];
-//  char *symbol_path;
+/* char old_path[MS_PATH_LENGTH]; */
+/* char *symbol_path; */
   int status=1;
   char szPath[MS_MAXPATHLEN], *pszSymbolPath=NULL;
 
@@ -638,11 +656,11 @@ int msGetMarkerSize(symbolSetObj *symbolset, styleObj *style, int *width, int *h
   const char *font=NULL;
   int size;
 
-  *width = *height = 0; // set a starting value
+  *width = *height = 0; /* set a starting value */
 
-  if(style->symbol > symbolset->numsymbols || style->symbol < 0) return(MS_FAILURE); // no such symbol, 0 is OK
+  if(style->symbol > symbolset->numsymbols || style->symbol < 0) return(MS_FAILURE); /* no such symbol, 0 is OK */
 
-  if(style->symbol == 0) { // single point
+  if(style->symbol == 0) { /* single point */
     *width = 1;
     *height = 1;
     return(MS_SUCCESS);
@@ -734,7 +752,7 @@ int msAddNewSymbol(mapObj *map, char *name)
 
 
 int msAppendSymbol(symbolSetObj *symbolset, symbolObj *symbol) {
-    // Possible to add another symbol?
+    /* Possible to add another symbol? */
     if (symbolset->numsymbols == MS_MAXSYMBOLS) {
         msSetError(MS_CHILDERR, "Maximum number of symbols, %d, has been reached", "msAppendSymbol()", MS_MAXSYMBOLS);
         return -1;
@@ -773,7 +791,7 @@ int msSaveSymbolSetStream(symbolSetObj *symbolset, FILE *stream) {
         msSetError(MS_SYMERR, "Cannot save symbolset.", "msSaveSymbolSetStream()");
         return MS_FAILURE;
     }
-    // Don't ever write out the default symbol at index 0
+    /* Don't ever write out the default symbol at index 0 */
     for (i=1; i<symbolset->numsymbols; i++) {
         symbolset->symbol[i].inmapfile = MS_TRUE;
         writeSymbol(&(symbolset->symbol[i]), stream);
@@ -808,8 +826,8 @@ int msLoadImageSymbol(symbolObj *symbol, const char *filename) {
           return MS_FAILURE;
     }
 
-    fread(bytes,8,1,stream); // read some bytes to try and identify the file
-    rewind(stream); // reset the image for the readers
+    fread(bytes,8,1,stream); /* read some bytes to try and identify the file */
+    rewind(stream); /* reset the image for the readers */
     if (memcmp(bytes,"GIF8",4)==0) 
     {
 #ifdef USE_GD_GIF
@@ -949,7 +967,7 @@ int msCopySymbolSet(symbolSetObj *dst, symbolSetObj *src, mapObj *map)
 
   MS_COPYSTELEM(imagecachesize);
   
-  // I have a feeling that the code below is not quite right - Sean
+  /* I have a feeling that the code below is not quite right - Sean */
   /*copyProperty(&(dst->imagecache), &(src->imagecache),
                sizeof(struct imageCacheObj));
    */
