@@ -5,7 +5,7 @@
  * Author:   Frank Warmerdam <warmerdam@pobox.com>
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,15 @@
  ******************************************************************************
  *
  * $Log: mapthread.h,v $
+ * Revision 1.7  2005/06/14 16:03:35  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.6  2005/02/02 17:42:55  frank
+ * added TLOCK_POOL
+ *
+ * Revision 1.5  2005/01/28 06:16:54  sdlime
+ * Applied patch to make function prototypes ANSI C compliant. Thanks to Petter Reinholdtsen. This fixes but 1181.
+ *
  * Revision 1.4  2004/10/21 04:30:55  frank
  * Added standardized headers.  Added MS_CVSID().
  *
@@ -40,8 +49,8 @@ extern "C" {
 #endif
 
 #ifdef USE_THREAD
-void msThreadInit();
-int msGetThreadId();
+void msThreadInit(void);
+int msGetThreadId(void);
 void msAcquireLock(int);
 void msReleaseLock(int);
 #else
@@ -56,6 +65,7 @@ void msReleaseLock(int);
 #define TLOCK_ERROROBJ  3
 #define TLOCK_PROJ      4
 #define TLOCK_TTF       5
+#define TLOCK_POOL      6
 
 #define TLOCK_STATIC_MAX 20
 #define TLOCK_MAX       100
