@@ -5,7 +5,7 @@
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,15 @@
  ******************************************************************************
  *
  * $Log: maptime.c,v $
+ * Revision 1.17  2005/06/14 16:03:35  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.16  2005/05/27 15:00:12  dan
+ * New regex wrappers to solve issues with previous version (bug 1354)
+ *
+ * Revision 1.15  2005/02/18 03:06:47  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
  * Revision 1.14  2004/10/21 10:54:17  assefa
  * Add postgis date_trunc support.
  *
@@ -43,11 +52,11 @@
 #include "maptime.h"
 #include "maperror.h"
 
-MS_CVSID("$Id: maptime.c,v 1.14 2004/10/21 10:54:17 assefa Exp $")
+MS_CVSID("$Id: maptime.c,v 1.17 2005/06/14 16:03:35 dan Exp $")
 
 typedef struct {
   char pattern[64];
-  regex_t *regex;
+  ms_regex_t *regex;
   char  format[32];  
   char userformat[32];
   MS_TIME_RESOLUTION resolution;
@@ -76,7 +85,7 @@ int ms_num_limited_pattern;
 
 void msTimeInit(struct tm *time)
 {
-  // set all members to zero
+  /* set all members to zero */
   time->tm_sec = 0;
   time->tm_min = 0;
   time->tm_hour = 0;
@@ -101,13 +110,13 @@ int msDateCompare(struct tm *time1, struct tm *time2)
   int result;
 
   if((result = compareIntVals(time1->tm_year, time2->tm_year)) != 0)
-    return result; // not equal based on year
+    return result; /* not equal based on year */
   else if((result = compareIntVals(time1->tm_mon, time2->tm_mon)) != 0)
-    return result; // not equal based on month
+    return result; /* not equal based on month */
   else if((result = compareIntVals(time1->tm_mday, time2->tm_mday)) != 0)
-    return result; // not equal based on day of month
+    return result; /* not equal based on day of month */
   
-  return(0); // must be equal
+  return(0); /* must be equal */
 }
 
 int msTimeCompare(struct tm *time1, struct tm *time2)
@@ -115,19 +124,19 @@ int msTimeCompare(struct tm *time1, struct tm *time2)
   int result;
 
   if((result = compareIntVals(time1->tm_year, time2->tm_year)) != 0)
-    return result; // not equal based on year
+    return result; /* not equal based on year */
   else if((result = compareIntVals(time1->tm_mon, time2->tm_mon)) != 0)
-    return result; // not equal based on month
+    return result; /* not equal based on month */
   else if((result = compareIntVals(time1->tm_mday, time2->tm_mday)) != 0)
-    return result; // not equal based on day of month
+    return result; /* not equal based on day of month */
   else if((result = compareIntVals(time1->tm_hour, time2->tm_hour)) != 0)
-    return result; // not equal based on hour
+    return result; /* not equal based on hour */
   else if((result = compareIntVals(time1->tm_min, time2->tm_min)) != 0)
-    return result; // not equal based on minute
+    return result; /* not equal based on minute */
   else if((result = compareIntVals(time1->tm_sec, time2->tm_sec)) != 0)
-    return result; // not equal based on second
+    return result; /* not equal based on second */
 
-  return(0); // must be equal
+  return(0); /* must be equal */
 }
 
 #if defined(_WIN32) && !defined(__CYGWIN__)
@@ -143,14 +152,14 @@ void msGettimeofday(struct mstimeval* tp, void* tzp)
 #endif
 
 #if defined(_WIN32) && !defined(__CYGWIN__)
-// we need to provide our own prototype on windows.
+/* we need to provide our own prototype on windows. */
 char *strptime( const char *buf, const char *format, struct tm *timeptr );
 #endif
 
 char *msStrptime(const char *s, const char *format, struct tm *tm)
 {
 #if defined(_WIN32) && !defined(__CYGWIN__)
-    //we are now using a local strptime found strptime.c
+    /* we are now using a local strptime found strptime.c */
     return strptime(s, format, tm);
 #else
     /* Use system strptime() on non-windows systems */
@@ -166,8 +175,8 @@ int msTimeMatchPattern(char *timestring, char *timeformat)
 {
     int i =-1;
 
-    //match the pattern format first and then check if the time string 
-    //matchs the pattern. If it is the case retrurn the MS_TRUE
+    /* match the pattern format first and then check if the time string  */
+    /* matchs the pattern. If it is the case retrurn the MS_TRUE */
     for (i=0; i<MS_NUMTIMEFORMATS; i++)
     {
         if (strcasecmp(ms_timeFormats[i].userformat, timeformat) == 0)
@@ -178,11 +187,11 @@ int msTimeMatchPattern(char *timestring, char *timeformat)
     {
         if(!ms_timeFormats[i].regex)
         {
-            ms_timeFormats[i].regex = (regex_t *) malloc(sizeof(regex_t));
-            regcomp(ms_timeFormats[i].regex, 
-                    ms_timeFormats[i].pattern, REG_EXTENDED|REG_NOSUB);
+            ms_timeFormats[i].regex = (ms_regex_t *) malloc(sizeof(ms_regex_t));
+            ms_regcomp(ms_timeFormats[i].regex, 
+                    ms_timeFormats[i].pattern, MS_REG_EXTENDED|MS_REG_NOSUB);
         }
-        if (regexec(ms_timeFormats[i].regex, timestring, 0,NULL, 0) == 0)
+        if (ms_regexec(ms_timeFormats[i].regex, timestring, 0,NULL, 0) == 0)
           return MS_TRUE;
         
     }
@@ -206,7 +215,7 @@ void msSetLimitedPattersToUse(char *patternstring)
 
     limitedpatternindice = (int *)malloc(sizeof(int)*MS_NUMTIMEFORMATS);
     
-    //free previous setting
+    /* free previous setting */
     msUnsetLimitedPatternToUse();
 
     if (patternstring)
@@ -250,8 +259,8 @@ int msParseTime(const char *string, struct tm *tm) {
   int i, indice = 0;
   int num_patterns = 0;
   
-  //if limited patterns are set, use then. Else use all the
-  //patterns defined
+  /* if limited patterns are set, use then. Else use all the */
+  /* patterns defined */
   if (ms_limited_pattern &&  ms_num_limited_pattern > 0)
     num_patterns = ms_num_limited_pattern;
   else
@@ -263,17 +272,17 @@ int msParseTime(const char *string, struct tm *tm) {
       else
         indice = i;
 
-      if(!ms_timeFormats[indice].regex) { // compile the expression
-      ms_timeFormats[indice].regex = (regex_t *) malloc(sizeof(regex_t)); 
-      if(regcomp(ms_timeFormats[indice].regex, ms_timeFormats[indice].pattern, REG_EXTENDED|REG_NOSUB) != 0) {
+      if(!ms_timeFormats[indice].regex) { /* compile the expression */
+      ms_timeFormats[indice].regex = (ms_regex_t *) malloc(sizeof(ms_regex_t)); 
+      if(ms_regcomp(ms_timeFormats[indice].regex, ms_timeFormats[indice].pattern, MS_REG_EXTENDED|MS_REG_NOSUB) != 0) {
 	msSetError(MS_REGEXERR, "Failed to compile expression (%s).", "msParseTime()", ms_timeFormats[indice].pattern);
 	return(MS_FALSE);
       }
     }  
 
-    // test the expression against the string
-    if(regexec(ms_timeFormats[indice].regex, string, 0, NULL, 0) == 0) 
-    { // match   
+    /* test the expression against the string */
+    if(ms_regexec(ms_timeFormats[indice].regex, string, 0, NULL, 0) == 0) 
+    { /* match    */
         msStrptime(string, ms_timeFormats[indice].format, tm);
         return(MS_TRUE);
     }
@@ -297,17 +306,17 @@ int msTimeGetResolution(const char *timestring)
      {
          if(!ms_timeFormats[i].regex) 
          {
-             ms_timeFormats[i].regex = (regex_t *) malloc(sizeof(regex_t));                
-             if(regcomp(ms_timeFormats[i].regex, ms_timeFormats[i].pattern, 
-                        REG_EXTENDED|REG_NOSUB) != 0) 
+             ms_timeFormats[i].regex = (ms_regex_t *) malloc(sizeof(ms_regex_t));                
+             if(ms_regcomp(ms_timeFormats[i].regex, ms_timeFormats[i].pattern, 
+                        MS_REG_EXTENDED|MS_REG_NOSUB) != 0) 
              {
                  msSetError(MS_REGEXERR, "Failed to compile expression (%s).", "msParseTime()", ms_timeFormats[i].pattern);
                  return -1;
              }
          }
-         // test the expression against the string
-         if(regexec(ms_timeFormats[i].regex, timestring, 0, NULL, 0) == 0) 
-         { // match   
+         /* test the expression against the string */
+         if(ms_regexec(ms_timeFormats[i].regex, timestring, 0, NULL, 0) == 0) 
+         { /* match    */
              return ms_timeFormats[i].resolution;
          }
      }
