@@ -1,12 +1,12 @@
 /******************************************************************************
- * $Id: maptime.h,v 1.8 2004/10/21 10:54:17 assefa Exp $
+ * $Id: maptime.h,v 1.11 2005/06/14 16:03:35 dan Exp $
  *
  * Project:  MapServer
  * Purpose:  Time processing related declarations.
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,6 +28,15 @@
  ******************************************************************************
  *
  * $Log: maptime.h,v $
+ * Revision 1.11  2005/06/14 16:03:35  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.10  2005/02/18 03:06:47  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.9  2005/01/28 06:16:54  sdlime
+ * Applied patch to make function prototypes ANSI C compliant. Thanks to Petter Reinholdtsen. This fixes but 1181.
+ *
  * Revision 1.8  2004/10/21 10:54:17  assefa
  * Add postgis date_trunc support.
  *
@@ -68,7 +77,7 @@ typedef enum
     TIME_RESOLUTION_YEAR =7
 }MS_TIME_RESOLUTION;
 
-// function prototypes
+/* function prototypes */
 void msTimeInit(struct tm *time);
 int msDateCompare(struct tm *time1, struct tm *time2);
 int msTimeCompare(struct tm *time1, struct tm *time2);
@@ -76,7 +85,7 @@ char *msStrptime(const char *s, const char *format, struct tm *tm);
 int msParseTime(const char *string, struct tm *tm);
 int msTimeMatchPattern(char *timestring, char *pattern);
 void msSetLimitedPattersToUse(char *patternstring);
-void msUnsetLimitedPatternToUse();
+void msUnsetLimitedPatternToUse(void);
 int msTimeGetResolution(const char *timestring);
 
 #endif /* MAPTIME_H */
