@@ -5,7 +5,7 @@
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,65 @@
  ******************************************************************************
  *
  * $Log: maputil.c,v $
+ * Revision 1.182.2.2  2005/07/27 19:01:10  sean
+ * Fixes in msTmpFile() to ensure counter actually increments and that forced basenames work again (bug 1312)
+ *
+ * Revision 1.182.2.1  2005/06/24 13:39:45  assefa
+ * in function msTmpFile : reinitialize always the sting tmpId. There was
+ * a problem with this string not being initialized when the function was
+ * call more than once : Bug 1312.
+ *
+ * Revision 1.182  2005/06/14 16:03:35  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.181  2005/05/27 15:00:12  dan
+ * New regex wrappers to solve issues with previous version (bug 1354)
+ *
+ * Revision 1.180  2005/04/21 15:09:28  julien
+ * Bug1244: Replace USE_SHAPE_Z_M by USE_POINT_Z_M
+ *
+ * Revision 1.179  2005/04/14 15:17:14  julien
+ * Bug 1244: Remove Z and M from point by default to gain performance.
+ *
+ * Revision 1.178  2005/04/12 23:56:44  sean
+ * change tmpId string to be non static in msTmpFile() (bug 1312).
+ *
+ * Revision 1.177  2005/04/07 17:23:16  assefa
+ * Remove #ifdef USE_SVG. It was added during development.
+ *
+ * Revision 1.176  2005/02/18 05:07:26  frank
+ * fixed return from msForceTmpFileBase
+ *
+ * Revision 1.175  2005/02/18 03:06:47  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.174  2005/02/09 21:51:17  frank
+ * added msForceTmpFileBase and -tmpbase mapserv switch
+ *
+ * Revision 1.173  2005/02/09 20:43:50  assefa
+ * Add SVG utility function msSaveImagetoFpSVG.
+ *
+ * Revision 1.172  2005/02/08 17:34:46  sean
+ * call msThreadInit in msSetup (bug 1223).
+ *
+ * Revision 1.171  2005/02/06 19:31:32  sean
+ * msSetup() function calls gdFontCacheSetup().  msSetup should be called immediately on mapscript module import to avoid race condition with font cache (bug 1203).
+ *
+ * Revision 1.170  2005/02/03 00:06:57  assefa
+ * Add SVG function prototypes and related calls.
+ *
+ * Revision 1.169  2005/01/28 06:16:54  sdlime
+ * Applied patch to make function prototypes ANSI C compliant. Thanks to Petter Reinholdtsen. This fixes but 1181.
+ *
+ * Revision 1.168  2005/01/20 01:59:19  sean
+ * protect against labelitemindex < 0 take 2, > 0 changed to >= 0
+ *
+ * Revision 1.167  2005/01/20 00:53:47  sean
+ * protect against labelitemindex < 0
+ *
+ * Revision 1.166  2004/12/07 21:33:28  sean
+ * protect against getting a shape value when values is NULL in msShapeGetAnnotation().  also implement setValue for mapscript::shapeObj so we can define labels for inline features (bug 1114)
+ *
  * Revision 1.165.2.1  2004/12/07 21:10:20  sean
  * protect against getting a shape value when values is NULL in msShapeGetAnnotation().  also implement setValue for mapscript::shapeObj so we can define labels for inline features (bug 1114)
  *
@@ -63,13 +122,13 @@
 #include <process.h>
 #endif
 
-MS_CVSID("$Id: maputil.c,v 1.165.2.1 2004/12/07 21:10:20 sean Exp $")
+MS_CVSID("$Id: maputil.c,v 1.182.2.2 2005/07/27 19:01:10 sean Exp $")
 
-extern int msyyparse();
-extern int msyylex();
+extern int msyyparse(void);
+extern int msyylex(void);
 extern char *msyytext;
 
-extern int msyyresult; // result of parsing, true/false
+extern int msyyresult; /* result of parsing, true/false */
 extern int msyystate;
 extern char *msyystring;
 
@@ -78,7 +137,7 @@ extern char *msyystring;
  * Used to get red, green, blue integers separately based upon the color index
  */
 int getRgbColor(mapObj *map,int i,int *r,int *g,int *b) {
-// check index range
+/* check index range */
     int status=1;
     *r=*g=*b=-1;
     if ((i > 0 ) && (i <= map->palette.numcolors) ) {
@@ -96,13 +155,13 @@ static int searchContextForTag(mapObj *map, char **ltags, char *tag, char *conte
 
   if(!context) return MS_FAILURE;
 
-  // printf("\tin searchContextForTag, searching %s for %s\n", context, tag);
+  /* printf("\tin searchContextForTag, searching %s for %s\n", context, tag); */
 
-  if(strstr(context, tag) != NULL) return MS_SUCCESS; // found the tag
+  if(strstr(context, tag) != NULL) return MS_SUCCESS; /* found the tag */
 
-  // check referenced layers for the tag too
+  /* check referenced layers for the tag too */
   for(i=0; i<map->numlayers; i++) {
-    if(strstr(context, ltags[i]) != NULL) { // need to check this layer
+    if(strstr(context, ltags[i]) != NULL) { /* need to check this layer */
       if(requires == MS_TRUE) {
         if(searchContextForTag(map, ltags, tag, map->layers[i].requires, MS_TRUE) == MS_SUCCESS) return MS_SUCCESS;
       } else {
@@ -135,9 +194,9 @@ int msValidateContexts(mapObj *map)
     }
   }
 
-  // check each layer's REQUIRES and LABELREQUIRES parameters
+  /* check each layer's REQUIRES and LABELREQUIRES parameters */
   for(i=0; i<map->numlayers; i++) { 
-    // printf("working on layer %s, looking for references to %s\n", map->layers[i].name, ltags[i]);
+    /* printf("working on layer %s, looking for references to %s\n", map->layers[i].name, ltags[i]); */
     if(searchContextForTag(map, ltags, ltags[i], map->layers[i].requires, MS_TRUE) == MS_SUCCESS) {
       msSetError(MS_PARSEERR, "Recursion error found for REQUIRES parameter for layer %s.", "msValidateContexts", map->layers[i].name);
       status = MS_FAILURE;
@@ -150,7 +209,7 @@ int msValidateContexts(mapObj *map)
     }
   }
 
-  // clean up
+  /* clean up */
   msFreeCharArray(ltags, map->numlayers);
 
   return status;
@@ -161,20 +220,20 @@ int msEvalContext(mapObj *map, layerObj *layer, char *context)
   int i, status;
   char *tmpstr1=NULL, *tmpstr2=NULL;
   int raster=MS_FALSE, visible;
-  int expresult;       // result of expression parsing operation
+  int expresult;       /* result of expression parsing operation */
 
-  if(!context) return(MS_TRUE); // no context requirements
+  if(!context) return(MS_TRUE); /* no context requirements */
 
   tmpstr1 = strdup(context);
 
-  for(i=0; i<map->numlayers; i++) { // step through all the layers
-    if(layer->index == i) continue; // skip the layer in question
-    // Layer without name cannot be used in contexts
+  for(i=0; i<map->numlayers; i++) { /* step through all the layers */
+    if(layer->index == i) continue; /* skip the layer in question */
+    /* Layer without name cannot be used in contexts */
     if (map->layers[i].name == NULL) continue;
     visible = msLayerIsVisible(map, &(map->layers[i]));
 
     if(map->layers[i].type == MS_LAYER_RASTER && visible)
-      raster = MS_TRUE; // there are raster layers ON/DEFAULT
+      raster = MS_TRUE; /* there are raster layers ON/DEFAULT */
 
     if(strstr(tmpstr1, map->layers[i].name)) {
       tmpstr2 = (char *)malloc(sizeof(char)*strlen(map->layers[i].name) + 3);
@@ -189,7 +248,7 @@ int msEvalContext(mapObj *map, layerObj *layer, char *context)
     }
   }
 
-  // special option to catch raster (i.e. background) layers, easier than having to list all layers individually
+  /* special option to catch raster (i.e. background) layers, easier than having to list all layers individually */
   if(raster == MS_TRUE)
     tmpstr1 = gsub(tmpstr1, "[raster]", "1");
   else
@@ -206,7 +265,7 @@ int msEvalContext(mapObj *map, layerObj *layer, char *context)
   {
     msSetError(MS_PARSEERR, "Failed to parse context",
                             "msEvalContext");
-    return MS_FALSE; // error in parse
+    return MS_FALSE; /* error in parse */
   }
   return expresult;
 }
@@ -225,9 +284,9 @@ int msEvalExpression(expressionObj *expression, int itemindex, char **items, int
   int i;
   char *tmpstr=NULL;
   int status;
-  int expresult;  // result of logical expression parsing operation
+  int expresult;  /* result of logical expression parsing operation */
   
-  if(!expression->string) return(MS_TRUE); // empty expressions are ALWAYS true
+  if(!expression->string) return(MS_TRUE); /* empty expressions are ALWAYS true */
 
   switch(expression->type) {
   case(MS_STRING):
@@ -239,7 +298,7 @@ int msEvalExpression(expressionObj *expression, int itemindex, char **items, int
       msSetError(MS_MISCERR, "Invalid item index.", "msEvalExpression()");
       return(MS_FALSE);
     }
-    if(strcmp(expression->string, items[itemindex]) == 0) return(MS_TRUE); // got a match
+    if(strcmp(expression->string, items[itemindex]) == 0) return(MS_TRUE); /* got a match */
     break;
   case(MS_EXPRESSION):
     tmpstr = strdup(expression->string);
@@ -249,7 +308,7 @@ int msEvalExpression(expressionObj *expression, int itemindex, char **items, int
 
     msAcquireLock( TLOCK_PARSER );
     msyystate = 4;
-    msyystring = tmpstr; // set lexer state to EXPRESSION_STRING
+    msyystring = tmpstr; /* set lexer state to EXPRESSION_STRING */
     status = msyyparse();
     expresult = msyyresult;
     msReleaseLock( TLOCK_PARSER );
@@ -276,14 +335,14 @@ int msEvalExpression(expressionObj *expression, int itemindex, char **items, int
     }
 
     if(!expression->compiled) {
-      if(regcomp(&(expression->regex), expression->string, REG_EXTENDED|REG_NOSUB) != 0) { // compile the expression
+      if(ms_regcomp(&(expression->regex), expression->string, MS_REG_EXTENDED|MS_REG_NOSUB) != 0) { /* compile the expression */
         msSetError(MS_REGEXERR, "Invalid regular expression.", "msEvalExpression()");
         return(MS_FALSE);
       }
       expression->compiled = MS_TRUE;
     }
 
-    if(regexec(&(expression->regex), items[itemindex], 0, NULL, 0) == 0) return(MS_TRUE); // got a match
+    if(ms_regexec(&(expression->regex), items[itemindex], 0, NULL, 0) == 0) return(MS_TRUE); /* got a match */
     break;
   }
 
@@ -308,16 +367,16 @@ int msShapeGetClass(layerObj *layer, shapeObj *shape, double scale)
 {
   int i;
 
-  // INLINE features do not work with expressions, allow the classindex
-  // value set prior to calling this function to carry through.
+  /* INLINE features do not work with expressions, allow the classindex */
+  /* value set prior to calling this function to carry through. */
   if(layer->connectiontype == MS_INLINE) {
     if(shape->classindex < 0 || shape->classindex >= layer->numclasses) return(-1);
 
-    if(scale > 0) {  // verify scale here
+    if(scale > 0) {  /* verify scale here */
       if((layer->class[shape->classindex].maxscale > 0) && (scale > layer->class[shape->classindex].maxscale))
-        return(-1); // can skip this feature
+        return(-1); /* can skip this feature */
       if((layer->class[shape->classindex].minscale > 0) && (scale <= layer->class[shape->classindex].minscale))
-        return(-1); // can skip this feature
+        return(-1); /* can skip this feature */
     }
 
     return(shape->classindex);
@@ -325,18 +384,18 @@ int msShapeGetClass(layerObj *layer, shapeObj *shape, double scale)
 
   for(i=0; i<layer->numclasses; i++) {
     
-    if(scale > 0) {  // verify scale here 
+    if(scale > 0) {  /* verify scale here  */
       if((layer->class[i].maxscale > 0) && (scale > layer->class[i].maxscale))
-        continue; // can skip this one, next class
+        continue; /* can skip this one, next class */
       if((layer->class[i].minscale > 0) && (scale <= layer->class[i].minscale))
-        continue; // can skip this one, next class
+        continue; /* can skip this one, next class */
     }
 
     if(layer->class[i].status != MS_DELETE && msEvalExpression(&(layer->class[i].expression), layer->classitemindex, shape->values, layer->numitems) == MS_TRUE)
       return(i);
   }
 
-  return(-1); // no match
+  return(-1); /* no match */
 }
 
 char *msShapeGetAnnotation(layerObj *layer, shapeObj *shape)
@@ -344,7 +403,7 @@ char *msShapeGetAnnotation(layerObj *layer, shapeObj *shape)
   int i;
   char *tmpstr=NULL;
 
-  if(layer->class[shape->classindex].text.string) { // test for global label first
+  if(layer->class[shape->classindex].text.string) { /* test for global label first */
     tmpstr = strdup(layer->class[shape->classindex].text.string);
     switch(layer->class[shape->classindex].text.type) {
     case(MS_STRING):
@@ -357,7 +416,7 @@ char *msShapeGetAnnotation(layerObj *layer, shapeObj *shape)
       break;
     }
   } else {
-    if (shape->values)
+    if (shape->values && layer->labelitemindex >= 0)
         tmpstr = strdup(shape->values[layer->labelitemindex]);
   }
 
@@ -394,7 +453,7 @@ double msAdjustExtent(rectObj *rect, int width, int height)
   if(cellsize <= 0) /* avoid division by zero errors */
     return(0);
 
-  ox = MS_MAX((width - (rect->maxx - rect->minx)/cellsize)/2,0); // these were width-1 and height-1
+  ox = MS_MAX((width - (rect->maxx - rect->minx)/cellsize)/2,0); /* these were width-1 and height-1 */
   oy = MS_MAX((height - (rect->maxy - rect->miny)/cellsize)/2,0);
 
   rect->minx = rect->minx - ox*cellsize;
@@ -414,25 +473,25 @@ int msConstrainExtent(rectObj *bounds, rectObj *rect, double overlay)
 {
   double offset=0;
 
-  // check left edge, and if necessary the right edge of bounds
+  /* check left edge, and if necessary the right edge of bounds */
   if(rect->maxx <= bounds->minx) {
     offset = overlay*(rect->maxx - rect->minx);
-    rect->minx += offset; // shift right
+    rect->minx += offset; /* shift right */
     rect->maxx += offset;
   } else if(rect->minx >= bounds->maxx) {
     offset = overlay*(rect->maxx - rect->minx);
-    rect->minx -= offset; // shift left
+    rect->minx -= offset; /* shift left */
     rect->maxx -= offset;
   }
 
-  // check top edge, and if necessary the bottom edge of bounds
+  /* check top edge, and if necessary the bottom edge of bounds */
   if(rect->maxy <= bounds->miny) {
     offset = overlay*(rect->maxy - rect->miny);
-    rect->miny -= offset; // shift down
+    rect->miny -= offset; /* shift down */
     rect->maxy -= offset;
   } else if(rect->miny >= bounds->maxy) {
     offset = overlay*(rect->maxy - rect->miny);
-    rect->miny += offset; // shift up
+    rect->miny += offset; /* shift up */
     rect->maxy += offset;
   }
 
@@ -503,6 +562,16 @@ int msSaveImage(mapObj *map, imageObj *img, char *filename)
               nReturnVal = msSaveImagePDF(img, filename);
         }
 #endif
+        else if(MS_DRIVER_SVG(img->format) )
+        {
+            if (map != NULL && filename != NULL )
+              nReturnVal = msSaveImageSVG(img, 
+                                          msBuildPath(szPath, map->mappath, 
+                                                      filename));
+            else
+              nReturnVal = msSaveImageSVG(img, filename);
+        }
+
         else
             msSetError(MS_MISCERR, "Unknown image type", 
                        "msSaveImage()"); 
@@ -533,6 +602,8 @@ void msFreeImage(imageObj *image)
         else if( MS_RENDERER_PDF(image->format) )
             msFreeImagePDF(image);
 #endif
+        else if( MS_RENDERER_SVG(image->format) )
+            msFreeImageSVG(image);
         else
             msSetError(MS_MISCERR, "Unknown image type", 
                        "msFreeImage()"); 
@@ -611,6 +682,7 @@ int *msGetLayersIndexByGroup(mapObj *map, char *groupname, int *pnCount)
 /************************************************************************/
 pointObj *msGetPointUsingMeasure(shapeObj *shape, double m)
 {
+#ifdef USE_POINT_Z_M
     pointObj    *point = NULL;
     lineObj     line;
     double      dfMin = 0;
@@ -657,13 +729,13 @@ pointObj *msGetPointUsingMeasure(shapeObj *shape, double m)
 /* -------------------------------------------------------------------- */
 /*      get the previous node xy values.                                */
 /* -------------------------------------------------------------------- */
-                        if (j > 0) //not the first point of the line
+                        if (j > 0) /* not the first point of the line */
                         {
                             dfFirstPointX = line.point[j-1].x;
                             dfFirstPointY = line.point[j-1].y;
                             dfFirstPointM = line.point[j-1].m;
                         }
-                        else // get last point of previous line
+                        else /* get last point of previous line */
                         {
                             dfFirstPointX = shape->line[i-1].point[0].x;
                             dfFirstPointY = shape->line[i-1].point[0].y;
@@ -699,6 +771,12 @@ pointObj *msGetPointUsingMeasure(shapeObj *shape, double m)
     }
 
     return NULL;
+#else
+    msSetError(MS_MISCERR, 
+               "The \"m\" parameter for points is unavailable in your build.",
+               "msGetPointUsingMeasure()");
+    return NULL;
+#endif /* USE_POINT_Z_M */
 }
 
 
@@ -816,7 +894,9 @@ pointObj *msIntersectionPointLine(pointObj *p, pointObj *a, pointObj *b)
             result->x = a->x + r*(b->x - a->x);
             result->y = a->y + r*(b->y - a->y);
         }
+#ifdef USE_POINT_Z_M
         result->m = 0;
+#endif
     }
 
     return result;
@@ -859,11 +939,15 @@ pointObj *msGetMeasureUsingPoint(shapeObj *shape, pointObj *point)
                 {
                     oFirst.x = line.point[j].x;
                     oFirst.y = line.point[j].y;
+#ifdef USE_POINT_Z_M
                     oFirst.m = line.point[j].m;
+#endif
                     
                     oSecond.x =  line.point[j+1].x;
                     oSecond.y =  line.point[j+1].y;
+#ifdef USE_POINT_Z_M
                     oSecond.m =  line.point[j+1].m;
+#endif
 
                     dfMinDist = dfDist;
                 }
@@ -887,7 +971,9 @@ pointObj *msGetMeasureUsingPoint(shapeObj *shape, pointObj *point)
 
             dfFactor = dfDistToIntersection / dfDistTotal;
 
+#ifdef USE_POINT_Z_M
             poIntersectionPt->m = oFirst.m + (oSecond.m - oFirst.m)*dfFactor;
+#endif
 
             return poIntersectionPt;
         }
@@ -960,29 +1046,69 @@ char **msGetAllGroupNames(mapObj *map, int *numTok)
     return papszGroups;
 }
 
+/************************************************************************/
+/*                         msForceTmpFileBase()                         */
+/************************************************************************/
+
+static int tmpCount = -1;
+static char *ForcedTmpBase = NULL;
+
+void msForceTmpFileBase( const char *new_base )
+{
+/* -------------------------------------------------------------------- */
+/*      Clear previous setting, if any.                                 */
+/* -------------------------------------------------------------------- */
+    if( ForcedTmpBase != NULL )
+    {
+        free( ForcedTmpBase );
+        ForcedTmpBase = NULL;
+    }
+    
+    tmpCount = -1;
+
+    if( new_base == NULL )
+        return;
+
+/* -------------------------------------------------------------------- */
+/*      Record new base.                                                */
+/* -------------------------------------------------------------------- */
+    ForcedTmpBase = strdup( new_base );
+    tmpCount = 0;
+}
+
 /**********************************************************************
  *                          msTmpFile()
  *
- * Generate a Unique temporary filename using PID + timestamp + extension
- * char* returned must be freed by caller
+ * Generate a Unique temporary filename using:
+ * 
+ *    PID + timestamp + sequence# + extension
+ * 
+ * If msForceTmpFileBase() has been call to control the temporary filename
+ * then the filename will be:
+ *
+ *    TmpBase + sequence# + extension
+ * 
+ * Returns char* which must be freed by caller.
  **********************************************************************/
 char *msTmpFile(const char *mappath, const char *tmppath, const char *ext)
 {
     char *tmpFname;
     char szPath[MS_MAXPATHLEN];
     const char *fullFname;
-    static char tmpId[128]; /* big enough for time + pid + ext */
-    static int tmpCount = -1;
+    char tmpId[128]; /* big enough for time + pid + ext */
 
-    if (tmpCount == -1)
+    if( ForcedTmpBase != NULL )
+    {
+        strncpy( tmpId, ForcedTmpBase, sizeof(tmpId) );
+    }
+    else 
     {
         /* We'll use tmpId and tmpCount to generate unique filenames */
         sprintf(tmpId, "%ld%d",(long)time(NULL),(int)getpid());
-        tmpCount = 0;
     }
 
     if (ext == NULL)  ext = "";
-    tmpFname = (char*)malloc(strlen(tmpId) + 4  + strlen(ext) + 1);
+    tmpFname = (char*)malloc(strlen(tmpId) + 10  + strlen(ext) + 1);
 
     sprintf(tmpFname, "%s%d.%s", tmpId, tmpCount++, ext);
 
@@ -1126,16 +1252,49 @@ void  msTransformShape(shapeObj *shape, rectObj extent, double cellsize,
         return;
     }
 #endif
+    if (image != NULL && MS_RENDERER_SVG(image->format) )
+    {
+        
+        msTransformShapeSVG(shape, extent, cellsize, image);
+
+        return;
+    }
 
     msTransformShapeToPixel(shape, extent, cellsize);
 }
- 
+
+/*
+-------------------------------------------------------------------------------
+ msSetup()
+
+ Contributed by Jerry Pisk in bug 1203.  Heads off potential race condition
+ in initializing GD font cache with multiple threads.  Should be called from
+ mapscript module initialization code.
+-------------------------------------------------------------------------------
+*/
+
+int msSetup()
+{
+#ifdef USE_THREAD
+    msThreadInit();
+#endif
+
+#ifdef USE_GD_FT
+    if (gdFontCacheSetup() != 0) 
+    {
+        return MS_FAILURE;
+    }
+#endif
+    return MS_SUCCESS;
+}
+  
 /* This is intended to be a function to cleanup anything that "hangs around"
    when all maps are destroyed, like Registered GDAL drivers, and so forth. */
-extern void lexer_cleanup();
+extern void lexer_cleanup(void);
 
 void msCleanup()
 {
+    msForceTmpFileBase( NULL );
     msConnPoolFinalCleanup();
     lexer_cleanup();
 #ifdef USE_OGR
