@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapwmslayer.c,v 1.73 2004/11/18 21:18:36 assefa Exp $
+ * $Id: mapwmslayer.c,v 1.76 2005/05/13 17:23:34 dan Exp $
  *
  * Project:  MapServer
  * Purpose:  Implementation of WMS CONNECTIONTYPE - client to WMS servers
@@ -27,6 +27,16 @@
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  * $Log: mapwmslayer.c,v $
+ * Revision 1.76  2005/05/13 17:23:34  dan
+ * First pass at properly handling XML exceptions from CONNECTIONTYPE WMS
+ * layers. Still needs some work. (bug 1246)
+ *
+ * Revision 1.75  2005/02/18 03:06:48  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.74  2005/02/14 04:33:32  sdlime
+ * Changed %.17g to %.15g for WMS/WFS server code.
+ *
  * Revision 1.73  2004/11/18 21:18:36  assefa
  * Make sure that msDrawWMSLayerLow calls msDrawLayer instead of msDrawRasterLayerLow
  * directly ensuring that some logic (transparency) that are in msDrawLayer are
@@ -374,21 +384,21 @@ static int msBuildWMSLayerURLBase(mapObj *map, layerObj *lp,
 
     if (pszStyle==NULL)
     {
-        // When no style is selected, use "" which is a valid default.
+        /* When no style is selected, use "" which is a valid default. */
         pszStyle = "";
     }
     else
     {
-        // Was a wms_style_..._sld URL provided?
+        /* Was a wms_style_..._sld URL provided? */
         char szBuf[100];
         sprintf(szBuf, "style_%.80s_sld", pszStyle);
         pszSLD = msOWSLookupMetadata(&(lp->metadata), "MO", szBuf);
 
         if (pszSLD)
         {
-            // SLD URL is set.  If this defn. came from a map context then
-            // the style name may just be an internal name: "Style{%d}" if
-            // that's the case then we should not pass this name via the URL
+            /* SLD URL is set.  If this defn. came from a map context then */
+            /* the style name may just be an internal name: "Style{%d}" if */
+            /* that's the case then we should not pass this name via the URL */
             if (strncmp(pszStyle, "Style{", 6) == 0)
                 pszStyle = "";
         }
@@ -396,18 +406,18 @@ static int msBuildWMSLayerURLBase(mapObj *map, layerObj *lp,
 
     if (pszSLD == NULL)
     {
-        // STYLES is mandatory if SLD not set
+        /* STYLES is mandatory if SLD not set */
         msSetWMSParamString(psWMSParams, "STYLES", pszStyle, MS_TRUE);
     }
     else if (strlen(pszStyle) > 0)
     {
-        // Both STYLES and SLD are set
+        /* Both STYLES and SLD are set */
         msSetWMSParamString(psWMSParams, "STYLES", pszStyle, MS_TRUE);
         msSetWMSParamString(psWMSParams, "SLD",    pszSLD,   MS_TRUE);
     }
     else
     {
-        // Only SLD is set
+        /* Only SLD is set */
         msSetWMSParamString(psWMSParams, "SLD",    pszSLD,   MS_TRUE);
     }
 
@@ -494,11 +504,11 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
          (pszVersion = strstr(lp->connection, "WMTVER=")) == NULL &&
          (pszVersion = strstr(lp->connection, "wmtver=")) == NULL ) )
     {
-        // CONNECTION missing or seems incomplete... try to build from metadata
+        /* CONNECTION missing or seems incomplete... try to build from metadata */
         if (msBuildWMSLayerURLBase(map, lp, psWMSParams) != MS_SUCCESS)
-            return MS_FAILURE;  // An error already produced.
+            return MS_FAILURE;  /* An error already produced. */
 
-        // If we received MS_SUCCESS then version must have been set
+        /* If we received MS_SUCCESS then version must have been set */
         pszVersion = msLookupHashTable(psWMSParams->params, "VERSION");
         if (pszVersion ==NULL)
             pszVersion = msLookupHashTable(psWMSParams->params, "WMTVER");
@@ -506,7 +516,7 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
     }
     else
     {
-        // CONNECTION string seems complete, start with that.
+        /* CONNECTION string seems complete, start with that. */
         psWMSParams->onlineresource = strdup(lp->connection);
         pszVersion = strchr(pszVersion, '=')+1;
     }
@@ -521,10 +531,10 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
       case OWS_1_0_7:
       case OWS_1_1_0:
       case OWS_1_1_1:
-        // All is good, this is a supported version.
+        /* All is good, this is a supported version. */
         break;
       default:
-        // Not a supported version
+        /* Not a supported version */
         msSetError(MS_WMSCONNERR, "MapServer supports only WMS 1.0.0 to 1.1.1 (please verify the VERSION parameter in the connection string).", "msBuildWMSLayerURL()");
         return MS_FAILURE;
     }
@@ -560,8 +570,8 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
         int nLen;
         char *pszPtr = NULL;
 
-        // If it's an AUTO projection then keep only id and strip off 
-        // the parameters for now (we'll restore them at the end)
+        /* If it's an AUTO projection then keep only id and strip off  */
+        /* the parameters for now (we'll restore them at the end) */
         if (strncasecmp(pszEPSG, "AUTO:", 5) == 0)
         {
             if ((pszPtr = strchr(pszEPSG, ',')))
@@ -577,12 +587,12 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
             (pszFound = strstr(pszLyrEPSG, pszEPSG)) == NULL ||
             ! ((*(pszFound+nLen) == '\0') || isspace(*(pszFound+nLen))) )
         {
-            // Not found in Layer's list of SRS (including projection object)
+            /* Not found in Layer's list of SRS (including projection object) */
             free(pszEPSG);
             pszEPSG = NULL;
         }
         if (pszEPSG && pszPtr)
-            *pszPtr = ',';  // Restore full AUTO:... definition
+            *pszPtr = ',';  /* Restore full AUTO:... definition */
     }
 
     if (pszEPSG == NULL &&
@@ -606,7 +616,7 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
         pointObj oPoint;
         char *pszNewEPSG;
 
-        // Use center of map view for lon0,lat0
+        /* Use center of map view for lon0,lat0 */
         oPoint.x = (map->extent.minx + map->extent.maxx)/2.0;
         oPoint.y = (map->extent.miny + map->extent.maxy)/2.0;
         msProjectPoint(&(map->projection), &(map->latlon), &oPoint);
@@ -623,7 +633,7 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
 /* ------------------------------------------------------------------
  * Set layer SRS and reproject map extents to the layer's SRS
  * ------------------------------------------------------------------ */
-    // No need to set lp->proj if it's already set to the right EPSG code
+    /* No need to set lp->proj if it's already set to the right EPSG code */
     if ((pszTmp = msOWSGetEPSGProj(&(lp->projection), NULL, "MO", MS_TRUE)) == NULL ||
         strcasecmp(pszEPSG, pszTmp) != 0)
     {
@@ -694,7 +704,7 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
         msSetWMSParamInt(   psWMSParams, "HEIGHT",  map->height);
         msSetWMSParamString(psWMSParams, "SRS",     pszEPSG, MS_FALSE);
 
-        snprintf(szBuf, 100, "%.17g,%.17g,%.17g,%.17g", 
+        snprintf(szBuf, 100, "%.15g,%.15g,%.15g,%.15g", 
                  bbox.minx, bbox.miny, bbox.maxx, bbox.maxy);
         msSetWMSParamString(psWMSParams, "BBOX",    szBuf, MS_TRUE);
  
@@ -704,9 +714,9 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
         msSetWMSParamString(psWMSParams, "EXCEPTIONS", pszExceptionsParam, MS_FALSE);
         msSetWMSParamString(psWMSParams, "INFO_FORMAT",pszInfoFormat, MS_TRUE);
 
-        // If FEATURE_COUNT <= 0 then don't pass this parameter
-        // The spec states that FEATURE_COUNT must be greater than zero
-        // and if not passed then the behavior is up to the server
+        /* If FEATURE_COUNT <= 0 then don't pass this parameter */
+        /* The spec states that FEATURE_COUNT must be greater than zero */
+        /* and if not passed then the behavior is up to the server */
         if (nFeatureCount > 0)
         {
             msSetWMSParamInt(psWMSParams, "FEATURE_COUNT", nFeatureCount);
@@ -722,17 +732,22 @@ int msBuildWMSLayerURL(mapObj *map, layerObj *lp, int nRequestType,
         else
             pszRequestParam = "map";
 
-        if (nVersion >= OWS_1_1_0)
-            pszExceptionsParam = "application/vnd.ogc.se_inimage";
-        else
-            pszExceptionsParam = "INIMAGE";
+        pszExceptionsParam = msOWSLookupMetadata(&(lp->metadata), 
+                                                 "MO", "exceptions_format");
+        if (pszExceptionsParam == NULL)
+        {
+            if (nVersion >= OWS_1_1_0)
+                pszExceptionsParam = "application/vnd.ogc.se_inimage";
+            else
+                pszExceptionsParam = "INIMAGE";
+        }
 
         msSetWMSParamString(psWMSParams, "REQUEST", pszRequestParam, MS_FALSE);
         msSetWMSParamInt(   psWMSParams, "WIDTH",   map->width);
         msSetWMSParamInt(   psWMSParams, "HEIGHT",  map->height);
         msSetWMSParamString(psWMSParams, "SRS",     pszEPSG, MS_FALSE);
 
-        snprintf(szBuf, 100, "%.17g,%.17g,%.17g,%.17g", 
+        snprintf(szBuf, 100, "%.15g,%.15g,%.15g,%.15g", 
                  bbox.minx, bbox.miny, bbox.maxx, bbox.maxy);
         msSetWMSParamString(psWMSParams, "BBOX",    szBuf, MS_TRUE);
         msSetWMSParamString(psWMSParams, "EXCEPTIONS",  pszExceptionsParam, MS_FALSE);
@@ -846,16 +861,16 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
 
         msFreeCharArray(tokens, n);
 
-        // Reproject latlonboundingbox to the selected SRS for the layer and
-        // check if it overlaps the bbox that we calculated for the request
+        /* Reproject latlonboundingbox to the selected SRS for the layer and */
+        /* check if it overlaps the bbox that we calculated for the request */
 
         msProjectRect(&(map->latlon), &(lp->projection), &ext);
         if (!msRectOverlap(&bbox, &ext))
         {
-            // No overlap... nothing to do
+            /* No overlap... nothing to do */
 
             msFreeWmsParamsObj(&sThisWMSParams);
-            return MS_SUCCESS;  // No overlap.
+            return MS_SUCCESS;  /* No overlap. */
         }
     }
 
@@ -864,7 +879,7 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
  * the case we will use it, else we use the default which is 30 seconds.
  * First check the metadata in the layer object and then in the map object.
  * ------------------------------------------------------------------ */
-    nTimeout = 30;  // Default is 30 seconds 
+    nTimeout = 30;  /* Default is 30 seconds  */
     if ((pszTmp = msOWSLookupMetadata(&(lp->metadata), 
                                       "MO", "connectiontimeout")) != NULL)
     {
@@ -901,7 +916,7 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
         key = msFirstKeyFromHashTable(sThisWMSParams.params);
         while (key != NULL && bOkToMerge == MS_TRUE)
         {
-            // Skip parameters whose values can be different
+            /* Skip parameters whose values can be different */
             if (!(strcmp(key, "LAYERS") == 0 ||
                   strcmp(key, "QUERY_LAYERS") == 0 ||
                   strcmp(key, "STYLES") == 0) )
@@ -983,7 +998,7 @@ int msPrepareWMSLayerRequest(int nLayerId, mapObj *map, layerObj *lp,
         pasReqInfo[(*numRequests)].nLayerId = nLayerId;
         pasReqInfo[(*numRequests)].pszGetUrl = pszURL;
         pszURL = NULL;
-        // We'll store the remote server's response to a tmp file.
+        /* We'll store the remote server's response to a tmp file. */
         if (map->web.imagepath == NULL || strlen(map->web.imagepath) == 0)
         {
             msSetError(MS_WMSERR, 
@@ -1084,19 +1099,71 @@ int msDrawWMSLayerLow(int nLayerId, httpRequestObj *pasReqInfo,
         return MS_SUCCESS;
     }
 
+/* ------------------------------------------------------------------
+ * Check the content-type of the response to see if we got an exception,
+ * if yes then try to parse it and pass the info to msSetError().
+ * We log an error but we still return SUCCESS here so that the layer 
+ * is only skipped intead of aborting the whole draw map.
+ * ------------------------------------------------------------------ */
+    if (pasReqInfo[iReq].pszContentType &&
+        (strcmp(pasReqInfo[iReq].pszContentType, "text/xml") == 0 ||
+         strcmp(pasReqInfo[iReq].pszContentType, "application/vnd.ogc.se_xml") == 0))
+    {
+        FILE *fp;
+        char szBuf[MS_BUFFER_LENGTH];
+
+        fp = fopen(pasReqInfo[iReq].pszOutputFile, "r");
+        if (fp)
+        {
+            /* TODO: For now we'll only read the first chunk and return it
+             * via msSetError()... we should really try to parse the XML
+             * and extract the exception code/message though
+             */
+             size_t nSize;
+
+            nSize = fread(szBuf, sizeof(char), MS_BUFFER_LENGTH-1, fp);
+            if (nSize >= 0 && nSize < MS_BUFFER_LENGTH)
+                szBuf[nSize] = '\0';
+            else
+                strcpy(szBuf, "(!!!)"); /* This should never happen */
+
+            fclose(fp);
+
+            /* We're done with the remote server's response... delete it. */
+            if (!lp->debug)
+                unlink(pasReqInfo[iReq].pszOutputFile);
+        }
+        else
+        {
+            strcpy(szBuf, "(Failed to open exception response)");
+        }
+
+        if (lp->debug)
+            msDebug("WMS GetMap request got XML exception for layer '%s': %s.",
+                    (lp->name?lp->name:"(null)"), szBuf );
+
+        msSetError(MS_WMSERR, 
+                   "WMS GetMap request got XML exception for layer '%s': %s.",
+                   "msDrawWMSLayerLow()", 
+                   (lp->name?lp->name:"(null)"), szBuf );
+
+        return MS_SUCCESS;
+    }
+        
+
 /* ------------------------------------------------------------------
  * Prepare layer for drawing, reprojecting the image received from the
  * server if needed...
  * ------------------------------------------------------------------ */
-    //keep the current type that will be restored at the end of this 
-    //function.
+    /* keep the current type that will be restored at the end of this  */
+    /* function. */
     currenttype = lp->type;
     currentconnectiontype = lp->connectiontype;
     lp->type = MS_LAYER_RASTER;
     lp->connectiontype = MS_SHAPEFILE;
 
-    //set the classes to 0 so that It won't do client side
-    //classification if an sld was set.
+    /* set the classes to 0 so that It won't do client side */
+    /* classification if an sld was set. */
     numclasses = lp->numclasses;
     if (msOWSLookupMetadata(&(lp->metadata), "MO", "sld_body") ||
         msOWSLookupMetadata(&(lp->metadata), "MO", "sld_url"))        
@@ -1107,10 +1174,10 @@ int msDrawWMSLayerLow(int nLayerId, httpRequestObj *pasReqInfo,
 
     if (!msProjectionsDiffer(&(map->projection), &(lp->projection)))
     {
-        // The simple case... no reprojection needed... render layer directly.
+        /* The simple case... no reprojection needed... render layer directly. */
         lp->transform = MS_FALSE;
-        //if (msDrawRasterLayerLow(map, lp, img) != 0)
-        //   status = MS_FAILURE;
+        /* if (msDrawRasterLayerLow(map, lp, img) != 0) */
+        /* status = MS_FAILURE; */
          if (msDrawLayer(map, lp, img) != 0)
             status = MS_FAILURE;
     }
@@ -1118,11 +1185,11 @@ int msDrawWMSLayerLow(int nLayerId, httpRequestObj *pasReqInfo,
     {
         FILE *fp;
         char *wldfile;
-        // OK, we have to resample the raster to map projection...
+        /* OK, we have to resample the raster to map projection... */
         lp->transform = MS_TRUE;
 
-        // Create a world file with raster extents
-        // One line per value, in this order: cx, 0, 0, cy, ulx, uly
+        /* Create a world file with raster extents */
+        /* One line per value, in this order: cx, 0, 0, cy, ulx, uly */
         wldfile = msBuildPath(szPath, lp->map->mappath, lp->data);
         if (wldfile)    
             strcpy(wldfile+strlen(wldfile)-3, "wld");
@@ -1145,7 +1212,7 @@ int msDrawWMSLayerLow(int nLayerId, httpRequestObj *pasReqInfo,
                     pasReqInfo[iReq].bbox.maxy + dfCellSizeY * 0.5 );
             fclose(fp);
 
-            // GDAL should be called to reproject automatically.
+            /* GDAL should be called to reproject automatically. */
             if (msDrawRasterLayerLow(map, lp, img) != 0)
                 status = MS_FAILURE;
 
@@ -1162,15 +1229,15 @@ int msDrawWMSLayerLow(int nLayerId, httpRequestObj *pasReqInfo,
 
     } 
 
-    // We're done with the remote server's response... delete it.
+    /* We're done with the remote server's response... delete it. */
     if (!lp->debug)
       unlink(lp->data);
 
-    //restore prveious type
+    /* restore prveious type */
     lp->type = currenttype;
     lp->connectiontype = currentconnectiontype;
 
-    //restore previous numclasses
+    /* restore previous numclasses */
     lp->numclasses = numclasses;
 
     free(lp->data);
