@@ -7,7 +7,7 @@
  * Derived from shapelib, and relicensed with permission.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,18 @@
  ******************************************************************************
  *
  * $Log: mapxbase.c,v $
+ * Revision 1.22  2005/06/14 16:03:35  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.21  2005/04/22 14:11:30  frank
+ * Bug 791: initialize some fields in msDBFCreate()
+ *
+ * Revision 1.20  2005/02/18 03:06:48  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.19  2005/02/07 18:37:43  frank
+ * Removed dangerous junk in leading space trimmer.
+ *
  * Revision 1.18  2004/10/21 04:30:54  frank
  * Added standardized headers.  Added MS_CVSID().
  *
@@ -38,7 +50,7 @@
 #include <stdlib.h> /* for atof() and atoi() */
 #include <math.h>
 
-MS_CVSID("$Id: mapxbase.c,v 1.18 2004/10/21 04:30:54 frank Exp $")
+MS_CVSID("$Id: mapxbase.c,v 1.22 2005/06/14 16:03:35 dan Exp $")
 
 /************************************************************************/
 /*                             SfRealloc()                              */
@@ -350,7 +362,11 @@ DBFHandle msDBFCreate( const char * pszFilename )
     psDBF->bCurrentRecordModified = MS_FALSE;
     psDBF->pszCurrentRecord = NULL;
 
+    psDBF->pszStringField = NULL;
+    psDBF->nStringFieldLen = 0;    
+
     psDBF->bNoHeader = MS_TRUE;
+    psDBF->bUpdated = MS_FALSE;
 
     return( psDBF );
 }
@@ -518,13 +534,13 @@ static char *msDBFReadAttribute(DBFHandle psDBF, int hEntity, int iField )
       }
     }
 
-    if(i == -1) psDBF->pszStringField[0] = '\0'; // whole string is blank (SDL fix)      
+    if(i == -1) psDBF->pszStringField[0] = '\0'; /* whole string is blank (SDL fix)       */
 
     /*
     ** Trim/skip leading blanks (SDL/DM Modification - only on numeric types)
     */ 
     if( psDBF->pachFieldType[iField] == 'N' || psDBF->pachFieldType[iField] == 'F' || psDBF->pachFieldType[iField] == 'D' ) {
-        for(i=0; i < psDBF->pszStringField[i] != '\0' ;i++) {
+        for(i=0; psDBF->pszStringField[i] != '\0' ;i++) {
             if(psDBF->pszStringField[i] != ' ')
                 break;	
         }
@@ -844,7 +860,7 @@ int *msDBFGetItemIndexes(DBFHandle dbffile, char **items, int numitems)
     itemindexes[i] = msDBFGetItemIndex(dbffile, items[i]);
     if(itemindexes[i] == -1) { 
       free(itemindexes);
-      return(NULL); // item not found
+      return(NULL); /* item not found */
     }
   }
 
