@@ -5,7 +5,7 @@
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,12 @@
  ******************************************************************************
  *
  * $Log: shp2img.c,v $
+ * Revision 1.17  2005/06/14 16:03:35  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.16  2005/02/18 03:06:48  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
  * Revision 1.15  2004/11/05 02:28:39  frank
  * Reorganize error cases a bit to cleanup properly so we can test error
  * logic leak issues, such as in bug 139.
@@ -44,7 +50,7 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: shp2img.c,v 1.15 2004/11/05 02:28:39 frank Exp $")
+MS_CVSID("$Id: shp2img.c,v 1.17 2005/06/14 16:03:35 dan Exp $")
 
 int main(int argc, char *argv[])
 {
@@ -56,7 +62,7 @@ int main(int argc, char *argv[])
   char **layers=NULL;
   int num_layers=0;
 
-  char *outfile=NULL; // no -o sends image to STDOUT
+  char *outfile=NULL; /* no -o sends image to STDOUT */
 
   if(argc > 1 && strcmp(argv[1], "-v") == 0) {
     printf("%s\n", msGetVersion());
